@@ -1,4 +1,4 @@
-; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -20,9 +20,9 @@
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
-; Department of Computer Sciences
+; Department of Computer Science
 ; University of Texas at Austin
-; Austin, TX 78712-1188 U.S.A.
+; Austin, TX 78701 U.S.A.
 
 (in-package "ACL2")
 
@@ -219,16 +219,14 @@
 ; of functions that return multiple values.  Presumably this is the correct
 ; list for the main Lisp package in other Common Lisp implementations as well.
 
-#||
- (let (ans)
-    (do-symbols (sym (find-package "LISP"))
-                (let ((tp (get sym 'compiler::return-type)))
-                  (when (and (consp tp)
-                             (member (car tp)
-                                     '(values system::returns-exactly)))
-                    (setq ans (cons sym ans)))))
-    ans)
-||#
+;  (let (ans)
+;     (do-symbols (sym (find-package "LISP"))
+;                 (let ((tp (get sym 'compiler::return-type)))
+;                   (when (and (consp tp)
+;                              (member (car tp)
+;                                      '(values system::returns-exactly)))
+;                     (setq ans (cons sym ans)))))
+;     ans)
 
   '(INTERN DECODE-FLOAT
            GET-MACRO-CHARACTER ; ansi only
@@ -594,7 +592,7 @@
 ; The loss of efficiency caused by using symbol-value below should be more than
 ; compensated for by the lack of a warning here when building the system.
 
-           (template (qfuncall defstobj-template name args))
+           (template (qfuncall defstobj-template name args nil))
            (raw-defs (qfuncall defstobj-raw-defs name template
 
 ; We do not want to rely on having the world available here, so we pass in nil
@@ -671,28 +669,26 @@
 ; Version_2.8, and we proclaim there since Warren Hunt thought that might be
 ; useful.
 
-#|
-Here is a summary of three comparable user times from certifying all the ACL2
-books in June 2000, just before Release 2.5 is complete.  The first column,
-labeled "Comp", is the one to be looked at for comparison purposes.  These are
-all done on the same Sun workstation, using Allegro 5.0.1.  The meanings of
-these numbers are explained below.
-
-                               Comp     Actual   Comments
-Recent ACL2 without proclaim:  5:41     5:36:42  no meta
-Recent ACL2 *with*  proclaim:  5:54     5:53:58
-April ACL2 (before non-std.):  5:48     5:35:58  missing some pipeline and ~40
-                                                 sec. user time on powerlists
-
-The "Comp" column estimates how long the run would have taken if all books had
-certified, except that no run gets past book batcher-sort in the powerlists/
-directory.  (The April run bogs down even slightly earlier.)  The first row is
-adjusted by about 4 minutes because the run started with book meta-plus-lessp.
-The April run broke on book basic-def from case-studies/pipeline and hence
-missed the rest of that directory's books.  The above points account for the
-addition of time from "Actual" to the appropriate comparison time in the first
-column.
-|#
+; Here is a summary of three comparable user times from certifying all the ACL2
+; books in June 2000, just before Release 2.5 is complete.  The first column,
+; labeled "Comp", is the one to be looked at for comparison purposes.  These are
+; all done on the same Sun workstation, using Allegro 5.0.1.  The meanings of
+; these numbers are explained below.
+; 
+;                                Comp     Actual   Comments
+; Recent ACL2 without proclaim:  5:41     5:36:42  no meta
+; Recent ACL2 *with*  proclaim:  5:54     5:53:58
+; April ACL2 (before non-std.):  5:48     5:35:58  missing some pipeline and ~40
+;                                                  sec. user time on powerlists
+; 
+; The "Comp" column estimates how long the run would have taken if all books had
+; certified, except that no run gets past book batcher-sort in the powerlists/
+; directory.  (The April run bogs down even slightly earlier.)  The first row is
+; adjusted by about 4 minutes because the run started with book meta-plus-lessp.
+; The April run broke on book basic-def from case-studies/pipeline and hence
+; missed the rest of that directory's books.  The above points account for the
+; addition of time from "Actual" to the appropriate comparison time in the first
+; column.
 
   (when *do-proclaims*
     (with-open-file
@@ -1169,9 +1165,10 @@ notation causes an error and (b) the use of ,. is not permitted."
 ; WARNING: Keep this in sync with the #-acl2-loop-only definition of setenv$.
 
   #+cmu
-  (and (boundp ext::*environment-list*)
-       (cdr (assoc-eq (intern string :keyword)
-                      ext::*environment-list*)))
+  (and (boundp 'ext::*environment-list*)
+       (cdr (assoc (intern string :keyword)
+                   ext::*environment-list*
+                   :test #'eq)))
   #+(or gcl allegro lispworks ccl sbcl clisp)
   (let ((fn
          #+gcl       'si::getenv
@@ -1301,19 +1298,42 @@ notation causes an error and (b) the use of ,. is not permitted."
            (t
             (concatenate 'string (our-pwd) name))))))
 
+(defun our-user-homedir-pathname ()
+
+; For ACL2 Version_4.2, Hanbing Liu reported the following error using Allegro
+; CL Enterprise Edition 8.0 for Linux, apparently printed before printing the
+; optimization settings.
+
+;     An unhandled error occurred during initialization:
+;     There is no file named /home/rcash/
+
+; The error was likely caused by our calling user-homedir-pathname without a
+; host, for a user without a home directory.  Quoting the Common Lisp Hyperspec
+; on (user-homedir-pathname &optional host):
+
+;   If it is impossible to determine the user's home directory on host, then
+;   nil is returned. user-homedir-pathname never returns nil if host is not
+;   supplied.
+
+; It's not clear that there is a meaningful notion of host on Linux.  So we
+; play it safe and define our own, error-free (for CLtL2) version of
+; user-homedir-pathname.  We have observed in the past that we needed our own
+; version of user-homedir-pathname for GCL 2.7.0 anyhow (see below).
+
+  #+gcl
+  (cond
+   ((gcl-version->= 2 7 0)
+
 ; It seems that GCL 2.7.0 has had problems with user-homedir-pathname in static
 ; versions because of how some system functions are relocated.  So we define
 ; our own version for use by all GCLs 2.7.0 and beyond.
 
-#+gcl
-(defun our-user-homedir-pathname (&optional host)
-  (declare (ignore host))
-  (pathname (concatenate 'string (si::getenv "HOME") "/")))
-
-#+gcl
-(when (gcl-version->= 2 7 0)
-  (setf (symbol-function 'user-homedir-pathname)
-        (symbol-function 'our-user-homedir-pathname)))
+    (let ((home (si::getenv "HOME")))
+      (and home
+           (pathname (concatenate 'string home "/")))))
+   (t (pathname (user-homedir-pathname))))
+  #-gcl ; presumably CLtL2
+  (ignore-errors (user-homedir-pathname)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            MISCELLANY
