@@ -1,4 +1,4 @@
-; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -20,21 +20,13 @@
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
-; Department of Computer Sciences
+; Department of Computer Science
 ; University of Texas at Austin
-; Austin, TX 78712-1188 U.S.A.
-
-; This file, acl2-init.lisp, is the inititialization file for ACL2.
-
-; ACL2 is designed to run in any Common Lisp, although we have focused
-; especially on AKCL, and we take advantage of a few aspects of AKCL
-; to assist our development.  This file need not be distributed with
-; ACL2 and is unimportant for the correct operation of ACL2.  This
-; file is loaded automatically by ACKL when it starts up.
+; Austin, TX 78701 U.S.A.
 
 ; This file cannot be compiled because it changes packages in the middle.
 
-; Next we restrict the hons version to CCL (starting with v3-6), because of
+; We restrict the hons version to CCL (starting with v3-6), because of
 ; CCL-specific changes made to hons-raw.lisp.  With some work this restriction
 ; could probably be relaxed, but it's not clear that it's worth the effort.
 #+(and hons (not ccl))
@@ -46,17 +38,56 @@ formerly OpenMCL).  CCL runs on many platforms, including
 both 32-bit and 64-bit Linux.  See the ACL2 installation
 instructions for how to obtain CCL.")
 
-; Allow taking advantage of threads in SBCL and CCL.
-#+(or (and sbcl sb-thread) ccl)
+; Allow taking advantage of threads in SBCL, CCL, and Lispworks (where we may
+; want to build a parallel version, which needs this to take place).
+#+(or (and sbcl sb-thread) ccl lispworks)
 (push :acl2-mv-as-values *features*)
 
+; Essay on Parallelism and Parallelism Warts
+
+; These sources incorporate code for an experimental extension for parallelism,
+; initiated in David Rager's dissertation work.  That extension may be produced
+; by setting the :acl2-par feature, for example using make (see :DOC
+; compiling-acl2p).  This work is evolving, but we have taken great pains to
+; preserve the functionality of ACL2 without parallelism.  As the parallelism
+; extension is still under active development as of this writing (May, 2011),
+; we use the phrase "Parallelism wart:" to annotate comments about known issues
+; for the #+acl2-par build of ACL2.  These range from minor comment suggestions
+; to deeper questions concerning the correctness of the code.  Rager intends to
+; address most of these parallelism warts by the time his dissertation work is
+; complete.
+
+; In an effort to avoid code duplication, we created a definition scheme that
+; supports defining both serial and parallel versions of a function with one
+; call to a defun-like macro.  See the definitions of @par-mappings and
+; defun@par for an explanation of this scheme.
+
+; Developer note on emacs and parallelism.  When comparing with versions up
+; through svn revision 335 (May 27, 2011), it may be useful to ignore "@par"
+; when ignoring whitespace with meta-x compare-windows in emacs.
+;   (setq compare-windows-whitespace "\\(@par\\)?[ \t\n]+")
+; To revert to the default behavior:
+;   (setq compare-windows-whitespace
+;   "\\(\\s-\\|
+;   \\)+")
+; We indent certain calls as we indent calls of defun.  (These forms are in
+; emacs/emacs-acl2.el.)
+;   (put 'er@par       'lisp-indent-function 'defun)
+;   (put 'warning$@par 'lisp-indent-function 'defun)
+; To revert:
+;   (put 'er@par       'lisp-indent-function nil)
+;   (put 'warning$@par 'lisp-indent-function nil)
+
 ; Only allow the feature :acl2-par in environments that support
-; multi-threading.  Keep this in sync with the error message about SBCL and
-; CCL in set-parallel-evaluation and with :doc parallelism-build.
-#+(and acl2-par (not ccl) (not (and sbcl sb-thread)))
+; multi-threading.  Keep this in sync with the error message about CCL,
+; Lispworks, and SBCL in set-parallel-evaluation-fn and with :doc
+; compiling-acl2p.  If we add support for non-ANSI GCL, consider providing a
+; call of reset-parallelism-variables analogous to the one generated by setting
+; *reset-parallelism-variables* in our-abort.
+#+(and acl2-par (not ccl) (not (and sbcl sb-thread)) (not lispworks))
 (error "It is currently illegal to build the parallel version of ACL2 in this
 Common Lisp.  See source file acl2-init.lisp for this error message,
-which is preceded information (Lisp features) indicating the legal Lisp
+which is preceded by information (Lisp features) indicating the legal Lisp
 implementations.")
 
 #+akcl
@@ -71,13 +102,58 @@ implementations.")
 #+clisp
 (setq CUSTOM:*DEFAULT-FILE-ENCODING* :unix)
 
-; #+lispworks ; 3.2.0
-;(lw::extend-current-stack 1000)
+#+lispworks
+(setq system::*stack-overflow-behaviour* nil) ; could be :warn
+
+; We have observed a significant speedup with Allegro CL when turning off
+; its cross-referencing capability.  Here are the times before and after
+; evaluating the setq form below, in an example from Dave Greve that spends
+; a lot of time loading compiled files.
+;
+; 165.43 seconds realtime, 163.84 seconds runtime.
+; 120.23 seconds realtime, 118.32 seconds runtime.
+;
+; The user is welcome to edit the form below.  Note that it doesn't seem to
+; affect the profiler.
+#+allegro
+(setq excl::*record-xref-info* nil
+      excl::*load-xref-info* nil
+      excl::*record-source-file-info* nil
+      excl::*load-source-file-info* nil)
 
 ; Create the packages we use.
 
 (load "acl2.lisp")
 
+; Fix a bug in SBCL 1.0.49 (https://bugs.launchpad.net/bugs/795705), thanks to
+; patch provided by Nikodemus Siivola.
+#+sbcl
+(in-package :sb-c)
+#+sbcl
+(when (equal (lisp-implementation-version) "1.0.49")
+  (without-package-locks
+
+   (defun undefine-fun-name (name)
+     (when name
+       (macrolet ((frob (type &optional val)
+                        `(unless (eq (info :function ,type name) ,val)
+                           (setf (info :function ,type name) ,val))))
+                 (frob :info)
+                 (frob :type (specifier-type 'function))
+                 (frob :where-from :assumed)
+                 (frob :inlinep)
+                 (frob :kind)
+                 (frob :macro-function)
+                 (frob :inline-expansion-designator)
+                 (frob :source-transform)
+                 (frob :structure-accessor)
+                 (frob :assumed-type)))
+     (values))
+   ))
+
+; WARNING: The next form should be an in-package (see in-package form for sbcl
+; just above).
+
 ;  Now over to the "ACL2" package for the rest of this file.
 
 (in-package "ACL2")
@@ -413,22 +489,22 @@ implementations.")
                                                    result)))
                                      (reverse result)))))
 
-; We want to be sure to include hons-raw.lisp and parallel-raw.lisp even if we
-; are not building the hons version, in order to assist in maintaining both
-; versions.
+; We want to be sure to include the *-raw.lisp files even if we are not
+; building the hons version, in order to assist in maintaining both versions.
 
                  (append lst (list "hons-raw.lisp"
                                    "memoize-raw.lisp"
-				   "multi-threading-raw.lisp"
+                                   "multi-threading-raw.lisp"
+                                   "futures-raw.lisp"
                                    "parallel-raw.lisp")))))
 
 (defvar *saved-build-date-lst*)
 (defvar *saved-mode*)
 
 (defconstant *acl2-svn-revision-string*
-   "~% WARNING: Do not redistribute.  This is NOT an ACL2 release; it is,~
-    ~% rather, an svn distribution, ~a.~
-    ~% The authors of ACL2 consider svn distributions to be experimental.~%")
+  "~% WARNING: Do not redistribute.  This is NOT an ACL2 release; it is,~
+   ~% rather, an svn distribution, ~a.~
+   ~% The authors of ACL2 consider svn distributions to be experimental.~%")
 
 (defun acl2-svn-revision-string ()
 
@@ -470,27 +546,15 @@ implementations.")
    #+acl2-par
    "~%~% Experimental modification for parallel evaluation.  Please expect at~
     ~% most limited maintenance for this version~%"
-   #+lispworks
-   "~% Type (LP) to enter the ACL2 command loop;~
-    ~% then see the documentation topic ~a for recent changes.~%"
-   #-lispworks
-   "~% See the documentation topic ~a for recent changes.~
-    ~% Note: We have modified the prompt in some underlying Lisps to further~
+   "~% See the documentation topic ~a for recent changes."
+   "~% Note: We have modified the prompt in some underlying Lisps to further~
     ~% distinguish it from the ACL2 prompt.~%"))
 
 (defun maybe-load-acl2-init ()
-
-; There is not a true notion of home directory for Windows systems, as far as
-; we know.  We may provide one at a future point, but for now, we simply act as
-; though ~/acl2-init.lsp does not exist on such systems.
-
-
-  #+mswindows
-  nil
-  #-mswindows
-  (let ((fl (probe-file (merge-pathnames (user-homedir-pathname)
-                                         "acl2-init.lsp"))))
-    (if fl (load fl))))
+  (let* ((home (our-user-homedir-pathname))
+         (fl (and home
+                  (probe-file (merge-pathnames home "acl2-init.lsp")))))
+    (when fl (load fl))))
 
 (defun chmod-executable (sysout-name)
   (system-call "chmod" (list "+x" sysout-name)))
@@ -844,6 +908,32 @@ implementations.")
   (maybe-load-acl2-init)
   (eval `(in-package ,*startup-package-name*))
 
+; Remark for #+acl2-par.  Here (for 64-bit Lispworks) and in parallel-raw.lisp
+; (for CCL and SBCL), we set the gc-threshold to a high number.  If the Lisps
+; support it, this threshold could be based off the actual memory in the
+; system, but we just leave it at 2 gigabytes for now.  We peform this setting
+; of the threshold for Lispworks inside the restart function, because Lispworks
+; doesn't save the GC configuration as part of the Lisp image.
+
+; Parallelism wart: the 1 gigabyte threshold may cause problems for machines
+; with less than 1 gigabytes of free RAM.
+
+  #+(and acl2-par (and lispworks lispworks-64bit))
+  (progn
+
+; Calling set-gen-num-gc-threshold sets the GC threshold for the given
+; generation of garbage.
+
+    (system:set-gen-num-gc-threshold 0 (expt 2 20))
+    (system:set-gen-num-gc-threshold 1 (expt 2 27))
+    (system:set-gen-num-gc-threshold 2 (expt 2 28))
+
+; This call to set-blocking-gen-num accomplishes two things: (1) It sets the
+; third generation as the "final" generation -- nothing can be promoted to
+; generation four or higher.  (2) It sets the GC threshold for generation 3.
+
+    (system:set-blocking-gen-num 3 :gc-threshold (expt 2 30)))
+   
 ; The following two lines follow the recommendation in Allegro CL's
 ; documentation file doc/delivery.htm.
 
@@ -851,17 +941,10 @@ implementations.")
   #+allegro (rplacd (assoc 'tpl::*saved-package*
                            tpl:*default-lisp-listener-bindings*)
                     'common-lisp:*package*)
-
   #+allegro (lp)
+  #+lispworks (lp)
   #+ccl (eval '(lp)) ; using eval to avoid compiler warning
 
-; See the comment in save-acl2-in-lispworks for why we need the following call.
-
-  #+lispworks (mp:initialize-multiprocessing)
-
-  ;;Lispworks 4.2.0 no longer recognizes the following:
-  ;;#+lispworks (lw::extend-current-stack 1000)
-
   (setq *acl2-default-restart-complete* t)
   nil)
 
@@ -884,50 +967,125 @@ implementations.")
                   :full-gc t))
 
 #+lispworks
-(defun save-acl2-in-lispworks (sysout-name &optional mode)
+(defun lispworks-save-exec-aux (sysout-name eventual-sysout-name)
+
+; LispWorks support (Dave Fox) pointed out, in the days of LispWorks 4, that we
+; need to be sure to call (mp:initialize-multiprocessing) when starting up.  Up
+; through ACL2 Version_4.2 we did that by making that call in
+; acl2-default-restart.  But when testing with LispWorks 6.0, we noticed that
+; some processes hang, and we wondered if that has to do with the fact that
+; (mp:initialize-multiprocessing) does not return.  That also got in the way of
+; our running (LP) in acl2-default-restart.  We experimented with removing
+; (mp:initialize-multiprocessing) from acl2-default-restart, instead passing
+; :multiprocessing t to system::save-image.  But with that change, we noticed
+; that upon exiting the ACL2 loop with :q, we got the following rather scary
+; message.
+
+; ;; No live processes except internal servers - stopping multiprocessing
+
+; So we have decided not to call :multiprocessing t, and also not to call
+; (mp:initialize-multiprocessing).  LispWorks 6.0 seems to work fine for ACL2,
+; so it seems that we need not think further about
+; mp:initialize-multiprocessing until perhaps LispWorks is supported with
+; #+acl2-par.
+
+; Parallelism wart: it's unclear whether calling stop-multiprocessing really
+; works.  We are currently seeing an error in ACL2(p) on Lispworks, where
+; cons's are reported as misaligned when including books and also when
+; executing proofs in parallel.  This error didn't occur initially, and it
+; started surfacing about the time that we started calling
+; stop-multiprocessing.
+
+  #+acl2-par
+  (when mp::*multiprocessing*
+    (send-die-to-all-except-initial-threads)
+    (mp::stop-multiprocessing)
+    (gc$))
+  #+acl2-par
+  (when *lp-ever-entered-p* ; don't print during compliation
+    (format t
+            "If you wish to continue using this image, you will need to call ~%~
+             'mp:initialize-multiprocessing' instead of calling 'lp'.  This ~%~
+             is necessary because of the way multiprocessing works in ~%~
+             Lispworks.~%~%"))
+
+; We just make a guess that Lispworks can be handled the way that GCL is
+; handled.
+
+  (if (probe-file "worklispext")
+      (delete-file "worklispext"))
+  (let* ((ext "lw")
+         (ext+
+
+; We deal with the apparent fact that Windows implementations of GCL append
+; ".exe" to the filename created by save-system -- and assume, until someone
+; tells us otherwise, that Lispworks does similarly.
+
+          #+mswindows "lw.exe"
+          #-mswindows "lw")
+         (lw-exec-file
+          (unix-full-pathname sysout-name ext+))
+         (eventual-lw-exec-file
+          (unix-full-pathname eventual-sysout-name ext+)))
+    (with-open-file (str "worklispext" :direction :output)
+                    (format str ext+))
+    (if (probe-file sysout-name)
+        (delete-file sysout-name))
+    (if (probe-file lw-exec-file)
+        (delete-file lw-exec-file))
+    (with-open-file (str sysout-name :direction :output)
+                    (write-exec-file str nil
+
+; We pass options "-init -" and "-siteinit -" to inhibit loading init and patch
+; files because because we assume that whatever such files were to be loaded,
+; were in fact loaded at the time the original Lispworks executable was saved.
+; Of course, individual users who doesn't like this decision and know better
+; could always edit this script file, i.e., lw-exec-file, in the same spirit as
+; changing the underlying Lisp implementation before building ACL2 (again,
+; presumably based on knowledge of the host Lisp implementation).
+
+                                     "~s -init - -siteinit - $*~%"
+                                     eventual-lw-exec-file))
+    (chmod-executable sysout-name)
+    (cond ((and system::*init-file-loaded*
+                system::*complain-about-init-file-loaded*)
+
+; We hope it's fine to save an image when an init-file has been loaded.  Maybe
+; somebody can explain to us why LispWorks causes a break in such a situation
+; by default (which explains the binding of
+; system::*complain-about-init-file-loaded* below).
+
+           (format t
+                   "Warning: Overriding LispWorks hesitation to save an image~%~
+                  after init-file has been loaded.~%")
+           (let ((system::*complain-about-init-file-loaded* nil))
+             (system::save-image lw-exec-file
+                                 :restart-function 'acl2-default-restart
+                                 #+acl2-par :multiprocessing
+                                 #+acl2-par t
+                                 :gc t)))
+          (t (system::save-image lw-exec-file
+                                 :restart-function 'acl2-default-restart
+                                 #+acl2-par :multiprocessing
+                                 #+acl2-par t
+                                 :gc t)))))
+
+#+lispworks
+(defun save-acl2-in-lispworks (sysout-name mode eventual-sysout-name)
   (setq *saved-mode* mode)
-
-; Increase the stack size.  Without doing this, Version_2.7 (as it existed
-; shortly before release) had a stack overflow involving collect-assumptions in
-; the proof of bitn-lam0 from books/rtl/rel2/support/lop3.lisp.  As Lispworks
-; support (Dave Fox) has pointed out, we need to be sure to call
-; (mp:initialize-multiprocessing) when starting up.  See LP for that call.
-
-; But even the following wasn't adequate for (verify-guards read-utf8-fast ...)
-; in books/unicode/read-utf8.lisp:
-; (setq sys:*sg-default-size* 128000)
-; So we try the following.
-
-  (setq sys:*sg-default-size* 1024000)
-
-  (system::save-image sysout-name :restart-function 'acl2-default-restart
-                      :gc t))
-
-; The definition of save-exec-raw for lispworks did not work (Lispworks 4.4.5,
-; and probably Lispworks 4.2).  An attempt resulted in the following error:
-
-;   ACL2 5 > (save-exec "my-lw" "Changes made")
-
-;   Error: You cannot save an image while multiprocessing or the environment is running.
-;   Place the call to HARLEQUIN-COMMON-LISP:SAVE-IMAGE in a file and use the -init command line option.
-
-; If it becomes important to save an ACL2 image for lispworks from an ACL2
-; session, we can figure this out at that time (and perhaps get some help from
-; the user, who may be knowledgeable about lispworks).
-
-; #+lispworks
-; (defun save-exec-raw (sysout-name)
-;   (setq *acl2-default-restart-complete* nil)
-;   (system::save-image sysout-name :restart-function 'acl2-default-restart
-;                       :gc t))
+  (if (probe-file "worklispext")
+      (delete-file "worklispext"))
+  (with-open-file (str "worklispext" :direction :output)
+                  (format str "lw"))
+  (lispworks-save-exec-aux sysout-name eventual-sysout-name))
 
 #+lispworks
 (defun save-exec-raw (sysout-name)
-  (declare (ignore sysout-name))
-  (error "Save-exec-raw is not implemented in lispworks.  Feel free to contact~%~
-          the implementors, and feel free to read the comment above the~%~
-          definition  of save-exec-raw for #+lispworks in ACL2 source file~%~
-          acl2-init.lisp."))
+
+; See the comment above about :multiprocessing t.
+
+  (setq *acl2-default-restart-complete* nil)
+  (lispworks-save-exec-aux sysout-name sysout-name))
 
 #+cmu
 (defun save-acl2-in-cmulisp-aux (sysout-name core-name)
@@ -1010,12 +1168,23 @@ implementations.")
 
         ("~a~%"
          (let ((contrib-dir
-                (and (fboundp 'sb-int::sbcl-homedir-pathname)
-                     (funcall 'sb-int::sbcl-homedir-pathname))))
+                (and (boundp 'sb-ext::*core-pathname*)
+                     (ignore-errors
+                      (let* ((core-dir
+                              (pathname-directory sb-ext::*core-pathname*))
+                             (contrib-dir-pathname
+                              (and (equal (car (last core-dir))
+                                          "output")
+                                   (make-pathname
+                                    :directory
+                                    (append (butlast core-dir 1)
+                                            (list "contrib"))))))
+                        (and (probe-file contrib-dir-pathname)
+                             (namestring contrib-dir-pathname)))))))
            (if contrib-dir
                (format nil
                        "export SBCL_HOME=~s"
-                       (namestring contrib-dir))
+                       contrib-dir)
              "")))
         "~s --core ~s~a --eval '(acl2::sbcl-restart)'"
         prog
@@ -1100,7 +1269,8 @@ implementations.")
 ; go with format arg (rc-filename save-dir), because we know of no way in
 ; Allegro 6.2 to avoid getting Allegro copyright information printed upon :q if
 ; we start up in the ACL2 read-eval-print loop.
-          #|               "~s -I ~s -L ~s~%" |#
+
+;         "~s -I ~s -L ~s~%"
 
           "~s -I ~s $*~%"
           (system::command-line-argument 0)
@@ -1337,7 +1507,7 @@ implementations.")
   #+lucid
   (save-acl2-in-lucid "nsaved_acl2" mode)
   #+lispworks
-  (save-acl2-in-lispworks "nsaved_acl2" mode)
+  (save-acl2-in-lispworks "nsaved_acl2" mode other-info)
   #+allegro
   (save-acl2-in-allegro "nsaved_acl2" mode other-info)
   #+cmu
