@@ -1,4 +1,4 @@
-; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -21,9 +21,9 @@
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
-; Department of Computer Sciences
+; Department of Computer Science
 ; University of Texas at Austin
-; Austin, TX 78712-1188 U.S.A.
+; Austin, TX 78701 U.S.A.
 
 ; Acknowledgments:  Bob Boyer contributed crucially to the design and
 ; implementation of early versions of ACL2.  Many others, largely at CLI, have
@@ -171,12 +171,12 @@
 
                      (space #+cmu 1 #-cmu 0)
 
-; WARNING:  Do not proclaim (cl-user::fixnum-safety 0) for Lispworks.  Any
+; WARNING:  Do not proclaim (cl-user::fixnum-safety 0) for LispWorks.  Any
 ; fixnum-safety less than 3 expects all integers to be fixnums!
 
                      (safety
 
-; Consider using (safety 3) if there is a problem with Lispworks.  It enabled
+; Consider using (safety 3) if there is a problem with LispWorks.  It enabled
 ; us to see a stack overflow involving collect-assumptions in the proof of
 ; bitn-lam0 from books/rtl/rel2/support/lop3.lisp.  (See save-acl2-in-lispworks
 ; for how we have eliminated that stack overflow.)
@@ -186,33 +186,33 @@
 ; decided to keep the safety at 0 by default and why safety 3 is not too bad.
 ;
 ; Safety 0:
-; 12955.537u 542.877s 3:46:24.68 99.3%	0+0k 0+0io 0pf+0w
+; 12955.537u 542.877s 3:46:24.68 99.3%  0+0k 0+0io 0pf+0w
 ;
 ; Safety 1:
-; 15343.578u 562.207s 4:27:03.67 99.2%	0+0k 0+0io 0pf+0w
+; 15343.578u 562.207s 4:27:03.67 99.2%  0+0k 0+0io 0pf+0w
 
 ; Try safety 2 or 3 to find violations with Allegro CL like the car of a
 ; non-nil atom.  (Note: safety 3 has failed in GCL due to a stack overflow.)
 ; Here are some numbers with Allegro CL, 8 processors with make -j 8:
 
 ; Safety 0:
-; 11262.699u 291.730s 38:23.72 501.5%	0+0k 0+0io 16pf+0w
+; 11262.699u 291.730s 38:23.72 501.5%   0+0k 0+0io 16pf+0w
 
 ; Safety 2:
-; 15588.206u 285.249s 54:19.72 486.9%	0+0k 0+0io 0pf+0w
+; 15588.206u 285.249s 54:19.72 486.9%   0+0k 0+0io 0pf+0w
 
 ; Safety 3:
-; 16450.508u 284.473s 57:46.03 482.8%	0+0k 0+0io 0pf+0w
+; 16450.508u 284.473s 57:46.03 482.8%   0+0k 0+0io 0pf+0w
 
 ; Here are GCL numbers, again with make -j 8 (and using "fast builds").
 
 ; Safety 0:
-; 10013.573u 566.983s 33:33.80 525.4%	0+0k 0+0io 0pf+0w
+; 10013.573u 566.983s 33:33.80 525.4%   0+0k 0+0io 0pf+0w
 
 ; Safety 2:
 ; [Note: books/clause-processors/SULFA/books/sat-tests/test-incremental.lisp
 ; ran out of space, saving perhaps a minute]
-; 15637.669u 511.811s 52:02.78 517.1%	0+0k 0+0io 0pf+0w
+; 15637.669u 511.811s 52:02.78 517.1%   0+0k 0+0io 0pf+0w
 
                       ,(let ((our-safety
                               #-CLTL2
@@ -267,7 +267,7 @@
 (error
  "FATAL ERROR: ACL2 does not yet support GCL ANSI.  Please use a non-ANSI GCL.")
 
-; To use ACL2 under Lispworks 3.2.0, execute the following to work around a
+; To use ACL2 under LispWorks 3.2.0, execute the following to work around a
 ; bug.
 
 ; #+lispworks
@@ -409,6 +409,12 @@
   (or (find-package inv)
       (make-package inv :use nil)))
 
+; LispWorks has a package named "DEF", and that name conflicts with an ACL2
+; package of that name introduced in books/coi/.  So we rename it here.
+#+lispworks
+(when (find-package "DEF")
+  (rename-package "DEF" "DEF-FROM-LW-RENAMED"))
+
 ; The value of the constant *the-live-state* is actually just a
 ; symbol, but that symbol is the unique representative of the one
 ; single active, global, real-time state of ACL2, which is represented
@@ -455,14 +461,45 @@
 
 (defvar acl2::*compiling-certified-file* nil)
 
+(defun acl2::defconst-redeclare-error (name)
+  (let ((stk (symbol-value 'acl2::*load-compiled-stack*)))
+    (cond (stk
+           (error
+            "Illegal attempt to redeclare the constant ~s.~%~
+             The problem appears to be that you are including a book,~%~
+             ~2T~a,~%~
+             that attempts to give a definition of this constant that~%~
+             is incompatible with its existing definition.  The ~%~
+             discrepancy is being discovered while loading that book's~%~
+             compiled (or expansion) file~:[, as the last such load for~%~
+             the following nested sequence of included books (outermost~%~
+             to innermost):~%~{  ~a~%~}~;.~]"
+            name
+            (caar stk)
+            (null (cdr stk))
+            (reverse (loop for x in stk collect (car x)))))
+          (t
+           (error "Illegal attempt to redeclare the constant ~s."
+                  name)))))
+
+(defparameter acl2::*safe-mode-verified-p*
+
+; This global may be bound to t when we are evaluating a form that we know will
+; not lead to an ill-guarded call in raw Lisp (say, because the form was
+; previously evaluated by ACL2 in safe-mode).  See also the comment in
+; ec-call1-raw.
+
+  nil)
+
 (defmacro acl2::defconst (name term &rest rst)
   (declare (ignore rst)) ; e.g., documentation
   (let ((disc (gensym)))
     `(defparameter ,name
-       (cond
-        ((boundp ',name)
+       (let ((acl2::*safe-mode-verified-p* t))
          (cond
-          (acl2::*compiling-certified-file*
+          ((boundp ',name)
+           (cond
+            (acl2::*compiling-certified-file*
 
 ; We avoid the potentially expensive redundancy check done below, which is
 ; legitimate since we are simply loading a compiled file at the end of
@@ -480,34 +517,33 @@
 ;    `(defconst ,name ',(tree n)))
 ; (deftree *big* 35)
 
-           (symbol-value ',name))
-          (t
+             (symbol-value ',name))
+            (t
 
 ; Even though ',name is bound, we refer to its value with
 ; (symbol-value ',name) rather than simply an in-line ,name, to avoid
 ; compiler warnings.
 
-           (let ((,disc (get ',name 'acl2::redundant-raw-lisp-discriminator)))
-             (cond
-              ((and (consp ,disc)
-                    (eq (car ,disc) 'acl2::defconst))
-               (assert (consp (cdr ,disc)))
+             (let ((,disc (get ',name
+                               'acl2::redundant-raw-lisp-discriminator)))
                (cond
-                ((and (eq (cdr (cdr ,disc)) (symbol-value ',name))
-                      (or (equal (car (cdr ,disc)) ',term)
-                          (equal (cdr (cdr ,disc)) ,term)))
-                 (symbol-value ',name))
-                (t (error "Illegal attempt to redeclare the constant ~s."
-                          ',name))))
-              ((acl2::raw-mode-p acl2::*the-live-state*)
+                ((and (consp ,disc)
+                      (eq (car ,disc) 'acl2::defconst))
+                 (assert (consp (cdr ,disc)))
+                 (cond
+                  ((and (eq (cdr (cdr ,disc)) (symbol-value ',name))
+                        (or (equal (car (cdr ,disc)) ',term)
+                            (equal (cdr (cdr ,disc)) ,term)))
+                   (symbol-value ',name))
+                  (t (acl2::defconst-redeclare-error ',name))))
+                ((acl2::raw-mode-p acl2::*the-live-state*)
 
 ; In this case we allow redeclaration of the constant; this is, after all, raw
 ; mode, where there are no guarantees.
 
-               ,term)
-              (t 
-               (error "Illegal attempt to redeclare the constant ~s."
-                      ',name)))))))
+                 ,term)
+                (t 
+                 (acl2::defconst-redeclare-error ',name)))))))
 
 ; If ',name is not bound, we must evaluate ,term.  Note that we do so
 ; outside of all local bindings, so as not to disturb variables in
@@ -515,11 +551,11 @@
 ; but you never know!)  We may want to enforce that this code is only executed
 ; during the boot-strap; see the Essay on Guard Checking.
 
-        (t (let* ((val ,term)
-                  (d (list* 'acl2::defconst ',term val)))
-             (setf (get ',name 'acl2::redundant-raw-lisp-discriminator)
-                   d)
-             (cdr (cdr d))))))))
+          (t (let* ((val ,term)
+                    (d (list* 'acl2::defconst ',term val)))
+               (setf (get ',name 'acl2::redundant-raw-lisp-discriminator)
+                     d)
+               (cdr (cdr d)))))))))
 
 ; We now get our imports for package ACL2, putting them into the
 ; variable acl2::*common-lisp-symbols-from-main-lisp-package*.
@@ -553,6 +589,7 @@
     "boot-strap-pass-2"
     "basis"
     "parallel" ; but only get special under-the-hood treatment with #+acl2-par
+    #+acl2-par "futures-raw"
     #+acl2-par "parallel-raw"
     #+hons "memoize-raw"
     #+hons "hons-raw"
@@ -634,7 +671,7 @@ ACL2 from scratch.")
    (setq acl2::*copy-of-acl2-version*
 ;  Keep this in sync with the value of acl2-version in *initial-global-table*.
          (concatenate 'string
-                      "ACL2 Version 4.2"
+                      "ACL2 Version 4.3"
                       #+non-standard-analysis
                       "(r)"
                       #+(and mcl (not ccl))
@@ -676,6 +713,21 @@ ACL2 from scratch.")
                                  #-(or ccl sbcl)
                                  t))
                           *the-live-state*)
+            (f-put-global
+             'host-lisp
+             ,(let () ; such empty binding has avoided errors in  GCL 2.6.7
+                #+gcl :gcl
+                #+ccl :ccl
+                #+sbcl :sbcl
+                #+allegro :allegro
+                #+clisp :clisp
+                #+cmu :cmu
+                #+lispworks :lispworks
+                #-(or gcl ccl sbcl allegro clisp cmu lispworks)
+                (error
+                 "The underlying host Lisp appears not to support ACL2. ~%~
+                  Contact the ACL2 implementors to request such support."))
+             *the-live-state*)
             #+unix
             (f-put-global 'tmp-dir "/tmp" *the-live-state*)
             #-acl2-mv-as-values
@@ -793,57 +845,115 @@ ACL2 from scratch.")
 
 (defmacro with-warnings-suppressed (&rest forms)
 
-; Keep this in sync with the handler-bind call in init.lisp.  Thanks to Juho
-; Snellman for the SBCL form.
+; See also the handler-bind call in init.lisp.  Thanks to Juho Snellman for his
+; assistance with handler-bind.
 
 ; We are happy to turn off redefinition warnings because we trust that
 ; functions such as add-trip know what they are doing.  Without this code, for
 ; example, :oops can cause many screens of such warnings.
 
   #+sbcl
+
+; Warning: We turn off all warnings in SBCL because they are so noisy.  We
+; eliminate most of them in the proclaim form earlier in this file, using
+; (sb-ext:inhibit-warnings 3), because without that the following doesn't work;
+; just start up sbcl and submit the following form without the leading
+; backquote and replacing ,@forms by (defun f (x y) x).
+
+; `(handler-bind
+;   ((warning       (lambda (c)
+;                     (declare (ignore c))
+;                     (invoke-restart 'muffle-warning)))
+;    (style-warning (lambda (c)
+;                     (declare (ignore c))
+;                     (invoke-restart 'muffle-warning))))
+;   ,@forms)
+
+; However, even with the above proclaim form we still get redefinition
+; warnings.  So we wrap the following handler-bind around forms in order to
+; eliminate redefinition warnings as well.
+
   `(handler-bind
+    ((warning (lambda (c)
+                (declare (ignore c))
+                (invoke-restart 'muffle-warning))))
+    ,@forms)
 
-; Warning: We turn off all warnings in SBCL because they are so noisy.  Here is
-; the story:
+  #+cmu
+  `(progn (setq ext:*gc-verbose* nil)
+          (handler-bind
+           ((warning (lambda (c)
+                       (declare (ignore c))
+                       (invoke-restart 'muffle-warning))))
+           ,@forms))
 
-; In the case of SBCL, we have noticed that the inhibition of style-warnings
-; eliminates a huge amount of noise during the build, probably because
-; with-warnings-suppressed is called in compile-acl2.  Unfortunately, some of
-; that "noise" is the reporting of unused formals.  So, developers may want to
-; compile in some Lisp other than SBCL in order to see warnings about unused
-; formals and perhaps other warnings as well.
+  #+lispworks
+  `(let ((compiler::*redefinition-action* :QUIET))
+     ,@forms)
 
-; Considerable noise can be eliminated even if sb-kernel:redefinition-warning
-; replaces style-warning below (at least, we tried this in SBCL 1.0.19;
-; sb-kernel:redefinition-warning doesn't work in SBCL 1.0.3).  But there is
-; still a lot of noise left when substituting sb-kernel:redefinition-warning
-; for style-warning, e.g. on constant names, ignorable variable CONDITION-P in
-; memoize, and undefined function warnings for *1* functions defined in
-; mutual-recursions.  So we leave style-warning in place.
+  #+allegro
+  `(let ((excl:*redefinition-warnings* nil)) ,@forms)
 
-    (#+ansi-cl
-     (style-warning (lambda (c)
+  #+clisp
+  `(let ((custom::*suppress-check-redefinition* t))
+
+; Unfortunately, the above binding seems to be ignored when in the scope of
+; with-compilation-unity, so we get redefinition warnings from clisp during the
+; build.  Here is an example that exhibits that behavior CLISP 2.44.1 (even
+; without ACL2); and the same problem occurs if we switch the order of
+; with-compilation-unit and the binding of *suppress-check-redefinition*.
+
+;   (common-lisp::with-compilation-unit
+;    ()
+;    (let ((custom::*suppress-check-redefinition* t))
+;      (dolist (name '("foo" "bar"))
+;        (let ((source (make-pathname :name name :type "lisp")))
+;          (load source)
+;          (progn
+;            (compile-file source)
+;            (load (make-pathname :name name :type "fas")))))))
+
+     ,@forms)
+
+  #+ccl
+  `(let ((ccl::*compiler-warn-on-duplicate-definitions* nil)) ,@forms)
+
+  #-(or cmucl sbcl lispworks allegro clisp ccl)
+  (if (cdr forms) `(progn ,@forms) (car forms)))
+
+(defmacro with-more-warnings-suppressed (&rest forms)
+
+; We add additional warning suppression beyond what is provided by
+; with-warnings-suppressed.  Warning: We do not necessarily suppress all
+; warnings that are suppressed by with-warnings-suppressed, so you will
+; probably want to call this macro in a context underneath
+; with-warnings-suppressed.
+
+; The handler-bind form given in with-warnings-suppressed for sbcl and cmucl is
+; sufficient; we do not need anything further here.  But even with the addition
+; of style-warnings (as commented there), that form doesn't seem to work for
+; CCL, Allegro CL, Lispworks, or CLISP.  So we bind some globals instead.
+
+  #+lispworks
+  `(let ((compiler::*compiler-warnings* nil))
+     ,@forms)
+
+  #+allegro
+  `(handler-bind
+    ((style-warning (lambda (c)
                       (declare (ignore c))
                       (invoke-restart 'muffle-warning))))
     ,@forms)
-  #+cmucl
-  `(progn (setq ext:*gc-verbose* nil) ,@forms)
-  #+lispworks
-  `(let ((compiler::*redefinition-action* :QUIET)) ,@forms)
-  #+allegro
-  `(let ((excl:*redefinition-warnings* nil)) ,@forms)
+
   #+clisp
+  `(let ((*compile-verbose* nil))
+     ,@forms)
 
-; Setting custom::*suppress-check-redefinition* seems harmless, but doesn't
-; help avoid warnings (as of CLISP 2.47) for more than one definition of the
-; same function in the same file.  We could bind *compile-verbose* to nil, but
-; that seems too extreme; although we do something analogous for SBCL, we felt
-; compelled there because of extreme noise, which isn't the case for CLISP.
-
-  `(let ((custom::*suppress-check-redefinition* t)) ,@forms)
   #+ccl
-  `(let ((ccl::*compiler-warn-on-duplicate-definitions* nil)) ,@forms)
-  #-(or cmucl sbcl lispworks allegro clisp ccl)
+  `(let ((ccl::*suppress-compiler-warnings* t))
+     ,@forms)
+
+  #-(or lispworks allegro clisp ccl)
   (if (cdr forms) `(progn ,@forms) (car forms)))
 
 (defmacro with-suppression (&rest forms)
@@ -937,13 +1047,15 @@ ACL2 from scratch.")
 #+allegro
 (setq compiler:tail-call-non-self-merge-switch t)
 
-; Lispworks Version 4.2.0 has issued the following complaint during compilation
+; LispWorks Version 4.2.0 has issued the following complaint during compilation
 ; until the next form was executed:
 ; **++++ Error in NTH-UPDATE-REWRITER1: 
 ;   Function size 73824 is too large.
 ; But even with the next form, we have seen the following:
 ; **++++ Error in XTRANS-EVAL: 
 ;   Function size 67910 is too large.
+; This problem has disappeared with LispWorks 6.0.  But it seems reasonable to
+; leave the following in place.
 #+lispworks
 (cl-user::toggle-source-debugging nil)
 
@@ -982,26 +1094,30 @@ ACL2 from scratch.")
 (our-with-compilation-unit
 
 ; At one time we avoided recompiling and had the following code inside a COND:
-#|
-   ((and (probe-file object-file)
-         (<= (file-write-date "acl2-fns.lisp")
-             (file-write-date object-file)))
-    (load object-file))
-|#
+
+;    ((and (probe-file object-file)
+;          (<= (file-write-date "acl2-fns.lisp")
+;              (file-write-date object-file)))
+;     (load object-file))
+
 ; But for example, if we compile with Allegro under SunOS and then later build
 ; the system with Allegro under Linux, the same ".fasl" extension could fool us
 ; into thinking that recompilation is not necessary.
 
  (progn
    (load "acl2-fns.lisp") ;we like to load before compiling
-   (when (not *suppress-compile-build-time*)
-     (compile-file "acl2-fns.lisp")
+   (let ((acl2-fns-compiled
+          (make-pathname :name "acl2-fns"
+                         :type *compiled-file-extension*)))
+     (when (probe-file acl2-fns-compiled)
+       (delete-file acl2-fns-compiled))
+     (when (not *suppress-compile-build-time*)
+       (compile-file "acl2-fns.lisp")
 
 ; Note that load-compiled is not used below, but on the other hand we are still
 ; using the original readtable here so that's not a problem.
 
-     (load (make-pathname :name "acl2-fns"
-                          :type *compiled-file-extension*)))))
+       (load acl2-fns-compiled)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                           ACL2-READTABLE
@@ -1141,123 +1257,121 @@ which is saved just in case it's needed later.")
   #-clisp
   (load filename))
 
-#|
-                      Remarks on *acl2-readtable*
+;                       Remarks on *acl2-readtable*
+; 
+; 
+; Because read-object$ calls the Common Lisp function read, read-object$
+; is a function of the values of the Common Lisp symbols (a)
+; *readtable*, (b) *package*, and (c) *features*.  In ACL2, the user can
+; specify the package to use, via in-package, which sets the global
+; current-package.  The user of ACL2 has no (legitimate) control over the
+; readtable, which is set above and discussed below.
+; 
+; As for *features*, we currently permit full use of the *features*
+; facility of the Common Lisp reader, with this caveat: it is an
+; official part of the ACL2 story that *features* should have the same
+; setting throughout all phases of ACL2 usage.  That is, the user must
+; set up *features* at the beginning, before starting to use ACL2 and
+; the user must then leave *features* alone (even though the
+; implementors of ACL2 put :acl2-loop-only onto *features* during
+; boot-strapping).  One bad consequence of our *features* policy is that
+; verified files will not in general be verifiable or useable in other
+; Lisp implementations or installations unless the settings of
+; *features* relevant to one's usages of the #+ and #- are the same in
+; the two Lisp implementations.  One simple way to obtain peace of mind
+; on this topic is for the user not to use #+ or #- at all!  It might be
+; cleaner for us simply to prohibit the use of the #+ and #- readmacros
+; all together in ACL2.  This could be done at the cost of replacing the
+; many uses of #+ and #- in axioms.lisp, and a few other places, with
+; some sort of regular macro.
+; 
+; Below is a detailed examination of the default Common Lisp readtable
+; from the perspective of ACL2.  We want to make sure that when we read,
+; we do not have side effects (e.g. via #.) of ACL2 but will merely
+; either (a) cause an error or (b) generate a Lisp object, which we then
+; will check with bad-lisp-object before handing it to ACL2 functions.
+; 
+; All of the standard Common Lisp characters are either white space or
+; constituent, with the following exceptions, which are read macros:
+; 
+;   "  quote
+;   #  number sign
+;   '  quote
+;   () parentheses
+;   ,  comma
+;   ;  comment
+;   \  single escape
+;   `  backquote
+;   |  multiple escape
+; 
+; With the exception of number sign, backquote and comma, we certainly
+; want all of these readmacros to have their usual Common Lisp
+; semantics.
+; 
+; We have defined our own backquote and discussed it above.
+; 
+; We now review the # situation:
+; 
+;   ## and #= are for reading possibly circular stuff
+;          bad-lisp-object may run forever
+;   #'  reads as function
+;          won't hurt anything
+;   #(  vector
+;          will be rejected by bad-lisp-object
+;   #)  signals an error
+;          enough said
+;   #*  bit vector
+;          will be rejected by bad-lisp-object
+;   #,  load-time evaluation
+;          we shut it off
+;   #0-#9 used for infix arguments
+;          ok
+;   #:  uninterned symbol
+;          will be rejected by bad-lisp-object
+;   #<  signals an error
+;          enough said
+;   #\  character object
+;          will be checked by bad-lisp-object; see also below
+;   #|  start comment, ended by |#
+;          ok
+;   #<backspace | tab | newline | linefeed | page | return | space>
+;       signals an error -- ok
+;   #+  and #-
+;       see the discussion of *features* above
+;   #.  read time evaluation
+;          we restrict it
+;   #A, #a  arrays
+;          will be checked by bad-lisp-object
+;   #B, #b  binary rational
+;          ok
+;   #C, #c complex
+;          ok (rejected by bad-lisp-object except for rational components)
+;   #O, #o octal
+;          ok
+;   #P, #p pathname
+;          will be checked by bad-lisp-object
+;   #R, #r radix-n
+;          fine
+;   #S, #s structure
+;          will be rejected by bad-lisp-object
+;   #X, #x hex
+;          ok
+; 
+; Eventually, it will be best to define a read function for ACL2 solely in terms
+; of ACL2 character reading primitives.  Common Lisp read is ambiguous.  There is
+; the ambiguity of backquote described above.  There is the abiguity of which
+; tokens get read as numbers.  To make matters a little more scary, there is
+; nothing that prevents a Common Lisp implementation from adding, for example, a
+; new # readmacro option that would provide something as potentially catastrophic
+; as full-blown sharp-dot.  One obstacle to doing a read within ACL2 this is
+; efficiency.  For example, ACL2 does not now support unread-char.  And given the
+; requirement that whatever is specified in a guard must be checkable, it is
+; unclear now how best to add unread-char since Common Lisp does permit one to
+; detect whether a stream is in a ``just unread'' state.  ACL2 could enable one
+; to answer such a question, but at the cost of having to store the information
+; every time that a character was unread.
 
 
-Because read-object$ calls the Common Lisp function read, read-object$
-is a function of the values of the Common Lisp symbols (a)
-*readtable*, (b) *package*, and (c) *features*.  In ACL2, the user can
-specify the package to use, via in-package, which sets the global
-current-package.  The user of ACL2 has no (legitimate) control over the
-readtable, which is set above and discussed below.
-
-As for *features*, we currently permit full use of the *features*
-facility of the Common Lisp reader, with this caveat: it is an
-official part of the ACL2 story that *features* should have the same
-setting throughout all phases of ACL2 usage.  That is, the user must
-set up *features* at the beginning, before starting to use ACL2 and
-the user must then leave *features* alone (even though the
-implementors of ACL2 put :acl2-loop-only onto *features* during
-boot-strapping).  One bad consequence of our *features* policy is that
-verified files will not in general be verifiable or useable in other
-Lisp implementations or installations unless the settings of
-*features* relevant to one's usages of the #+ and #- are the same in
-the two Lisp implementations.  One simple way to obtain peace of mind
-on this topic is for the user not to use #+ or #- at all!  It might be
-cleaner for us simply to prohibit the use of the #+ and #- readmacros
-all together in ACL2.  This could be done at the cost of replacing the
-many uses of #+ and #- in axioms.lisp, and a few other places, with
-some sort of regular macro.
-
-Below is a detailed examination of the default Common Lisp readtable
-from the perspective of ACL2.  We want to make sure that when we read,
-we do not have side effects (e.g. via #.) of ACL2 but will merely
-either (a) cause an error or (b) generate a Lisp object, which we then
-will check with bad-lisp-object before handing it to ACL2 functions.
-
-All of the standard Common Lisp characters are either white space or
-constituent, with the following exceptions, which are read macros:
-
-  "  quote
-  #  number sign
-  '  quote
-  () parentheses
-  ,  comma
-  ;  comment
-  \  single escape
-  `  backquote
-  |  multiple escape
-
-With the exception of number sign, backquote and comma, we certainly
-want all of these readmacros to have their usual Common Lisp
-semantics.
-
-We have defined our own backquote and discussed it above.
-
-We now review the # situation:
-
-  ## and #= are for reading possibly circular stuff
-         bad-lisp-object may run forever
-  #'  reads as function
-         won't hurt anything
-  #(  vector
-         will be rejected by bad-lisp-object
-  #)  signals an error
-         enough said
-  #*  bit vector
-         will be rejected by bad-lisp-object
-  #,  load-time evaluation
-         we shut it off
-  #0-#9 used for infix arguments
-         ok
-  #:  uninterned symbol
-         will be rejected by bad-lisp-object
-  #<  signals an error
-         enough said
-  #\  character object
-         will be checked by bad-lisp-object; see also below
-  #|  start comment, ended by |#
-         ok
-  #<backspace | tab | newline | linefeed | page | return | space>
-      signals an error -- ok
-  #+  and #-
-      see the discussion of *features* above
-  #.  read time evaluation
-         we restrict it
-  #A, #a  arrays
-         will be checked by bad-lisp-object
-  #B, #b  binary rational
-         ok
-  #C, #c complex
-         ok (rejected by bad-lisp-object except for rational components)
-  #O, #o octal
-         ok
-  #P, #p pathname
-         will be checked by bad-lisp-object
-  #R, #r radix-n
-         fine
-  #S, #s structure
-         will be rejected by bad-lisp-object
-  #X, #x hex
-         ok
-
-Eventually, it will be best to define a read function for ACL2 solely in terms
-of ACL2 character reading primitives.  Common Lisp read is ambiguous.  There is
-the ambiguity of backquote described above.  There is the abiguity of which
-tokens get read as numbers.  To make matters a little more scary, there is
-nothing that prevents a Common Lisp implementation from adding, for example, a
-new # readmacro option that would provide something as potentially catastrophic
-as full-blown sharp-dot.  One obstacle to doing a read within ACL2 this is
-efficiency.  For example, ACL2 does not now support unread-char.  And given the
-requirement that whatever is specified in a guard must be checkable, it is
-unclear now how best to add unread-char since Common Lisp does permit one to
-detect whether a stream is in a ``just unread'' state.  ACL2 could enable one
-to answer such a question, but at the cost of having to store the information
-every time that a character was unread.
-
-|#
-
 ;          ACL2's Implementation of the character reader
 
 ; We have decided to take full charge of everything that # reader
@@ -1331,51 +1445,49 @@ every time that a character was unread.
 ; In what follows we'll quote from dpANS (an emacs Info version).  We
 ; quote here from three sections, separated by ++++++++++++++++++.
 
-#|
+; 
+; Sharpsign Backslash
+; ...................
+; 
+; Syntax: #\<<x>>
+; 
+; [[[ text omitted ]]]
+; 
+; When the token x is more than one character long, the x must have the
+; syntax of a symbol with no embedded package markers.  In this case, the
+; sharpsign backslash notation parses as the character whose name is
+; (string-upcase x); see *See Character Names::.
+; 
+; ++++++++++++++++++
+; 
+; char-name                                                        [Function]
+; ---------------------------------------------------------------------------
+; 
+; `char-name'  character =>  name
+; 
+; Arguments and Values::
+; ......................
+; 
+; character--a character.
+; 
+; name--a string or nil.
+; 
+; Description::
+; .............
+; 
+; Returns a string that is the name of the character, or nil if the
+; character has no name.
+; 
+; ++++++++++++++++++
+; 
+; Character Names
+; ---------------
+; 
+; The following character names must be present in all conforming
+; implementations:
+; 
+; Newline
 
-Sharpsign Backslash
-...................
-
-Syntax: #\<<x>>
-
-[[[ text omitted ]]]
-
-When the token x is more than one character long, the x must have the
-syntax of a symbol with no embedded package markers.  In this case, the
-sharpsign backslash notation parses as the character whose name is
-(string-upcase x); see *See Character Names::.
-
-++++++++++++++++++
-
-char-name                                                        [Function]
----------------------------------------------------------------------------
-
-`char-name'  character =>  name
-
-Arguments and Values::
-......................
-
-character--a character.
-
-name--a string or nil.
-
-Description::
-.............
-
-Returns a string that is the name of the character, or nil if the
-character has no name.
-
-++++++++++++++++++
-
-Character Names
----------------
-
-The following character names must be present in all conforming
-implementations:
-
-Newline
-
-|# ; |
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                       COMPILING and LOADING, PART 2
@@ -1962,18 +2074,16 @@ Newline
 
 ; Bob Boyer uses the following at times.
 
-#||
-#+ccl
-(in-package "CCL")
-#+ccl
-(flet ((go-slow
-        ()
-        (set-current-compiler-policy (new-compiler-policy :inline-self-calls #'false))
-        (set-current-file-compiler-policy (new-compiler-policy :inline-self-calls #'false))
-        (proclaim '(optimize (debug 3) (compilation-speed 0)
-                             (safety 3) (speed 0) (space 0)))))
-  (go-slow))
-||#
+; #+ccl
+; (in-package "CCL")
+; #+ccl
+; (flet ((go-slow
+;         ()
+;         (set-current-compiler-policy (new-compiler-policy :inline-self-calls #'false))
+;         (set-current-file-compiler-policy (new-compiler-policy :inline-self-calls #'false))
+;         (proclaim '(optimize (debug 3) (compilation-speed 0)
+;                              (safety 3) (speed 0) (space 0)))))
+;   (go-slow))
 
 ; The following two assignments seemed to speed up regression runs by about
 ; 7.7% as opposed to only the second, which seemed to have little effect.
