@@ -1,3 +1,20 @@
+; Arithmetic-3 Library
+; Copyright (C) 2004 Robert Krug <rkrug@cs.utexas.edu>
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.
+;
+; This program is distributed in the hope that it will be useful but WITHOUT
+; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+; details.
+;
+; You should have received a copy of the GNU General Public License along with
+; this program; if not, write to the Free Software Foundation, Inc., 51
+; Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; collect.lisp
@@ -43,8 +60,7 @@
 
 (in-theory (disable collect-+-problem-finder))
 
-(theory-invariant (not (member-equal '(:rewrite collect-+-problem-finder)
-                                     theory))
+(theory-invariant (not (active-runep '(:rewrite collect-+-problem-finder)))
                   :error nil)
 
 (defthm |(+ x x)|
@@ -76,26 +92,18 @@
     (equal (collect-+ y x)
            (collect-+ x y)))
 
-(theory-invariant (or (not (member-equal '(:rewrite collect-+)
-                                         theory))
-                      (and (member-equal '(:rewrite |(collect-+ y x)|)
-                                         theory)
-                           (member-equal '(:rewrite |(+ x x)|)
-                                         theory)
-                           (member-equal '(:rewrite |(+ x (- x))|)
-                                         theory)
-                           (member-equal '(:rewrite |(+ x (* c x))|)
-                                         theory)
-                           (member-equal '(:rewrite |(+ (- x) (* c x))|)
-                                         theory)
-                           (member-equal '(:rewrite |(+ (* c x) (* d x))|)
-                                         theory)))
+(theory-invariant (or (not (active-runep '(:rewrite collect-+)))
+                      (and (active-runep '(:rewrite |(collect-+ y x)|))
+                           (active-runep '(:rewrite |(+ x x)|))
+                           (active-runep '(:rewrite |(+ x (- x))|))
+                           (active-runep '(:rewrite |(+ x (* c x))|))
+                           (active-runep '(:rewrite |(+ (- x) (* c x))|))
+                           (active-runep '(:rewrite |(+ (* c x) (* d x))|))))
                   :error nil)
 
 (in-theory (disable collect-+))
 
-(theory-invariant (not (member-equal '(:definition collect-+)
-                                     theory))
+(theory-invariant (not (active-runep '(:definition collect-+)))
                   :error nil)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -118,8 +126,7 @@
 
 (in-theory (disable collect-*-problem-finder))
 
-(theory-invariant (not (member-equal '(:rewrite collect-*-problem-finder)
-                                     theory))
+(theory-invariant (not (active-runep '(:rewrite collect-*-problem-finder)))
                   :error nil)
 
 (defthm |(* (expt x n) (expt y n))|
@@ -180,6 +187,21 @@
                         0
                       (expt x (- n 1))))))
 
+; (Matt K., Nov. 2006) The following new rule (after ACL 3.0.1) has slowed down
+; books/workshops/2004/legato/support/proof-by-generalization-mult.lisp from
+; 103.33s to 150.21s (with proof inhibited).  Although that seemed to be the
+; only noticeable slowdown it caused in the regression suite, we were tempted
+; to leave it disabled by default.  But instead we are taking Robert Krug's
+; advice to leave it enabled, since otherwise collect-* expressions won't be
+; reduced.
+
+(defthm |(* (/ x) (/ (expt x n)))|
+  (implies (integerp n)
+           (equal (collect-* (/ x) (/ (expt x n)))
+                  (if (equal (fix x) 0)
+                      0
+                    (/ (expt x (+ n 1)))))))
+
 (defthm |(* (/ x) (expt (- x) n))|
     (implies (integerp n)
              (equal (collect-* (/ x) (expt (- x) n))
@@ -309,52 +331,32 @@
     (equal (collect-* y x)
            (collect-* x y)))
 
-(theory-invariant (or (not (member-equal '(:rewrite collect-*)
-                                         theory))
-                      (and (member-equal '(:rewrite |(collect-* y x)|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (expt x n) (expt y n))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* x x)|)
-                                         theory)
-                           (member-equal '(:rewrite |(* x (/ x))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* x (expt x n))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* x (expt (- x) n))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* x (/ (expt x n)))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* x (/ (expt (- x) n)))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (/ x) (expt x n))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (/ x) (expt (- x) n))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (expt x m) (expt x n))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (expt (- x) m) (expt x n))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (expt x m) (expt (- x) n))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (/ (expt x m)) (expt x n))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (/ (expt (- x) m)) (expt x n))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (/ (expt x m)) (expt (- x) n))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (expt x m) (/ (expt x n)))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (expt (- x) m) (/ (expt x n)))|)
-                                         theory)
-                           (member-equal '(:rewrite |(* (expt x m) (/ (expt (- x) n)))|)
-                                         theory)))
-                  :error nil)
+(theory-invariant
+ (or (not (active-runep '(:rewrite collect-*)))
+     (and (active-runep '(:rewrite |(collect-* y x)|))
+          (active-runep '(:rewrite |(* (expt x n) (expt y n))|))
+          (active-runep '(:rewrite |(* x x)|))
+          (active-runep '(:rewrite |(* x (/ x))|))
+          (active-runep '(:rewrite |(* x (expt x n))|))
+          (active-runep '(:rewrite |(* x (expt (- x) n))|))
+          (active-runep '(:rewrite |(* x (/ (expt x n)))|))
+          (active-runep '(:rewrite |(* x (/ (expt (- x) n)))|))
+          (active-runep '(:rewrite |(* (/ x) (expt x n))|))
+          (active-runep '(:rewrite |(* (/ x) (expt (- x) n))|))
+          (active-runep '(:rewrite |(* (expt x m) (expt x n))|))
+          (active-runep '(:rewrite |(* (expt (- x) m) (expt x n))|))
+          (active-runep '(:rewrite |(* (expt x m) (expt (- x) n))|))
+          (active-runep '(:rewrite |(* (/ (expt x m)) (expt x n))|))
+          (active-runep '(:rewrite |(* (/ (expt (- x) m)) (expt x n))|))
+          (active-runep '(:rewrite |(* (/ (expt x m)) (expt (- x) n))|))
+          (active-runep '(:rewrite |(* (expt x m) (/ (expt x n)))|))
+          (active-runep '(:rewrite |(* (expt (- x) m) (/ (expt x n)))|))
+          (active-runep '(:rewrite |(* (expt x m) (/ (expt (- x) n)))|))))
+ :error nil)
 
 (in-theory (disable collect-*))
 
-(theory-invariant (not (member-equal '(:definition collect-*)
-                                     theory))
+(theory-invariant (not (active-runep '(:definition collect-*)))
                   :error nil)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -378,9 +380,9 @@
 
 (in-theory (disable bubble-down-+-problem-finder))
 
-(theory-invariant (not (member-equal '(:rewrite bubble-down-+-problem-finder)
-                                     theory))
-                  :error nil)
+(theory-invariant
+ (not (active-runep '(:rewrite bubble-down-+-problem-finder)))
+ :error nil)
 
 (defthm bubble-down-+-bubble-down
     (equal (+ (bubble-down x match) y z)
@@ -401,17 +403,13 @@
              (equal (+ (bubble-down x match) y z)
                     (+ (collect-+ x y) z))))
 
-(theory-invariant (or (not (member-equal '(:rewrite bubble-down)
-                                         theory))
-                      (and (member-equal '(:rewrite bubble-down-+-bubble-down)
-                                         theory)
-                           (member-equal '(:rewrite bubble-down-+-match-1)
-                                         theory)
-                           (member-equal '(:rewrite bubble-down-+-match-2)
-                                         theory)
-                           (member-equal '(:rewrite bubble-down-+-match-3)
-                                         theory)))
-                  :error nil)
+(theory-invariant
+ (or (not (active-runep '(:rewrite bubble-down)))
+     (and (active-runep '(:rewrite bubble-down-+-bubble-down))
+          (active-runep '(:rewrite bubble-down-+-match-1))
+          (active-runep '(:rewrite bubble-down-+-match-2))
+          (active-runep '(:rewrite bubble-down-+-match-3))))
+ :error nil)
 
 (defthm bubble-down-*-problem-finder
     (implies (equal x x)
@@ -420,9 +418,9 @@
 
 (in-theory (disable bubble-down-*-problem-finder))
 
-(theory-invariant (not (member-equal '(:rewrite bubble-down-*-problem-finder)
-                                     theory))
-                  :error nil)
+(theory-invariant
+ (not (active-runep '(:rewrite bubble-down-*-problem-finder)))
+ :error nil)
 
 (defthm bubble-down-*-bubble-down
     (equal (* (bubble-down x match) y z)
@@ -443,23 +441,17 @@
              (equal (* (bubble-down x match) y z)
                     (* (collect-* x y) z))))
 
-(theory-invariant (or (not (member-equal '(:rewrite bubble-down)
-                                         theory))
-                      (and (member-equal '(:rewrite bubble-down-*-bubble-down)
-                                         theory)
-                           (member-equal '(:rewrite bubble-down-*-match-1)
-                                         theory)
-                           (member-equal '(:rewrite bubble-down-*-match-2)
-                                         theory)
-                           (member-equal '(:rewrite bubble-down-*-match-3)
-                                         theory)))
-                  :error nil)
+(theory-invariant
+ (or (not (active-runep '(:rewrite bubble-down)))
+     (and (active-runep '(:rewrite bubble-down-*-bubble-down))
+          (active-runep '(:rewrite bubble-down-*-match-1))
+          (active-runep '(:rewrite bubble-down-*-match-2))
+          (active-runep '(:rewrite bubble-down-*-match-3))))
+ :error nil)
 
 (in-theory (disable bubble-down (:executable-counterpart bubble-down)))
 
 (theory-invariant 
- (and (not (member-equal '(:rewrite bubble-down)
-                         theory))
-      (not (member-equal '(:executable-counterpart bubble-down)
-                         theory)))
+ (and (not (active-runep '(:rewrite bubble-down)))
+      (not (active-runep '(:executable-counterpart bubble-down))))
  :error nil)
