@@ -1,3 +1,20 @@
+; Arithmetic-3 Library
+; Copyright (C) 2004 Robert Krug <rkrug@cs.utexas.edu>
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.
+;
+; This program is distributed in the hope that it will be useful but WITHOUT
+; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+; details.
+;
+; You should have received a copy of the GNU General Public License along with
+; this program; if not, write to the Free Software Foundation, Inc., 51
+; Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; normalize.lisp
@@ -139,11 +156,11 @@
         (reverse (cons (cons saved-denominator (append factors-2 factors))
                        to-be-found))))))
 
-(defun set-equal (x y)
-  (declare (xargs :guard (and (true-listp x)
-                              (true-listp y))))
-  (and (subsetp-equal x y)
-       (subsetp-equal y x)))
+(defun set-equal (a b)
+  (declare (xargs :guard (and (true-listp a)
+                              (true-listp b))))
+  (and (subsetp-equal a b)
+       (subsetp-equal b a)))
 
 (defun remainder-aaa (to-be-found rest remainder)
   (declare (xargs :guard (and (true-list-listp to-be-found)
@@ -308,15 +325,7 @@
     (make-product (set-difference-equal (factors sum)
                                         common-factors))))
 
-(defun intersection-equal (x y)
-  (declare (xargs :guard (and (true-listp x)
-                              (true-listp y))))
-  (cond ((endp x)
-         nil)
-        ((member-equal (car x) y)
-         (cons (car x) (intersection-equal (cdr x) y)))
-        (t
-         (intersection-equal (cdr x) y))))
+; Intersection-equal was added to ACL2 in Version 4.0.
 
 (defun common-factors (factors sum)
   (declare (xargs :measure (acl2-count sum)
@@ -383,18 +392,12 @@
                     (+ (bubble-down x match) y))))
 
 (theory-invariant 
- (or (not (member-equal '(:rewrite normalize-addends)
-                        theory))
-     (and (member-equal '(:rewrite bubble-down-+-bubble-down)
-                        theory)
-          (member-equal '(:rewrite bubble-down-+-match-1)
-                        theory)
-          (member-equal '(:rewrite bubble-down-+-match-2)
-                        theory)
-          (not (member-equal '(:rewrite bubble-down)
-                         theory))
-          (not (member-equal '(:executable-counterpart bubble-down)
-                             theory))))
+ (or (not (active-runep '(:rewrite normalize-addends)))
+     (and (active-runep '(:rewrite bubble-down-+-bubble-down))
+          (active-runep '(:rewrite bubble-down-+-match-1))
+          (active-runep '(:rewrite bubble-down-+-match-2))
+          (not (active-runep '(:rewrite bubble-down)))
+          (not (active-runep '(:executable-counterpart bubble-down)))))
  :error nil)
 
 (local
@@ -428,18 +431,12 @@
                     (* (bubble-down x match) y))))
 
 (theory-invariant 
- (or (not (member-equal '(:rewrite normalize-factors-gather-exponents)
-                        theory))
-     (and (member-equal '(:rewrite bubble-down-*-bubble-down)
-                        theory)
-          (member-equal '(:rewrite bubble-down-*-match-1)
-                        theory)
-          (member-equal '(:rewrite bubble-down-*-match-2)
-                        theory)
-          (not (member-equal '(:rewrite bubble-down)
-                         theory))
-          (not (member-equal '(:executable-counterpart bubble-down)
-                             theory))))
+ (or (not (active-runep '(:rewrite normalize-factors-gather-exponents)))
+     (and (active-runep '(:rewrite bubble-down-*-bubble-down))
+          (active-runep '(:rewrite bubble-down-*-match-1))
+          (active-runep '(:rewrite bubble-down-*-match-2))
+          (not (active-runep '(:rewrite bubble-down)))
+          (not (active-runep '(:executable-counterpart bubble-down)))))
  :error nil)
 
 (local
@@ -473,18 +470,12 @@
                     (* (bubble-down x match) y))))
 
 (theory-invariant 
- (or (not (member-equal '(:rewrite normalize-scatter-exponents)
-                        theory))
-     (and (member-equal '(:rewrite bubble-down-*-bubble-down)
-                        theory)
-          (member-equal '(:rewrite bubble-down-*-match-1)
-                        theory)
-          (member-equal '(:rewrite bubble-down-*-match-2)
-                        theory)
-          (not (member-equal '(:rewrite bubble-down)
-                         theory))
-          (not (member-equal '(:executable-counterpart bubble-down)
-                             theory))))
+ (or (not (active-runep '(:rewrite normalize-scatter-exponents)))
+     (and (active-runep '(:rewrite bubble-down-*-bubble-down))
+          (active-runep '(:rewrite bubble-down-*-match-1))
+          (active-runep '(:rewrite bubble-down-*-match-2))
+          (not (active-runep '(:rewrite bubble-down)))
+          (not (active-runep '(:executable-counterpart bubble-down)))))
  :error nil)
 
 (local
@@ -493,10 +484,8 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (theory-invariant 
- (not (and (member-equal '(:rewrite normalize-factors-gather-exponents)
-                         theory)
-           (member-equal '(:rewrite normalize-factors-scatter-exponents)
-                         theory)))
+ (not (and (active-runep '(:rewrite normalize-factors-gather-exponents))
+           (active-runep '(:rewrite normalize-factors-scatter-exponents))))
  :error nil)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
