@@ -1,3 +1,20 @@
+; Arithmetic-5 Library
+; Copyright (C) 2009 Robert Krug <rkrug@cs.utexas.edu>
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.
+;
+; This program is distributed in the hope that it will be useful but WITHOUT
+; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+; details.
+;
+; You should have received a copy of the GNU General Public License along with
+; this program; if not, write to the Free Software Foundation, Inc., 51
+; Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; floor-mod-basic.lisp
@@ -31,12 +48,24 @@
 (local
  (include-book "floor-mod-basic-helper"))
 
-;(set-default-hints '((nonlinearp-default-hint stable-under-simplificationp 
-;                                              hist pspv)))
-
 (SET-DEFAULT-HINTS
      '((NONLINEARP-DEFAULT-HINT++ ID STABLE-UNDER-SIMPLIFICATIONP
                                   HIST NIL)))
+
+
+;; Jared adding this to speed up certification
+(local (in-theory (disable not-integerp-type-set-rules
+                           default-times-1
+                           rationalp-x
+                           reduce-rationalp-*
+                           acl2-numberp-x
+                           rationalp-/
+                           default-plus-1
+                           default-plus-2
+                           floor-positive)))
+
+
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -66,11 +95,17 @@
 			  (< 0 i))))
   :rule-classes (:rewrite :type-prescription))
 
+(local (in-theory (disable integerp-mod-1
+                           integerp-mod-2
+                           integerp-mod-3)))
+
 (defthm rationalp-mod
   (implies (rationalp x)
            (rationalp (mod x y)))
   :rule-classes (:rewrite :type-prescription))
 
+(local (in-theory (disable rationalp-mod)))
+
 (defthm floor-mod-elim
   (implies (acl2-numberp x)
 	   (equal (+ (mod x y)
@@ -199,6 +234,7 @@
       (:linear mod-bounds-1)
       (:linear mod-bounds-2)
       (:linear mod-bounds-3)))
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -816,6 +852,9 @@ mod equalities.
   (equal (mod x x) 0)
   :hints (("Goal" :cases ((equal x 0))))
   :rule-classes (:rewrite :type-prescription))
+
+(local (in-theory (disable mod-zero-2 mod-zero)))
+
 
 (defthm mod-x-y-=-x+y
   (implies (and ;(acl2-numberp x)
