@@ -18,8 +18,18 @@
     (and (iff (car x) (car y))
          (n-bleq (1- n) (cdr x) (cdr y)))))
 
-(defun xor (x y)
-  (if x (if y nil t) (if y t nil)))
+; Changed by Matt Kaufmann to be compatible with ACL2 after Version 3.3:
+(defun xor (p q)
+
+  ":Doc-Section ACL2::Programming
+
+  logical ``exclusive or''~/
+
+  ~c[Xor] is the ACL2 exclusive-or function.  ~c[(xor P Q)] means that either
+  ~c[P] or ~c[Q], but not both, is false (i.e., ~c[nil]).~/~/"
+
+  (declare (xargs :guard t))
+  (if p (if q nil t) (if q t nil)))
 
 (defun xor3 (x y z)
   (xor x (xor y z)))
@@ -609,6 +619,16 @@
            (submaskp 256 x (bv-or 256 y z)))
  :hints (("Goal" :clause-processor (:function sat :hint nil))))
 
+; The rest are commented out by Matt K. because SBCL Version 1.0.16 with
+; #+sb-thread has caused the following error on 32-bit linux running on a
+; 64-bit machine, for the first of these:
+
+; Error:  Heap exhausted: 1327104 bytes available, 12000016 requested. PROCEED WITH CAUTION!
+
+; A similar error seems to occur for other forms after the first below, as
+; well, so we comment them all out.
+
+#||
 (defthm n-bleq-1000
   (n-bleq 1000 x x)
   :hints (("Goal" :clause-processor (:function sat :hint nil))))
@@ -674,7 +694,7 @@
          (simple-mult 4 b a))
  :hints (("Goal" :clause-processor (:function sat :hint nil))))
 
-#| ;; Commented because it takes a while 
+#| ;; Commented because it takes a while
 ;; DEBUG---turn on
 (defthm mult-8x8
  (n-bleq 16
@@ -696,6 +716,7 @@
   (valid_digits 100 x)
   (valid_digits 100 (next_counter_state 100 x)))
   :hints (("Goal" :clause-processor (:function sat :hint nil))))
+||#
 
 ;; 64x6 Add-shift False
 ;; (thm
