@@ -28,6 +28,7 @@ use strict;
 use Getopt::Long qw(:config bundling); 
 use File::Spec;
 use FindBin qw($RealBin);
+use Storable;
 
 # Note: Trying out FindBin::$RealBin.  If breaks, we can go back to
 # the system below.
@@ -77,8 +78,8 @@ my $HELP_MESSAGE = "
      cert.pl top.lisp -c    # clean
      cert.pl top.lisp -j 8  # recertify using 8 parallel processors
 
- 2. Run critpath.pl [OPTIONS] <top-book> ..., where each <top-book> is a
-    .lisp or .cert file of the book of interest.
+ 2. Run critpath.pl [OPTIONS] <target> ..., where each <target> is a
+    book with or without a .lisp or .cert extension.
 
     The options are as follows:
 
@@ -88,11 +89,16 @@ my $HELP_MESSAGE = "
        --nopath     Suppress critical path information.
        --nolist     Suppress individual-files list.
 
+       -r, --real   Toggle real time versus user+system
+
        --help       Print this help message and exit.
 
        -t, --targets <filename>
-                    Take the list of top-books from <filename>, in addition
+                    Add all targets listed in <filename>, in addition
                     to any given on the command line.
+
+       -p, --deps-of <filename>
+                    Add as targets all dependencies of the book <filename>.
  ";
 
 my %OPTIONS = (
@@ -101,10 +107,17 @@ my %OPTIONS = (
   'nowarn'  => '',
   'nopath'  => '',
   'nolist'  => '',
-  'short'   => 1
+  'short'   => 1,
+  'real'    => 0
 );
 
+my @user_targets = ();
 my @targets = ();
+my @deps_of = ();
+
+my $debug = 0;
+
+my $cache_file = 0;
 
 my $options_okp = GetOptions('h|html' => \$OPTIONS{'html'},
 			     'help'   => \$OPTIONS{'help'},
@@ -112,13 +125,25 @@ my $options_okp = GetOptions('h|html' => \$OPTIONS{'html'},
 			     'nopath' => \$OPTIONS{'nopath'},
 			     'nolist' => \$OPTIONS{'nolist'},
 			     'short=i' =>  \$OPTIONS{'short'},
+			     'real|r'  => \$OPTIONS{'real'},
+			     'debug|d' => \$debug,
 			     "targets|t=s"          
 			              => sub { shift;
-					       read_targets(shift, \@targets);
-					   }
+					       read_targets(shift, \@user_targets);
+					   },
+			     "deps-of|p=s"
+			              => \@deps_of,
+			     "cache|h=s"
+			              => \$cache_file
 			     );
 
-push (@targets, @ARGV);
+my $cache = {};
+if ($cache_file && -e $cache_file) {
+    $cache = retrieve($cache_file);
+}
+my %tscache = ();
+
+push (@user_targets, @ARGV);
 
 if (!$options_okp || $OPTIONS{"help"})
 {
@@ -142,13 +167,44 @@ certlib_set_opts(\%certlib_opts);
 certlib_add_dir("SYSTEM", $RealBin);
 
 
-foreach my $target (@targets) {
-    my $topfile = canonical_path($target);
-    $topfile =~ s/\.lisp$/.cert/;
-    add_deps($topfile, \%deps, \@sources);
-    ($costs, $warnings) = make_costs_table($topfile, \%deps, $costs, $warnings, $OPTIONS{"short"});
+foreach my $target (@user_targets) {
+    push (@targets, to_basename($target) . ".cert");
 }
 
+foreach my $top (@deps_of) {
+    (my $deps, my $two_pass) = find_deps(to_basename($top), $cache, 1, \%tscache);
+    push (@targets, @{$deps});
+}
+
+unless (@targets) {
+    print "\nError: No targets provided.\n";
+    print $HELP_MESSAGE;
+    exit 1;
+}
+
+foreach my $target (@targets) {
+    if ($target =~ /\.cert/) {
+	add_deps($target, $cache, \%deps, \@sources, \%tscache, 0);
+    }
+}
+
+$cache_file && store($cache, $cache_file);
+
+my $basecosts = {};
+read_costs(\%deps, $basecosts, $warnings, $OPTIONS{'real'});
+print "done read_costs\n" if $debug;
+
+compute_cost_paths(\%deps, $basecosts, $costs, $warnings);
+print "done compute_cost_paths\n" if $debug;
+
+print "costs: " .  $costs . "\n" if $debug;
+
+(my $topbook, my $topbook_cost) = find_most_expensive(\@targets, $costs);
+
+my %savings = %{compute_savings($costs, $basecosts, \@targets, $debug, \%deps)}; 
+
+
+	# ($costs, $warnings) = make_costs_table($target, \%deps, $costs, $warnings, $OPTIONS{"short"});
 
 
 
@@ -157,9 +213,20 @@ unless ($OPTIONS{'nowarn'}) {
 }
 
 unless ($OPTIONS{'nopath'}) {
-    print critical_path_report($costs, $OPTIONS{"html"});
+    print critical_path_report($costs, $basecosts, \%savings, $topbook, $OPTIONS{"html"}, $OPTIONS{"short"});
 }
 
 unless ($OPTIONS{'nolist'}) {
-    print individual_files_report($costs, $OPTIONS{"html"});
+    print individual_files_report($costs, $basecosts, $OPTIONS{"html"}, $OPTIONS{"short"});
+}
+
+
+
+my ($max_parallel, $max_start_time, $max_end_time, $avg_parallel, $sum_parallel)
+    = parallelism_stats($costs, $basecosts);
+
+if (! $OPTIONS{"html"}) {
+    print "Maximum parallelism: $max_parallel processes, from time $max_start_time to $max_end_time\n";
+    print "Average level of parallelism: $avg_parallel.\n";
+    print "Total time for all files: " . human_time($sum_parallel,0) . ".\n";
 }
