@@ -1,7 +1,10 @@
 ; Defsort - Defines a stable sort when given a comparison function
 ; Copyright (C) 2008 Centaur Technology
 ;
-; Contact: Jared Davis <jared@cs.utexas.edu>
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
 ;
 ; This program is free software; you can redistribute it and/or modify it under
 ; the terms of the GNU General Public License as published by the Free Software
@@ -11,10 +14,12 @@
 ; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 ; more details.  You should have received a copy of the GNU General Public
 ; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+;
+; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "ACL2")
-(include-book "generic-impl")
+(local (include-book "generic-impl"))
 (set-enforce-redundancy t)
 
 (defund duplicity (a x)
@@ -47,7 +52,7 @@
           (declare (xargs :guard t))
           (natp x)))
 
- (local (defun compare< (x y) 
+ (local (defun compare< (x y)
           (declare (xargs :guard (and (comparablep x)
                                       (comparablep y))))
           (< x y)))
@@ -115,6 +120,26 @@
   :hints(("Goal" :in-theory (enable comparable-merge))))
 
 
+(defund comparable-merge-tr (x y acc)
+  (declare (xargs :measure (+ (acl2-count x)
+                              (acl2-count y))
+                  :guard (and (comparable-listp x)
+                              (comparable-listp y)
+                              (true-listp acc))))
+  (cond ((atom x)
+         (revappend acc y))
+        ((atom y)
+         (revappend acc x))
+        ((compare< (car y) (car x))
+         (comparable-merge-tr x (cdr y) (cons (car y) acc)))
+        (t
+         (comparable-merge-tr (cdr x) y (cons (car x) acc)))))
+
+(defthm comparable-merge-tr-removal
+  (equal (comparable-merge-tr x y nil)
+         (comparable-merge x y)))
+
+
 (defconst *mergesort-fixnum-threshold* 536870912)
 
 (defund fast-comparable-mergesort-fixnums (x len)
@@ -138,15 +163,14 @@
                                (logand (the (signed-byte 30) len) 1)))))
                  (part1 (fast-comparable-mergesort-fixnums x len1))
                  (part2 (fast-comparable-mergesort-fixnums (nthcdr len1 x) len2)))
-            (comparable-merge part1 part2)))))
+            (comparable-merge-tr part1 part2 nil)))))
 
 (defund fast-comparable-mergesort-integers (x len)
   (declare (xargs :guard (and (true-listp x)
                               (comparable-listp x)
                               (natp len)
                               (<= len (len x)))
-                  :measure (nfix len)
-                  :verify-guards nil)
+                  :measure (nfix len))
            (type integer len))
   (cond ((mbe :logic (zp len)
               :exec (= (the integer len) 0))
@@ -164,7 +188,7 @@
                 (part2 (if (< (the integer len2) *mergesort-fixnum-threshold*)
                            (fast-comparable-mergesort-fixnums (nthcdr len1 x) len2)
                          (fast-comparable-mergesort-integers (nthcdr len1 x) len2))))
-           (comparable-merge part1 part2)))))
+           (comparable-merge-tr part1 part2 nil)))))
 
 (defund comparable-mergesort (x)
   (declare (xargs :guard (and (true-listp x)
@@ -222,7 +246,7 @@
                 (comparable-listp (cdr x))))
   :rule-classes nil)
 
-(defthm comparable-merge-admission 
+(defthm comparable-merge-admission
   (and (o-p (+ (acl2-count x) (acl2-count y)))
        (implies (and (not (atom x))
                      (not (atom y))
@@ -236,7 +260,7 @@
                     (+ (acl2-count x) (acl2-count y)))))
   :rule-classes nil)
 
-(defthm comparable-merge-guards 
+(defthm comparable-merge-guards
   (and (implies (and (comparable-listp y)
                      (comparable-listp x)
                      (not (atom x))
@@ -261,7 +285,117 @@
                 (comparable-listp (cdr x))))
   :rule-classes nil)
 
+(defthm comparable-merge-tr-guards
+  (AND (IMPLIES (AND (TRUE-LISTP ACC)
+                     (COMPARABLE-LISTP Y)
+                     (COMPARABLE-LISTP X)
+                     (NOT (ATOM X))
+                     (NOT (ATOM Y))
+                     (COMPARE< (CAR Y) (CAR X)))
+                (COMPARABLE-LISTP (CDR Y)))
+       (IMPLIES (AND (TRUE-LISTP ACC)
+                     (COMPARABLE-LISTP Y)
+                     (COMPARABLE-LISTP X)
+                     (NOT (ATOM X))
+                     (NOT (ATOM Y))
+                     (NOT (COMPARE< (CAR Y) (CAR X))))
+                (COMPARABLE-LISTP (CDR X))))
+  :rule-classes nil)
+
 (defthm fast-comparable-mergesort-fixnums-guards
+  (AND
+   (IMPLIES (AND (TRUE-LISTP X)
+                 (COMPARABLE-LISTP X)
+                 (NATP LEN))
+            (RATIONALP (LEN X)))
+   (IMPLIES (AND (TRUE-LISTP X)
+                 (COMPARABLE-LISTP X)
+                 (NATP LEN))
+            (RATIONALP LEN))
+   (IMPLIES (AND (SIGNED-BYTE-P 30 LEN)
+                 (<= LEN (LEN X))
+                 (NATP LEN)
+                 (COMPARABLE-LISTP X)
+                 (TRUE-LISTP X))
+            (EQUAL (ZP LEN) (= LEN 0)))
+   (IMPLIES (AND (SIGNED-BYTE-P 30 LEN)
+                 (<= LEN (LEN X))
+                 (NATP LEN)
+                 (COMPARABLE-LISTP X)
+                 (TRUE-LISTP X))
+            (LET ((VAR LEN))
+                 (SIGNED-BYTE-P 30 VAR)))
+   (IMPLIES (AND (SIGNED-BYTE-P 30 LEN)
+                 (<= LEN (LEN X))
+                 (NATP LEN)
+                 (COMPARABLE-LISTP X)
+                 (TRUE-LISTP X))
+            (ACL2-NUMBERP LEN))
+   (IMPLIES (AND (SIGNED-BYTE-P 30 LEN)
+                 (<= LEN (LEN X))
+                 (NATP LEN)
+                 (COMPARABLE-LISTP X)
+                 (TRUE-LISTP X)
+                 (NOT (ZP LEN))
+                 (NOT (= LEN 1)))
+            (INTEGERP LEN))
+   (IMPLIES
+    (AND (SIGNED-BYTE-P 30 LEN)
+         (<= LEN (LEN X))
+         (NATP LEN)
+         (COMPARABLE-LISTP X)
+         (TRUE-LISTP X)
+         (NOT (ZP LEN))
+         (NOT (= LEN 1)))
+    (LET
+     ((LEN1 (ASH LEN -1)))
+     (AND
+      (LET ((VAR LEN1))
+           (SIGNED-BYTE-P 30 VAR))
+      (LET ((VAR LEN)) (SIGNED-BYTE-P 30 VAR))
+      (INTEGERP LEN)
+      (LET ((VAR (LOGAND LEN 1)))
+           (SIGNED-BYTE-P 30 VAR))
+      (ACL2-NUMBERP LEN1)
+      (ACL2-NUMBERP (LOGAND LEN 1))
+      (LET ((VAR (+ LEN1 (LOGAND LEN 1))))
+           (SIGNED-BYTE-P 30 VAR))
+      (LET
+       ((LEN2 (+ LEN1 (LOGAND LEN 1))))
+       (AND
+        (TRUE-LISTP X)
+        (SIGNED-BYTE-P 30 LEN1)
+        (NATP LEN1)
+        (COMPARABLE-LISTP X)
+        (<= LEN1 (LEN X))
+        (LET
+         ((PART1 (FAST-COMPARABLE-MERGESORT-FIXNUMS X LEN1)))
+         (AND (INTEGERP LEN1)
+              (TRUE-LISTP X)
+              (<= 0 LEN1)
+              (TRUE-LISTP (NTHCDR LEN1 X))
+              (SIGNED-BYTE-P 30 LEN2)
+              (NATP LEN2)
+              (COMPARABLE-LISTP (NTHCDR LEN1 X))
+              (<= LEN2 (LEN (NTHCDR LEN1 X)))
+              (LET ((PART2 (FAST-COMPARABLE-MERGESORT-FIXNUMS (NTHCDR LEN1 X)
+                                                              LEN2)))
+                   (AND (COMPARABLE-LISTP PART1)
+                        (COMPARABLE-LISTP NIL)
+                        (COMPARABLE-LISTP PART2)
+                        (TRUE-LISTP NIL))))))))))
+   (IMPLIES (AND (SIGNED-BYTE-P 30 LEN)
+                 (<= LEN (LEN X))
+                 (NATP LEN)
+                 (COMPARABLE-LISTP X)
+                 (TRUE-LISTP X)
+                 (NOT (ZP LEN))
+                 (NOT (= LEN 1)))
+            (LET ((VAR (ASH LEN -1)))
+                 (SIGNED-BYTE-P 30 VAR))))
+  :rule-classes nil)
+
+(defthm fast-comparable-mergesort-integers-guards
   (AND
     (IMPLIES (AND (TRUE-LISTP X)
                   (COMPARABLE-LISTP X)
@@ -271,35 +405,14 @@
                   (COMPARABLE-LISTP X)
                   (NATP LEN))
              (RATIONALP LEN))
-    (IMPLIES (AND (SIGNED-BYTE-P 30 LEN)
+    (IMPLIES (AND (INTEGERP LEN)
                   (<= LEN (LEN X))
                   (NATP LEN)
                   (COMPARABLE-LISTP X)
                   (TRUE-LISTP X))
              (EQUAL (ZP LEN) (= LEN 0)))
-    (IMPLIES (AND (SIGNED-BYTE-P 30 LEN)
-                  (<= LEN (LEN X))
-                  (NATP LEN)
-                  (COMPARABLE-LISTP X)
-                  (TRUE-LISTP X))
-             (LET ((VAR LEN))
-                  (SIGNED-BYTE-P 30 VAR)))
-    (IMPLIES (AND (SIGNED-BYTE-P 30 LEN)
-                  (<= LEN (LEN X))
-                  (NATP LEN)
-                  (COMPARABLE-LISTP X)
-                  (TRUE-LISTP X))
-             (ACL2-NUMBERP LEN))
-    (IMPLIES (AND (SIGNED-BYTE-P 30 LEN)
-                  (<= LEN (LEN X))
-                  (NATP LEN)
-                  (COMPARABLE-LISTP X)
-                  (TRUE-LISTP X)
-                  (NOT (ZP LEN))
-                  (NOT (= LEN 1)))
-             (INTEGERP LEN))
     (IMPLIES
-     (AND (SIGNED-BYTE-P 30 LEN)
+     (AND (INTEGERP LEN)
           (<= LEN (LEN X))
           (NATP LEN)
           (COMPARABLE-LISTP X)
@@ -309,39 +422,75 @@
      (LET
       ((LEN1 (ASH LEN -1)))
       (AND
-       (LET ((VAR LEN1))
-            (SIGNED-BYTE-P 30 VAR))
-       (LET ((VAR LEN)) (SIGNED-BYTE-P 30 VAR))
+       (LET ((VAR LEN1)) (INTEGERP VAR))
+       (LET ((VAR LEN)) (INTEGERP VAR))
        (INTEGERP LEN)
        (LET ((VAR (LOGAND LEN 1)))
-            (SIGNED-BYTE-P 30 VAR))
+            (INTEGERP VAR))
        (ACL2-NUMBERP LEN1)
        (ACL2-NUMBERP (LOGAND LEN 1))
        (LET ((VAR (+ LEN1 (LOGAND LEN 1))))
-            (SIGNED-BYTE-P 30 VAR))
+            (INTEGERP VAR))
        (LET
         ((LEN2 (+ LEN1 (LOGAND LEN 1))))
         (AND
-         (TRUE-LISTP X)
-         (SIGNED-BYTE-P 30 LEN1)
-         (NATP LEN1)
-         (COMPARABLE-LISTP X)
-         (<= LEN1 (LEN X))
+         (LET ((VAR LEN1)) (INTEGERP VAR))
+         (RATIONALP LEN1)
+         (OR (<= 536870912 LEN1) (TRUE-LISTP X))
+         (OR (<= 536870912 LEN1)
+             (SIGNED-BYTE-P 30 LEN1))
+         (OR (<= 536870912 LEN1) (NATP LEN1))
+         (OR (<= 536870912 LEN1)
+             (COMPARABLE-LISTP X))
+         (OR (<= 536870912 LEN1)
+             (<= LEN1 (LEN X)))
+         (OR (< LEN1 536870912) (TRUE-LISTP X))
+         (OR (< LEN1 536870912) (INTEGERP LEN1))
+         (OR (< LEN1 536870912) (NATP LEN1))
+         (OR (< LEN1 536870912)
+             (COMPARABLE-LISTP X))
+         (OR (< LEN1 536870912)
+             (<= LEN1 (LEN X)))
          (LET
-          ((PART1 (FAST-COMPARABLE-MERGESORT-FIXNUMS X LEN1)))
-          (AND (INTEGERP LEN1)
-               (TRUE-LISTP X)
-               (<= 0 LEN1)
-               (TRUE-LISTP (NTHCDR LEN1 X))
-               (SIGNED-BYTE-P 30 LEN2)
-               (NATP LEN2)
-               (COMPARABLE-LISTP (NTHCDR LEN1 X))
-               (<= LEN2 (LEN (NTHCDR LEN1 X)))
-               (LET ((PART2 (FAST-COMPARABLE-MERGESORT-FIXNUMS (NTHCDR LEN1 X)
-                                                               LEN2)))
-                    (AND (COMPARABLE-LISTP PART1)
-                         (COMPARABLE-LISTP PART2))))))))))
-    (IMPLIES (AND (SIGNED-BYTE-P 30 LEN)
+          ((PART1 (IF (< LEN1 536870912)
+                      (FAST-COMPARABLE-MERGESORT-FIXNUMS X LEN1)
+                      (FAST-COMPARABLE-MERGESORT-INTEGERS X LEN1))))
+          (AND
+           (LET ((VAR LEN2)) (INTEGERP VAR))
+           (RATIONALP LEN2)
+           (OR (<= 536870912 LEN2) (INTEGERP LEN1))
+           (OR (<= 536870912 LEN2) (TRUE-LISTP X))
+           (OR (<= 536870912 LEN2) (<= 0 LEN1))
+           (OR (<= 536870912 LEN2)
+               (TRUE-LISTP (NTHCDR LEN1 X)))
+           (OR (<= 536870912 LEN2)
+               (SIGNED-BYTE-P 30 LEN2))
+           (OR (<= 536870912 LEN2) (NATP LEN2))
+           (OR (<= 536870912 LEN2)
+               (COMPARABLE-LISTP (NTHCDR LEN1 X)))
+           (OR (<= 536870912 LEN2)
+               (<= LEN2 (LEN (NTHCDR LEN1 X))))
+           (OR (< LEN2 536870912) (INTEGERP LEN1))
+           (OR (< LEN2 536870912) (TRUE-LISTP X))
+           (OR (< LEN2 536870912) (<= 0 LEN1))
+           (OR (< LEN2 536870912)
+               (TRUE-LISTP (NTHCDR LEN1 X)))
+           (OR (< LEN2 536870912) (INTEGERP LEN2))
+           (OR (< LEN2 536870912) (NATP LEN2))
+           (OR (< LEN2 536870912)
+               (COMPARABLE-LISTP (NTHCDR LEN1 X)))
+           (OR (< LEN2 536870912)
+               (<= LEN2 (LEN (NTHCDR LEN1 X))))
+           (LET ((PART2 (IF (< LEN2 536870912)
+                            (FAST-COMPARABLE-MERGESORT-FIXNUMS (NTHCDR LEN1 X)
+                                                               LEN2)
+                            (FAST-COMPARABLE-MERGESORT-INTEGERS (NTHCDR LEN1 X)
+                                                                LEN2))))
+                (AND (COMPARABLE-LISTP PART1)
+                     (COMPARABLE-LISTP NIL)
+                     (COMPARABLE-LISTP PART2)
+                     (TRUE-LISTP NIL))))))))))
+    (IMPLIES (AND (INTEGERP LEN)
                   (<= LEN (LEN X))
                   (NATP LEN)
                   (COMPARABLE-LISTP X)
@@ -349,117 +498,9 @@
                   (NOT (ZP LEN))
                   (NOT (= LEN 1)))
              (LET ((VAR (ASH LEN -1)))
-                  (SIGNED-BYTE-P 30 VAR))))
+                  (INTEGERP VAR))))
   :rule-classes nil)
 
-(defthm fast-comparable-mergesort-integers-guards
-  (AND
-     (IMPLIES (AND (TRUE-LISTP X)
-                   (COMPARABLE-LISTP X)
-                   (NATP LEN))
-              (RATIONALP (LEN X)))
-     (IMPLIES (AND (TRUE-LISTP X)
-                   (COMPARABLE-LISTP X)
-                   (NATP LEN))
-              (RATIONALP LEN))
-     (IMPLIES (AND (INTEGERP LEN)
-                   (<= LEN (LEN X))
-                   (NATP LEN)
-                   (COMPARABLE-LISTP X)
-                   (TRUE-LISTP X))
-              (EQUAL (ZP LEN) (= LEN 0)))
-     (IMPLIES
-      (AND (INTEGERP LEN)
-           (<= LEN (LEN X))
-           (NATP LEN)
-           (COMPARABLE-LISTP X)
-           (TRUE-LISTP X)
-           (NOT (ZP LEN))
-           (NOT (= LEN 1)))
-      (LET
-       ((LEN1 (ASH LEN -1)))
-       (AND
-        (LET ((VAR LEN1)) (INTEGERP VAR))
-        (LET ((VAR LEN)) (INTEGERP VAR))
-        (INTEGERP LEN)
-        (LET ((VAR (LOGAND LEN 1)))
-             (INTEGERP VAR))
-        (ACL2-NUMBERP LEN1)
-        (ACL2-NUMBERP (LOGAND LEN 1))
-        (LET ((VAR (+ LEN1 (LOGAND LEN 1))))
-             (INTEGERP VAR))
-        (LET
-         ((LEN2 (+ LEN1 (LOGAND LEN 1))))
-         (AND
-          (LET ((VAR LEN1)) (INTEGERP VAR))
-          (RATIONALP LEN1)
-          (OR (<= 536870912 LEN1) (TRUE-LISTP X))
-          (OR (<= 536870912 LEN1)
-              (SIGNED-BYTE-P 30 LEN1))
-          (OR (<= 536870912 LEN1) (NATP LEN1))
-          (OR (<= 536870912 LEN1)
-              (COMPARABLE-LISTP X))
-          (OR (<= 536870912 LEN1)
-              (<= LEN1 (LEN X)))
-          (OR (< LEN1 536870912) (TRUE-LISTP X))
-          (OR (< LEN1 536870912) (INTEGERP LEN1))
-          (OR (< LEN1 536870912) (NATP LEN1))
-          (OR (< LEN1 536870912)
-              (COMPARABLE-LISTP X))
-          (OR (< LEN1 536870912)
-              (<= LEN1 (LEN X)))
-          (LET
-           ((PART1 (IF (< LEN1 536870912)
-                       (FAST-COMPARABLE-MERGESORT-FIXNUMS X LEN1)
-                       (FAST-COMPARABLE-MERGESORT-INTEGERS X LEN1))))
-           (AND
-            (LET ((VAR LEN2)) (INTEGERP VAR))
-            (RATIONALP LEN2)
-            (OR (<= 536870912 LEN2) (INTEGERP LEN1))
-            (OR (<= 536870912 LEN2) (TRUE-LISTP X))
-            (OR (<= 536870912 LEN2) (<= 0 LEN1))
-            (OR (<= 536870912 LEN2)
-                (TRUE-LISTP (NTHCDR LEN1 X)))
-            (OR (<= 536870912 LEN2)
-                (SIGNED-BYTE-P 30 LEN2))
-            (OR (<= 536870912 LEN2) (NATP LEN2))
-            (OR (<= 536870912 LEN2)
-                (COMPARABLE-LISTP (NTHCDR LEN1 X)))
-            (OR (<= 536870912 LEN2)
-                (<= LEN2 (LEN (NTHCDR LEN1 X))))
-            (OR (< LEN2 536870912) (INTEGERP LEN1))
-            (OR (< LEN2 536870912) (TRUE-LISTP X))
-            (OR (< LEN2 536870912) (<= 0 LEN1))
-            (OR (< LEN2 536870912)
-                (TRUE-LISTP (NTHCDR LEN1 X)))
-            (OR (< LEN2 536870912) (INTEGERP LEN2))
-            (OR (< LEN2 536870912) (NATP LEN2))
-            (OR (< LEN2 536870912)
-                (COMPARABLE-LISTP (NTHCDR LEN1 X)))
-            (OR (< LEN2 536870912)
-                (<= LEN2 (LEN (NTHCDR LEN1 X))))
-            (LET ((PART2 (IF (< LEN2 536870912)
-                             (FAST-COMPARABLE-MERGESORT-FIXNUMS (NTHCDR LEN1 X)
-                                                                LEN2)
-                             (FAST-COMPARABLE-MERGESORT-INTEGERS (NTHCDR LEN1 X)
-                                                                 LEN2))))
-                 (AND (COMPARABLE-LISTP PART1)
-                      (COMPARABLE-LISTP PART2))))))))))
-     (IMPLIES (AND (INTEGERP LEN)
-                   (<= LEN (LEN X))
-                   (NATP LEN)
-                   (COMPARABLE-LISTP X)
-                   (TRUE-LISTP X)
-                   (NOT (ZP LEN))
-                   (NOT (= LEN 1)))
-              (LET ((VAR (ASH LEN -1)))
-                   (INTEGERP VAR))))
-  :rule-classes nil)
-
-
-
-
-
 
 
 
