@@ -55,7 +55,7 @@
               (create-pkgs-for-cert-file1 file1 file2 ch known-pkgs
                                           new-pkgs-sofar ctx state)
               (er-progn
-               (defpkg-fn pkg nil state nil nil
+               (defpkg-fn pkg nil state nil nil nil
                  `(defpkg ,pkg nil))
                (create-pkgs-for-cert-file1 file1 file2 ch known-pkgs
                                            (cons pkg new-pkgs-sofar)
@@ -134,7 +134,7 @@
        dir-value
        (prog2$ (cw "~%Fixing .cert file for ~x0~%" user-book-name)
                user-book-name)
-       ".lisp" state)
+       ".lisp" ctx state)
      (declare (ignorable new-directory-name-with-slash new-familiar-name))
      (er-let*
       ((new-pkg-lst (create-pkgs-for-cert-file new-full-book-name ctx state))
