@@ -21,5 +21,5 @@
                                         :check-expansion
                                         (defun test10 (x) (cons x x)))
                                        ))
-                                     (VALUE-TRIPLE T))))))
+                                     (VALUE-TRIPLE 'T))))))
             (mv erp nil state))))
