@@ -1,9 +1,10 @@
 (in-package "ACL2")
 
-(defun natp (x)
-  (declare (xargs :guard t))
-  (and (integerp x)
-       (<= 0 x)))
+(local ; ACL2 primitive
+ (defun natp (x)
+   (declare (xargs :guard t))
+   (and (integerp x)
+        (<= 0 x))))
 
 (defund bvecp (x k)
   (declare (xargs :guard (integerp k)))
