@@ -582,7 +582,7 @@
         (implies (and (,field-predicate ,x)
                       (natp ,n)
                       (< ,n (len ,x)))
-                 ,(translate-declaration-to-guard elt-type `(nth ,n ,x))))
+                 ,(translate-declaration-to-guard elt-type `(nth ,n ,x) nil)))
      `(defun ,copy-to-name (,n ,field-name ,stobj-name)
         (declare (xargs :stobjs (,stobj-name)
                         :measure (acl2-count (1+ ,n)) ;needed?
