@@ -425,9 +425,7 @@
 ; defattach-merge-into-component.
 
   (cond ((atom path) (null path))
-        (t (and (consp (car path))
-                (pseudo-function-symbolp (caar path) nil)
-                (pseudo-function-symbolp (cdar path) nil)
+        (t (and (pseudo-function-symbolp (car path) nil)
                 (attachment-components-pathp (cdr path))))))
 
 (defun attachment-component-p (c)
@@ -1364,6 +1362,7 @@
     (EMBEDDED-EVENT-LST (pseudo-embedded-event-lstp val))
     (CLTL-COMMAND (pseudo-cltl-commandp val))
     (TOP-LEVEL-CLTL-COMMAND-STACK (pseudo-cltl-command-listp val))
+    (HONS-ENABLED (booleanp val))
     (INCLUDE-BOOK-ALIST (pseudo-include-book-alistp val))
     (INCLUDE-BOOK-ALIST-ALL (pseudo-include-book-alist-allp val))
     (INCLUDE-BOOK-PATH (pseudo-include-book-pathp val))
