@@ -60,18 +60,18 @@
            (equal (ordered-symbol-alistp (add-pair key val list))
                   (symbolp key))))
 
-(defthm delete-pair-ordered-symbol-alistp
+(defthm delete-assoc-eq-ordered-symbol-alistp
   (implies (ordered-symbol-alistp list)
-           (ordered-symbol-alistp (delete-pair key list))))
+           (ordered-symbol-alistp (delete-assoc-eq key list))))
 
 (defthm add-pair-open-channel-listp
   (implies (open-channel-listp list)
            (equal (open-channel-listp (add-pair key val list))
                   (open-channel1 val))))
 
-(defthm delete-pair-open-channel-listp
+(defthm delete-assoc-eq-open-channel-listp
   (implies (open-channel-listp list)
-           (open-channel-listp (delete-pair key list))))
+           (open-channel-listp (delete-assoc-eq key list))))
 
 (defthm add-pair-open-channels-p
   (implies (open-channels-p list)
@@ -79,9 +79,9 @@
                   (and (symbolp key)
                        (open-channel1 val)))))
 
-(defthm delete-pair-open-channels-p
+(defthm delete-assoc-eq-open-channels-p
   (implies (open-channels-p list)
-           (open-channels-p (delete-pair key list))))
+           (open-channels-p (delete-assoc-eq key list))))
 
 
 
@@ -124,9 +124,10 @@
 ; list will be successful.  Secondly, we know that the type of the channel
 ; returned will be the same as type in our call to open-input-channel-p1.
 
-(local (defthm assoc-eq-is-assoc-equal
-         (equal (assoc-eq a x)
-                (assoc-equal a x))))
+; [Removed by Matt K. to handle changes to member, assoc, etc. after ACL2 4.2.]
+; (local (defthm assoc-eq-is-assoc-equal
+;          (equal (assoc-eq a x)
+;                 (assoc-equal a x))))
 
 (defthm assoc-open-input-channels-exists
   (implies (open-input-channel-p1 channel type state)
