@@ -1,5 +1,5 @@
 ; XDOC Documentation System for ACL2
-; Copyright (C) 2009-2010 Centaur Technology
+; Copyright (C) 2009-2011 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
@@ -29,26 +29,17 @@
 (program)
 
 
-(defun fmt-to-chars (string alist state) ;; ==> (MV CHARS STATE)
+(defun fmt-to-chars (string alist state)
 
-; We can't use fmt to print directly to a string, which sucks.  So, we print to
-; a temporary file and read it back in.  This is truly horrible, but it's a lot
-; easier than reimplementing ppr.
+; Use ACL2's fancy new string-printing stuff to pretty-print an object into a
+; string.
 
-  (b* (((mv channel state)  (open-output-channel ".fmt-to-chars.tmp" :character state))
+  (b* (((mv channel state)  (open-output-channel :string :character state))
        ((mv & state)        (fmt string alist channel state nil))
-       (state               (close-output-channel channel state))
-       ((mv data state)     (acl2::read-file-characters ".fmt-to-chars.tmp" state)))
-      (if (stringp data)
-          (prog2$
-           (cw "; xdoc note: in fmt-to-chars, failed to read .fmt-to-chars.tmp~%")
-           ;; In this funny return value, we insert a newline to be consistent
-           ;; with fmt.
-           (mv (coerce "
-[[ error reading .fmt-to-chars.tmp ]]" 'list)
-               state))
-        (mv data state))))
-
+       ((mv err str state)  (get-output-stream-string$ channel state)))
+    (or (not err)
+        (er hard? 'fmt-to-chars "Error with get-output-stream-string$???"))
+    (mv (coerce str 'list) state)))
 
 (defun fmt-to-chars-and-encode (string alist state acc) ;; ==> (MV ACC-PRIME STATE)
 
@@ -121,7 +112,8 @@
 
 (defun get-event (name world)
   ;; A general purpose event lookup as in :pe
-  (let ((evt (acl2::get-event name world)))
+  (let* ((props (acl2::getprops name 'current-acl2-world world))
+        (evt   (and props (acl2::get-event name world))))
     (or evt
         (cw "; xdoc note: get-event failed for ~x0.~%" name)
         (concatenate 'string
@@ -131,28 +123,66 @@
                      (symbol-name name)))))
 
 (defun get-def (fn world)
-  ;; This used to do something else.  Now we try to be permissive.
-  (let ((def (acl2::get-def fn world)))
-    (if def
-        (cons 'defun def)
-      (get-event fn world))))
+  (get-event fn world))
 
 (defun get-theorem (name world)
   ;; BOZO maybe do some cleaning to remove hints, etc.
   (get-event name world))
 
+;; (defmacro foo ()
+;;   `(progn (logic)
+;;           (make-event
+;;            '(encapsulate
+;;               (((h *) => *))
+;;               (local (defun h (x) (+ x 1)))
+;;               (defun f (x) (+ x 1))
+;;               (defun g (x) (+ x 2))))))
 
+;; (defstobj st fld)
+
+;; (defun-sk all-integerp (x)
+;;   (forall a (implies (member-equal a x)
+;;                      (integerp a))))
+
+;; (defconst *const* 3)
+
+;; (foo)
+
+;; (get-event 'undefined (w state)) ; good, fails
 ;; (get-event 'append (w state))
-;; (get-def 'append (w state))
 ;; (get-event 'binary-append (w state))
-;; (get-def 'binary-append (w state))
+;; (get-event 'st (w state))
+;; (get-event 'fld (w state)) ;; bad? returns the whole stobj
+;; (get-event 'all-integerp (w state))
+;; (get-event 'all-integerp-witness (w state)) ;; good i guess - returns the encapsulate
+;; (get-event 'f (w state))
+;; (get-event 'h (w state)) ;; good i guess, returns the encapsulate
+;; (get-event 'acl2::car-cons (w state))
+;; (get-event '*const* (w state))
+
 ;; (get-formals 'binary-append (w state))  ;; --> (ACL2::X ACL2::Y)
 ;; (get-formals 'append (w state)) ;; --> (ACL2::X ACL2::Y &REST ACL2::RST)
-;; (get-measure 'binary-append (w state))
-;; (get-guard 'binary-append (w state))
-;; (get-body 'binary-append (w state))
-;; (get-def 'binary-append (w state))
-;; (get-theorem 'acl2::car-cons (w state))
+;; (get-formals 'all-integerp-witness (w state)) ;; good, works
+;; (get-formals 'all-integerp (w state)) ;; good, works
+;; (get-formals 'fld (w state)) ;; good, works
+;; (get-formals 'st (w state)) ;; good, fails
+
+;; (get-measure 'binary-append (w state)) ;; good, works
+;; (get-measure 'append (w state))  ;; good, fails
+;; (get-measure 'st (w state)) ;; good, fails
+;; (get-measure 'fld (w state)) ;; good, fails
+;; (get-measure 'all-integerp-witness (w state)) ;; good, fails
+;; (get-measure 'all-integerp (w state)) ;; good, fails
+
+;; (get-guard 'binary-append (w state)) ;; good, works
+;; (get-guard 'append (w state)) ;; hrmn -- fails?
+;; (get-guard 'all-integerp-witness (w state)) ;; NIL???
+;; (get-guard 'all-integerp (w state)) ;; NIL???
+;; (get-guard 'fld (w state)) ;; works
+;; (get-guard 'st (w state)) ;; good, fails
+
+
+
 
 
 
