@@ -1,5 +1,5 @@
 ; XDOC Documentation System for ACL2
-; Copyright (C) 2009-2010 Centaur Technology
+; Copyright (C) 2009-2011 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
@@ -27,11 +27,6 @@
 
 (in-package "XDOC")
 (include-book "base")
-; (depends-on "save.cert")
-; (depends-on "display.cert")
-; (depends-on "topics.cert")
-; (depends-on "defxdoc-raw.cert")
-; (depends-on "mkdir-raw.cert")
 
 (make-event `(defconst *xdoc-dir/save*
                ,(acl2::extend-pathname *xdoc-dir* "save" state)))
@@ -45,6 +40,7 @@
                ,(acl2::extend-pathname *xdoc-dir* "mkdir-raw" state)))
 
 
+
 (defmacro xdoc (name)
   (declare (xargs :guard (or (symbolp name)
                              (and (quotep name)
@@ -57,42 +53,50 @@
          (include-book ,*xdoc-dir/defxdoc-raw* :ttags :all)
          (include-book ,*xdoc-dir/topics*)
          (include-book ,*xdoc-dir/display*)
+         (import-acl2doc)
+         (maybe-import-bookdoc)
          (make-event
           (b* (((mv all-xdoc-topics state) (all-xdoc-topics state))
-               ((mv & & state) (colon-xdoc-fn ',name nil all-xdoc-topics state)))
+               ((mv & & state) (colon-xdoc-fn ',name all-xdoc-topics state)))
             (value '(value-triple :invisible))))))))
 
-(defmacro xdoc! (name)
-  (declare (xargs :guard (or (symbolp name)
-                             (and (quotep name)
-                                  (symbolp (unquote name))))))
-  (let ((name (if (symbolp name)
-                  name
-                (unquote name))))
-    `(with-output :off (summary event)
-       (progn
-         (include-book ,*xdoc-dir/defxdoc-raw* :ttags :all)
-         (include-book ,*xdoc-dir/topics*)
-         (include-book ,*xdoc-dir/display*)
-         (make-event
-          (b* (((mv all-xdoc-topics state) (all-xdoc-topics state))
-               ((mv & & state) (colon-xdoc-fn ',name t all-xdoc-topics state)))
-            (value '(value-triple :invisible))))))))
-
-(defmacro save (dir &key (index-pkg 'acl2::foo))
+(defmacro save (dir &key
+                    (index-pkg 'acl2::foo)
+                    (expand-level '1)
+                    (import 't))
   `(progn
      (include-book ,*xdoc-dir/defxdoc-raw* :ttags :all)
      (include-book ,*xdoc-dir/mkdir-raw* :ttags :all)
-     (include-book ,*xdoc-dir/topics*)
      (include-book ,*xdoc-dir/save*)
 
      ;; ugh, stupid stupid writes-ok stupidity
      (defttag :xdoc)
      (remove-untouchable acl2::writes-okp nil)
-
+     ,@(and import
+            `((include-book ,*xdoc-dir/topics*)
+              (import-acl2doc)
+              (maybe-import-bookdoc)))
      (make-event
       (b* (((mv all-xdoc-topics state) (all-xdoc-topics state))
+           (- (cw "(len all-xdoc-topics): ~x0~%" (len all-xdoc-topics)))
            ((mv & & state) (assign acl2::writes-okp t))
-           (state (save-topics all-xdoc-topics ,dir ',index-pkg state)))
+           (state (save-topics all-xdoc-topics ,dir ',index-pkg ',expand-level state)))
         (value '(value-triple :invisible))))))
 
+(defmacro xdoc-just-from-events (events)
+  `(encapsulate
+     ()
+     (local (include-book ,*xdoc-dir/topics*))
+     (local ,events)
+     (make-event
+      (mv-let (er val state)
+        (let ((state (acl2::f-put-global 'acl2::xdoc-alist nil state)))
+          (time$ (acl2::write-xdoc-alist :skip-topics xdoc::*acl2-ground-zero-names*)
+                 :msg "; Importing :doc topics took ~st sec, ~sa bytes~%"
+                 :mintime 1))
+        (declare (ignore er val))
+        (b* ((topics (acl2::f-get-global 'acl2::xdoc-alist state))
+             (- (cw "(len topics): ~x0~%" (len topics))))
+          (value `(table xdoc 'doc ',topics)))))))
+
+
