@@ -1,5 +1,5 @@
 ; XDOC Documentation System for ACL2
-; Copyright (C) 2009-2010 Centaur Technology
+; Copyright (C) 2009-2011 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
@@ -26,8 +26,8 @@
 ; time of top.lisp.
 
 (in-package "XDOC")
-(include-book "base")
-
+(include-book "import-acl2doc")  ;; For base acl2 documentation
+(program)
 
 (defxdoc xdoc
   :short "<em>XDOC</em> is a tool for documenting ACL2 libraries, and is
@@ -46,9 +46,8 @@ provides a (nearly) complete interface to the XDOC system, including:</p>
 <ul>
 <li>@(see defxdoc), the basic command for adding documentation, which is the
 XDOC replacement for ACL2's <tt>defdoc</tt> command.</li>
-<li>The <tt>:xdoc</tt> and <tt>:xdoc!</tt> commands for viewing documentation
-within the terminal, which are the XDOC replacements for ACL2's <tt>:doc</tt>
-and <tt>:doc!</tt> commands.</li>
+<li>The <tt>:xdoc</tt> command for viewing documentation within the terminal,
+which is the XDOC replacement for ACL2's <tt>:doc</tt> command.</li>
 <li>The @(see save) command, which exports all XDOC documentation as XML files
 that can be viewed in a web browser or transformed into formats like HTML.</li>
 </ul>
@@ -196,6 +195,10 @@ Technology</a>.</p>
 <p>Paragraphs should be wrapped in <tt>&lt;p&gt;</tt> and <tt>&lt;/p&gt;</tt>
 tags.</p>
 
+<blockquote>You can use <tt>&lt;blockquote&gt;</tt> to create indented
+paragraphs like this one.  (Let's put enough text here to make it word-wrap.
+Mumble mumble mumble.  Mumble.  Mumble mumble.)</blockquote>
+
 <p><tt>&lt;br/&gt;</tt> can be used<br/>
 to write haikus and so on<br/>
 but is discouraged</p>
