@@ -1,4 +1,4 @@
-; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -20,9 +20,9 @@
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
-; Department of Computer Sciences
+; Department of Computer Science
 ; University of Texas at Austin
-; Austin, TX 78712-1188 U.S.A.
+; Austin, TX 78701 U.S.A.
 
 ; This file, defpkgs.lisp, illustrates the idea that defpkg forms
 ; should be off in files all by themselves.  We require defpkg forms
@@ -45,99 +45,97 @@
 ; topics has some correspondence with what one may want to export from ACL2,
 ; but for now we provide this utility only as a comment.
 
-#||
-(verify-termination symbol-class)
-
-(defun member-eq-t (a lst)
-  (or (eq lst t)
-      (member-eq a lst)))
-
-(defun filter-topics
-  (in-sections out-sections mode doc-alist wrld acc)
-  (declare (xargs :guard (and (member-eq mode '(:logic :program
-                                                       ;; nil for non-functions
-                                                       nil))
-                              (or (eq in-sections t)
-                                  (symbol-listp in-sections))
-                              (symbol-listp out-sections))
-                  :verify-guards nil))
-
-; Need to compile this.
-
-  (cond
-   ((endp doc-alist) acc)
-   ((and (symbolp (caar doc-alist))
-         (not (equal (symbol-package-name (caar doc-alist)) "ACL2-PC"))
-         (member-eq-t (cadr (car doc-alist)) in-sections)
-         (not (member-eq (cadr (car doc-alist)) out-sections))
-         (let ((fn-symb-p (function-symbolp (caar doc-alist) wrld)))
-           (cond ((eq mode :logic)
-                  (if fn-symb-p
-                      (eq (fdefun-mode (caar doc-alist) wrld) :logic)
-                    (or (getprop (caar doc-alist) 'macro-body nil 'current-acl2-world wrld)
-                        (getprop (caar doc-alist) 'const nil 'current-acl2-world wrld))))
-                 ((eq mode :program)
-                  ;; really means "all but logic"
-                  (and fn-symb-p
-                       (eq (fdefun-mode (caar doc-alist) wrld) :program)))
-                 (t
-                  ;; topics other than functions, macros, and constants
-                  (not (or (getprop (caar doc-alist) 'macro-body nil 'current-acl2-world wrld)
-                           (getprop (caar doc-alist) 'const nil 'current-acl2-world wrld)
-                           fn-symb-p))))))
-    (filter-topics in-sections out-sections mode (cdr doc-alist) wrld
-                   (cons (caar doc-alist) acc)))
-   (t
-    (filter-topics in-sections out-sections mode (cdr doc-alist) wrld acc))))
-
-(comp 'filter-topics)
-
-; Now consider the following table (`P' is "Programming", `A' is "Arrays").
-; "In" lists doc sections that we may want included, while "Out" lists those
-; to be excluded.  Mode :logic is what we may want to export if we choose to
-; stay in defun-mode :logic; :program is what is left.
-
+; (verify-termination symbol-class)
+; 
+; (defun member-eq-t (a lst)
+;   (or (eq lst t)
+;       (member-eq a lst)))
+; 
+; (defun filter-topics
+;   (in-sections out-sections mode doc-alist wrld acc)
+;   (declare (xargs :guard (and (member-eq mode '(:logic :program
+;                                                        ;; nil for non-functions
+;                                                        nil))
+;                               (or (eq in-sections t)
+;                                   (symbol-listp in-sections))
+;                               (symbol-listp out-sections))
+;                   :verify-guards nil))
+; 
+; ; Need to compile this.
+; 
+;   (cond
+;    ((endp doc-alist) acc)
+;    ((and (symbolp (caar doc-alist))
+;          (not (equal (symbol-package-name (caar doc-alist)) "ACL2-PC"))
+;          (member-eq-t (cadr (car doc-alist)) in-sections)
+;          (not (member-eq (cadr (car doc-alist)) out-sections))
+;          (let ((fn-symb-p (function-symbolp (caar doc-alist) wrld)))
+;            (cond ((eq mode :logic)
+;                   (if fn-symb-p
+;                       (eq (fdefun-mode (caar doc-alist) wrld) :logic)
+;                     (or (getprop (caar doc-alist) 'macro-body nil 'current-acl2-world wrld)
+;                         (getprop (caar doc-alist) 'const nil 'current-acl2-world wrld))))
+;                  ((eq mode :program)
+;                   ;; really means "all but logic"
+;                   (and fn-symb-p
+;                        (eq (fdefun-mode (caar doc-alist) wrld) :program)))
+;                  (t
+;                   ;; topics other than functions, macros, and constants
+;                   (not (or (getprop (caar doc-alist) 'macro-body nil 'current-acl2-world wrld)
+;                            (getprop (caar doc-alist) 'const nil 'current-acl2-world wrld)
+;                            fn-symb-p))))))
+;     (filter-topics in-sections out-sections mode (cdr doc-alist) wrld
+;                    (cons (caar doc-alist) acc)))
+;    (t
+;     (filter-topics in-sections out-sections mode (cdr doc-alist) wrld acc))))
+; 
+; (comp 'filter-topics)
+; 
+; ; Now consider the following table (`P' is "Programming", `A' is "Arrays").
+; ; "In" lists doc sections that we may want included, while "Out" lists those
+; ; to be excluded.  Mode :logic is what we may want to export if we choose to
+; ; stay in defun-mode :logic; :program is what is left.
+; 
+; ; In    Out  Mode
+; ; P,A   ()   :logic
+; ; P,A   ()   :program
+; ; P,A   ()   nil
+; ; T     P,A  :logic
+; ; T     P,A  :program
+; ; T     P,A  nil
+; 
+; Thus we have:
+; 
 ; In    Out  Mode
 ; P,A   ()   :logic
+; (filter-topics '(programming arrays) nil :logic
+;                (global-val 'documentation-alist (w state)) (w state) nil)
+; 
+; In    Out  Mode
 ; P,A   ()   :program
-; P,A   ()   nil
+; (filter-topics '(programming arrays) nil :program
+;                (global-val 'documentation-alist (w state)) (w state) nil)
+; 
+; In    Out  Mode
+; P,A   ()   :logic
+; (filter-topics '(programming arrays) nil nil
+;                (global-val 'documentation-alist (w state)) (w state) nil)
+; 
+; In    Out  Mode
 ; T     P,A  :logic
+; 
+; (filter-topics t '(programming arrays) :logic
+;                (global-val 'documentation-alist (w state)) (w state) nil)
+; 
+; In    Out  Mode
 ; T     P,A  :program
+; (filter-topics t '(programming arrays) :program
+;                (global-val 'documentation-alist (w state)) (w state) nil)
+; 
+; In    Out  Mode
 ; T     P,A  nil
-
-Thus we have:
-
-In    Out  Mode
-P,A   ()   :logic
-(filter-topics '(programming arrays) nil :logic
-               (global-val 'documentation-alist (w state)) (w state) nil)
-
-In    Out  Mode
-P,A   ()   :program
-(filter-topics '(programming arrays) nil :program
-               (global-val 'documentation-alist (w state)) (w state) nil)
-
-In    Out  Mode
-P,A   ()   :logic
-(filter-topics '(programming arrays) nil nil
-               (global-val 'documentation-alist (w state)) (w state) nil)
-
-In    Out  Mode
-T     P,A  :logic
-
-(filter-topics t '(programming arrays) :logic
-               (global-val 'documentation-alist (w state)) (w state) nil)
-
-In    Out  Mode
-T     P,A  :program
-(filter-topics t '(programming arrays) :program
-               (global-val 'documentation-alist (w state)) (w state) nil)
-
-In    Out  Mode
-T     P,A  nil
-(filter-topics t '(programming arrays) nil
-               (global-val 'documentation-alist (w state)) (w state) nil)
-||#
+; (filter-topics t '(programming arrays) nil
+;                (global-val 'documentation-alist (w state)) (w state) nil)
 
 ; The following ``policy'' was used to determine this setting of *acl2-exports*.
 ; First, if the user wishes to program in ACL2, he or she will import
@@ -202,7 +200,7 @@ T     P,A  nil
         ADD-MATCH-FREE-OVERRIDE ADD-NTH-ALIAS
         ADD-OVERRIDE-HINTS ADD-OVERRIDE-HINTS!
         ADD-PAIR ADD-PAIR-PRESERVES-ALL-BOUNDP
-        ADD-RAW-ARITY ADD-TIMERS ADD-TO-SET-EQ
+        ADD-RAW-ARITY ADD-TIMERS ADD-TO-SET ADD-TO-SET-EQ
         ADD-TO-SET-EQL ADD-TO-SET-EQUAL
         ALISTP ALISTP-FORWARD-TO-TRUE-LISTP
         ALL-BOUNDP ALL-BOUNDP-PRESERVES-ASSOC
@@ -294,7 +292,7 @@ T     P,A  nil
         COMPRESS21 COMPRESS211 CONCATENATE
         COND COND-CLAUSESP COND-MACRO
         CONJUGATE CONS CONS-EQUAL CONSP
-        CONSP-ASSOC CONSP-ASSOC-EQ COROLLARY
+        CONSP-ASSOC-EQUAL COROLLARY
         CPU-CORE-COUNT CURRENT-PACKAGE
         CURRENT-THEORY CW CW! CW-GSTACK DECLARE
         DECREMENT-BIG-CLOCK DEFABBREV DEFATTACH
@@ -326,7 +324,8 @@ T     P,A  nil
         DEFPROXY DEFREFINEMENT DEFSTOBJ DEFSTUB DEFTHEORY
         DEFTHM DEFTHMD DEFTTAG DEFUN DEFUN-NX
         DEFUN-SK DEFUND DEFUND-NX DEFUNS
-        DELETE-INCLUDE-BOOK-DIR DELETE-PAIR
+        DELETE-ASSOC DELETE-ASSOC-EQ DELETE-ASSOC-EQUAL
+        DELETE-INCLUDE-BOOK-DIR
         DENOMINATOR DIGIT-CHAR-P DIGIT-TO-CHAR
         DIMENSIONS DISABLE DISABLE-FORCING
         DISABLE-IMMEDIATE-FORCE-MODEP
@@ -343,6 +342,7 @@ T     P,A  nil
         EQLABLE-LISTP-FORWARD-TO-ATOM-LISTP
         EQLABLEP EQLABLEP-RECOG EQUAL
         EQUAL-CHAR-CODE ER ER-PROGN ER-PROGN-FN
+        ER-PROGN@PAR ER-PROGN-FN@PAR
         EVENP EVENS EVENT EVISC-TUPLE
         EXECUTABLE-COUNTERPART-THEORY
         EXIT EXPLODE-ATOM
@@ -354,15 +354,17 @@ T     P,A  nil
         FGETPROP FIFTH FILE-CLOCK FILE-CLOCK-P
         FILE-CLOCK-P-FORWARD-TO-INTEGERP
         FIRST FIRST-N-AC
-        FIX FIX-TRUE-LIST FLOOR FLUSH-COMPRESS
-        FMS FMS! FMT FMT! FMT-TO-COMMENT-WINDOW
-        FMT1 FMT1! FORCE FOURTH FUNCTION-SYMBOLP
+        FIX FIX-TRUE-LIST FLET FLOOR FLUSH-COMPRESS
+        FMS FMS! FMT FMT! FMT-TO-COMMENT-WINDOW FMT1 FMT1!
+        FMS-TO-STRING FMS!-TO-STRING FMT-TO-STRING FMT!-TO-STRING
+        FMT1-TO-STRING FMT1!-TO-STRING
+        FORCE FOURTH FUNCTION-SYMBOLP
         FUNCTION-THEORY GAG-MODE GC$ GENERALIZE
         GET-GLOBAL GET-OUTPUT-STREAM-STRING$ GET-TIMER GET-WORMHOLE-STATUS
         GETENV$ GETPROP GETPROP-DEFAULT GETPROPS
         GETPROPS1 GLOBAL-TABLE GLOBAL-TABLE-CARS
         GLOBAL-TABLE-CARS1 GLOBAL-VAL
-        GOOD-BYE GROUND-ZERO
+        GOOD-ATOM-LISTP GOOD-BYE GROUND-ZERO
         GUARD GUARD-OBLIGATION HARD-ERROR
         HAS-PROPSP HAS-PROPSP1 HEADER HELP HIDE
         HONS-ASSOC-EQUAL HONS-COPY-PERSISTENT
@@ -383,9 +385,9 @@ T     P,A  nil
         INTEGER-STEP INTEGERP INTERN
         INTERN$ INTERN-IN-PACKAGE-OF-SYMBOL
         INTERN-IN-PACKAGE-OF-SYMBOL-SYMBOL-NAME
-        INTERSECTION-EQ
+        INTERSECTION$ INTERSECTION-EQ
         INTERSECTION-EQUAL INTERSECTION-THEORIES
-        INTERSECTP-EQ
+        INTERSECTP INTERSECTP-EQ
         INTERSECTP-EQUAL INVERSE-OF-*
         INVERSE-OF-+ INVISIBLE-FNS-TABLE
         KEYWORD-PACKAGE KEYWORD-VALUE-LISTP
@@ -422,13 +424,13 @@ T     P,A  nil
         MAKE-VAR-LST MAKE-VAR-LST1
         MAKE-WORMHOLE-STATUS MAKUNBOUND-GLOBAL
         MAX MAXIMUM-LENGTH MAY-NEED-SLASHES
-        MBE MBT MEMBER MEMBER-EQ
-        MEMBER-EQUAL MEMBER-SYMBOL-NAME
-        MFC MIN MINIMAL-THEORY MINUSP
+        MBE MBT
+        MEMBER MEMBER-EQ MEMBER-EQUAL
+        MEMBER-SYMBOL-NAME MFC MIN MINIMAL-THEORY MINUSP
         MOD MOD-EXPT MONITOR MONITORED-RUNES
         MORE MORE! MORE-DOC MUST-BE-EQUAL
         MUTUAL-RECURSION MUTUAL-RECURSION-GUARDP
-        MV MV-LET MV-LIST
+        MV MV? MV-LET MV?-LET MV-LIST
         MV-NTH NATP NEEDS-SLASHES NEWLINE
         NFIX NIL NIL-IS-NOT-CIRCULAR NINTH
         NO-DUPLICATESP NO-DUPLICATESP-EQ NO-DUPLICATESP-EQUAL
@@ -440,7 +442,7 @@ T     P,A  nil
         NTH-ALIASES NTH-UPDATE-NTH NTHCDR
         NULL NUMERATOR O-FINP O-FIRST-COEFF
         O-FIRST-EXPT O-INFP O-P O-RST O<
-        O<= O> O>= OBSERVATION ODDP ODDS OK-IF
+        O<= O> O>= OBSERVATION OBSERVATION-CW ODDP ODDS OK-IF
         OOPS OPEN-CHANNEL-LISTP OPEN-CHANNEL1
         OPEN-CHANNEL1-FORWARD-TO-TRUE-LISTP-AND-CONSP
         OPEN-CHANNELS-P OPEN-CHANNELS-P-FORWARD
@@ -461,7 +463,7 @@ T     P,A  nil
         ORDERED-SYMBOL-ALISTP-ADD-PAIR-FORWARD
         ORDERED-SYMBOL-ALISTP-FORWARD-TO-SYMBOL-ALISTP
         ORDERED-SYMBOL-ALISTP-GETPROPS
-        ORDERED-SYMBOL-ALISTP-REMOVE-FIRST-PAIR
+        ORDERED-SYMBOL-ALISTP-DELETE-ASSOC-EQ
         OTHERWISE
         OUR-DIGIT-CHAR-P OVERRIDE-HINTS
         P! PKG-IMPORTS PAIRLIS$ PAIRLIS2 PAND PARGS
@@ -475,15 +477,16 @@ T     P,A  nil
         PR! PREPROCESS PRIN1$ PRIN1-WITH-SLASHES
         PRIN1-WITH-SLASHES1 PRINC$ PRINT-GV
         PRINT-OBJECT$ PRINT-RATIONAL-AS-DECIMAL
-        PRINT-TIMER PROG2$ PROGN PROGN! PROGRAM
+        PRINT-TIMER PROFILE
+        PROG2$ PROGN PROGN! PROGN$ PROGRAM
         PROOF-TREE PROOFS-CO PROPER-CONSP
         PROPS PROVE PSEUDO-TERM-LISTP
         PSEUDO-TERM-LISTP-FORWARD-TO-TRUE-LISTP
         PSEUDO-TERMP PSO PSO! PSOG PSTACK
         PUFF PUFF* PUSH-TIMER PUSH-UNTOUCHABLE
-        PUT-ASSOC-EQ PUT-ASSOC-EQL
+        PUT-ASSOC PUT-ASSOC-EQ PUT-ASSOC-EQL
         PUT-ASSOC-EQUAL PUT-GLOBAL PUTPROP
-        QUIT QUOTE QUOTEP R-EQLABLE-ALISTP
+        QUIT QUOTE QUOTEP R-SYMBOL-ALISTP R-EQLABLE-ALISTP
         RANDOM$ RASSOC RASSOC-EQ RASSOC-EQUAL
         RATIO RATIONAL RATIONAL-IMPLIES1
         RATIONAL-IMPLIES2 RATIONAL-LISTP
@@ -515,9 +518,9 @@ T     P,A  nil
         REMOVE-BINOP REMOVE-CUSTOM-KEYWORD-HINT
         REMOVE-DEFAULT-HINTS
         REMOVE-DEFAULT-HINTS!
-        REMOVE-DUPLICATES REMOVE-DUPLICATES-EQL
+        REMOVE-DUPLICATES REMOVE-DUPLICATES-EQ REMOVE-DUPLICATES-EQL
         REMOVE-DUPLICATES-EQUAL
-        REMOVE-EQ REMOVE-EQUAL REMOVE-FIRST-PAIR
+        REMOVE-EQ REMOVE-EQUAL
         REMOVE-INVISIBLE-FNS REMOVE-MACRO-ALIAS
         REMOVE-NTH-ALIAS REMOVE-OVERRIDE-HINTS
         REMOVE-OVERRIDE-HINTS!
@@ -539,7 +542,7 @@ T     P,A  nil
         SET-DEFAULT-BACKCHAIN-LIMIT
         SET-DEFAULT-HINTS SET-DEFAULT-HINTS!
         SET-DEFERRED-TTAG-NOTES
-        SET-DIFFERENCE-EQ SET-DIFFERENCE-EQUAL
+        SET-DIFFERENCE-EQ SET-DIFFERENCE-EQUAL SET-DIFFERENCE$
         SET-DIFFERENCE-THEORIES
         SET-ENFORCE-REDUNDANCY
         SET-EQUALP-EQUAL SET-EVISC-TUPLE
@@ -573,11 +576,15 @@ T     P,A  nil
         SET-RAW-MODE-ON! SET-RAW-PROOF-FORMAT
         SET-REWRITE-STACK-LIMIT
         SET-RULER-EXTENDERS
+        SET-RW-CACHE-STATE SET-RW-CACHE-STATE!
         SET-SAVED-OUTPUT SET-STATE-OK
+        SET-PROVER-STEP-LIMIT
         SET-TAINTED-OK SET-TAINTED-OKP
         SET-TIMER SET-TRACE-EVISC-TUPLE
         SET-VERIFY-GUARDS-EAGERNESS
         SET-W SET-WELL-FOUNDED-RELATION
+        SET-WATERFALL-PARALLELISM
+        SET-WATERFALL-PRINTING
         SET-WORMHOLE-DATA
         SET-WORMHOLE-ENTRY-CODE SET-WRITE-ACL2X SETENV$ SEVENTH
         SGETPROP SHOW-ACCUMULATED-PERSISTENCE
@@ -585,7 +592,7 @@ T     P,A  nil
         SHOW-FC-CRITERIA SHRINK-32-BIT-INTEGER-STACK
         SHRINK-T-STACK
         SIGNED-BYTE SIGNUM SIMPLIFY
-        SIXTH SKIP-PROOFS SOME-SLASHABLE
+        SIXTH SKIP-PROOFS SOME-SLASHABLE SPEC-MV-LET
         STABLE-UNDER-SIMPLIFICATIONP
         STANDARD-CHAR STANDARD-CHAR-LISTP
         STANDARD-CHAR-LISTP-APPEND
@@ -602,6 +609,7 @@ T     P,A  nil
         STATE-P1 STATE-P1-FORWARD
         STATE-P1-UPDATE-MAIN-TIMER
         STATE-P1-UPDATE-NTH-2-WORLD
+        STEP-LIMIT
         STOP-PROOF-TREE
         STRING STRING-APPEND STRING-APPEND-LST
         STRING-DOWNCASE STRING-DOWNCASE1
@@ -616,7 +624,7 @@ T     P,A  nil
         STRING<= STRING> STRING>=
         STRINGP STRINGP-SYMBOL-PACKAGE-NAME
         STRIP-CARS STRIP-CDRS SUBLIS
-        SUBSEQ SUBSEQ-LIST SUBSETP SUBSETP-EQUAL
+        SUBSEQ SUBSEQ-LIST SUBSETP SUBSETP-EQ SUBSETP-EQUAL
         SUBST SUBSTITUTE SUBSTITUTE-AC SUMMARY
         SYMBOL SYMBOL-< SYMBOL-<-ASYMMETRIC
         SYMBOL-<-IRREFLEXIVE SYMBOL-<-TRANSITIVE
@@ -640,7 +648,7 @@ T     P,A  nil
         TOP-LEVEL TRACE! TRACE$ TRANS
         TRANS! TRANS1 TRICHOTOMY TRUE-LIST-LISTP
         TRUE-LIST-LISTP-FORWARD-TO-TRUE-LISTP
-        TRUE-LIST-LISTP-FORWARD-TO-TRUE-LISTP-ASSOC-EQ
+        TRUE-LIST-LISTP-FORWARD-TO-TRUE-LISTP-ASSOC-EQUAL
         TRUE-LISTP
         TRUE-LISTP-CADR-ASSOC-EQ-FOR-OPEN-CHANNELS-P
         TRUE-LISTP-UPDATE-NTH
@@ -649,7 +657,7 @@ T     P,A  nil
         U UBT UBT!
         UBT-PREHISTORY UBU UBU! UNARY-- UNARY-/
         UNARY-FUNCTION-SYMBOL-LISTP UNICITY-OF-0
-        UNICITY-OF-1 UNION-EQ UNION-EQUAL
+        UNICITY-OF-1 UNION$ UNION-EQ UNION-EQUAL
         UNION-THEORIES UNIVERSAL-THEORY
         UNMONITOR UNSAVE UNSIGNED-BYTE UNTRACE$
         UNTRANSLATE UPDATE-32-BIT-INTEGER-STACK
@@ -670,8 +678,10 @@ T     P,A  nil
         VALUE-TRIPLE VERBOSE-PSTACK VERIFY
         VERIFY-GUARDS VERIFY-GUARDS-FORMULA
         VERIFY-TERMINATION W WALKABOUT
-        WARNING! WET WITH-GUARD-CHECKING
-        WITH-LIVE-STATE WITH-OUTPUT
+        WARNING! WATERFALL-PARALLELISM WATERFALL-PRINTING 
+        WET WITH-GUARD-CHECKING
+        WITH-LIVE-STATE WITH-LOCAL-STATE WITH-LOCAL-STOBJ WITH-OUTPUT
+        WITH-PROVER-STEP-LIMIT WITH-PROVER-STEP-LIMIT!
         WITH-PROVER-TIME-LIMIT WITHOUT-EVISC
         WORLD WORMHOLE WORMHOLE-DATA
         WORMHOLE-ENTRY-CODE WORMHOLE-EVAL
