@@ -1,4 +1,4 @@
-; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 was produced by modifying ACL2 Version 1.9, Copyright
@@ -19,7 +19,7 @@
 ; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
-; email:       Kaufmann@aus.edsr.eds.com   and Moore@cs.utexas.edu
+; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
 ; Department of Computer Sciences
 ; University of Texas at Austin
 ; Austin, TX 78712-1188 U.S.A.
@@ -607,14 +607,15 @@ a TeXinfo file to be used to create an info file.  See also
                       (princ$ ") " channel state))
             state)
 	  (acl2::print-doc-string-part 0 str ""
-				       (texinfo-doc-markup-table state)
-				       *texinfo-doc-char-subst-table*
+                                       (texinfo-doc-markup-table state)
+                                       *texinfo-doc-char-subst-table*
                                        (acl2::doc-fmt-alist state)
                                        channel
-				       name
+                                       name
                                        nil
-                                       nil ; undocumented-file
-				       state)))
+                                       nil  ; undocumented-file
+                                       nil  ; vp
+                                       state)))
 
 (defun write-doc-menu1 (lst doc-alist tex-only-flg channel state)
   ;; lst is a list of names
@@ -684,38 +685,33 @@ a TeXinfo file to be used to create an info file.  See also
                                 channel
                                 (car doc-tuple)
                                 nil
-                                nil ; undocumented-file
+                                nil  ; undocumented-file
+                                nil  ; vp
                                 state)
    (princ$ "@end format" channel state)
    (newline channel state)
    (acl2::print-doc-string-part 1 (cadddr doc-tuple) ""
-				(texinfo-doc-markup-table state)
-				*texinfo-doc-char-subst-table*
+                                (texinfo-doc-markup-table state)
+                                *texinfo-doc-char-subst-table*
                                 (acl2::doc-fmt-alist state)
                                 channel
-				(car doc-tuple)
+                                (car doc-tuple)
                                 nil
-                                nil ; undocumented-file
-				state)
+                                nil  ; undocumented-file
+                                nil  ; vp
+                                state)
    (newline channel state)
    (write-doc-menu doc-tuple doc-alist tex-only-flg channel state)
-   (let* ((str (cadddr doc-tuple))
-          (k (acl2::scan-to-doc-string-part 2 str))
-          (maximum (length str)))
-     (acl2::print-doc-string-part1 str
-				   k
-				   maximum
-				   (acl2::get-doc-string-de-indent str)
-				   ""
-				   (texinfo-doc-markup-table state)
-				   *texinfo-doc-char-subst-table*
-                                   (acl2::doc-fmt-alist state)
-				   channel
-                                   (car doc-tuple)
-				   state
-				   t
-                                   nil ; undocumented-file
-                                   ))
+   (acl2::print-doc-string-part 2 (cadddr doc-tuple) ""
+                                (texinfo-doc-markup-table state)
+                                *texinfo-doc-char-subst-table*
+                                (acl2::doc-fmt-alist state)
+                                channel
+                                (car doc-tuple)
+                                t
+                                nil  ; undocumented-file
+                                nil  ; vp
+                                state)
    (newline channel state)))
 
 (mutual-recursion
@@ -994,8 +990,8 @@ a TeXinfo file to be used to create an info file.  See also
    (my-state-global-let*
     ((acl2::bar-sep-p t) ; print :: as || in node names
      (acl2::write-for-read t)
-     (acl2::fmt-soft-right-margin 100000)
-     (acl2::fmt-hard-right-margin 200000)
+     (acl2::fmt-soft-right-margin 100000 acl2::set-fmt-soft-right-margin)
+     (acl2::fmt-hard-right-margin 200000 acl2::set-fmt-hard-right-margin)
      (acl2::doc-fmt-alist
       (texinfo-fmt-alist
        (acl2::global-val 'acl2::documentation-alist (acl2::w state))
