@@ -1,4 +1,4 @@
-; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -726,10 +726,12 @@ beginning of the string that was processed."
 (define-key ctl-t-keymap "\C-F" 'bury-buffer)
 
 ;; Make some functions' indentation behave as for defun.
-(put 'case       'lisp-indent-function 'defun)
-(put 'case!      'lisp-indent-function 'defun)
-(put 'case-match 'lisp-indent-function 'defun)
-(put 'dolist     'lisp-indent-function 'defun)
+(put 'case         'lisp-indent-function 'defun)
+(put 'case!        'lisp-indent-function 'defun)
+(put 'case-match   'lisp-indent-function 'defun)
+(put 'dolist       'lisp-indent-function 'defun)
+(put 'er@par       'lisp-indent-function 'defun)
+(put 'warning$@par 'lisp-indent-function 'defun)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; ACL2 proof-tree support
@@ -914,6 +916,8 @@ of the current s-expression in the enclosing list"
 ; Turn off display-time with:
 ;   (display-time-mode) in emacs
 ;   (display-time-stop) in xemacs
+; Needed for displaying day and date in addition to time:
+; (setq display-time-day-and-date t)
 
 ; Disable commands that we do not want to execute by mistake:
 (put 'shell-resync-dirs 'disabled t)
