@@ -1,4 +1,4 @@
-; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -21,9 +21,9 @@
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
-; Department of Computer Sciences
+; Department of Computer Science
 ; University of Texas at Austin
-; Austin, TX 78712-1188 U.S.A.
+; Austin, TX 78701 U.S.A.
 
 ; hons.lisp -- Logical definitions for hash cons and fast alists.  Note that
 ; the memoization and watch functionality previously provided by this file have
@@ -45,6 +45,9 @@
 Normed objects are ACL2 Objects that are \"canonical\" or \"unique\" in a
 certain sense.~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 In Common Lisp, we can tell whether an ACL2 object is ~st[normed] or not, but
 there is no way for an ordinary ACL2 function to see whether an object is
 normed.  Hence, whether or not an object is normed is an implementation-level
@@ -108,6 +111,9 @@ to occasionally, manually clear out these Hons Space tables with
 
 ~c[(hons-copy x)] returns a ~il[normed] object that is equal to X.~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 In the logic, ~c[hons-copy] is just the identity function; we leave it enabled
 and would think it odd to ever prove a theorem about it.
 
@@ -145,6 +151,9 @@ object.~/"
 ~c[(hons-copy-persistent x)] returns a ~il[normed] object that is equal to X
 and which will be re-normed after any calls to ~ilc[hons-clear].~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically ~c[hons-copy-persistent] is the identity; we leave it enabled and
 would think it odd to ever prove a theorem about it.
 
@@ -164,6 +173,9 @@ out more efficiently than, say, an ordinary ~ilc[hons-copy].~/~/"
 
 ~c[(hons x y)] returns a ~il[normed] object equal to ~c[(cons x y)].~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 In the logic, ~c[hons] is just ~ilc[cons]; we leave it enabled and would think
 it odd to ever prove a theorem about it.
 
@@ -194,6 +206,9 @@ Generally speaking, these extra operations make ~c[hons] much slower than
 ~c[(hons-equal x y)] is a recursive equality check that optimizes when parts of
 its arguments are ~il[normed].~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 In the logic, ~c[hons-equal] is just ~ilc[equal]; we leave it enabled and would
 think it odd to ever prove a theorem about it.
 
@@ -218,6 +233,9 @@ are normed."
 ~c[(hons-equal-lite x y)] is a non-recursive equality check that optimizes if
 its arguments are ~il[normed].~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 In the logic, ~c[hons-equal-lite] is just ~ilc[equal]; we leave it enabled and
 would think it odd to ever prove a theorem about it.
 
@@ -241,6 +259,9 @@ whether the subtrees of x and y are normed."
 ~c[(hons-clear gc)] is a drastic garbage collection mechanism that clears out
 the underlying Hons Space.~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically, ~c[hons-clear] just returns nil; we leave it enabled and would think
 it odd to ever prove a theorem about it.
 
@@ -272,6 +293,9 @@ discipline."
 ~c[(hons-wash)] is like ~ilc[gc$] but can also garbage collect ~il[normed]
 objects (CCL Only).~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically, ~c[hons-wash] just returns nil; we leave it enabled and would think
 it odd to ever prove a theorem about it.
 
@@ -303,6 +327,9 @@ discipline."
 ~c[(hons-summary)] prints basic information about the sizes of the tables in
 the current Hons Space.~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically, ~c[hons-summary] just returns nil; we leave it enabled and would
 think it odd to ever prove a theorem about it.
 
@@ -325,6 +352,9 @@ your particular computation; see ~ilc[hons-resize]."
 ~c[(hons-resize ...)] can be used to manually adjust the sizes of the hash
 tables that govern which ACL2 Objects are considered ~il[normed].~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 General form:
 ~bv[]
  (hons-resize [:str-ht size]
@@ -405,7 +435,10 @@ soon after a ~ilc[hons-clear]."
 (defdoc fast-alists
   ":Doc-Section Hons-and-Memoization
 
-Alists with hidden hash tables for faster execution.~/
+alists with hidden hash tables for faster execution~/
+
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
 
 The implementation of fast alists is, in many ways, similar to that of ACL2
 arrays.  Logically, ~ilc[hons-acons] is just like ~c[acons], and
@@ -452,6 +485,9 @@ that will no longer be used.")
 ~c[(hons-assoc-equal key alist)] is ~st[not fast]; it serves as the logical
 definition for ~ilc[hons-get].~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 The definition of ~c[hons-assoc-equal] is similar to that of ~ilc[assoc-equal],
 except that (1) it does not require ~ilc[alistp] as a guard, and (2) it \"skips
 over\" any non-conses when its alist argument is malformed.~/
@@ -471,7 +507,10 @@ discipline warnings that might arise from execution during theorem proving."
 (defdoc slow-alist-warning
   ":Doc-Section Hons-and-Memoization
 
-Warnings issued when ~ilc[fast-alists] are used inefficiently.~/
+warnings issued when ~ilc[fast-alists] are used inefficiently~/
+
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
 
 Obtaining hash-table performance from ~ilc[hons-get] requires one to follow
 a certain discipline.  If this discipline is violated, you may see a \"slow
@@ -514,6 +553,9 @@ and books, wrapped in ~ilc[local], and so on.")
 ~c[(hons-get key alist)] is the efficient lookup operation for
 ~il[fast-alists].~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically, ~c[hons-get] is just an alias for ~ilc[hons-assoc-equal]; we
 typically leave it enabled and prefer to reason about ~c[hons-assoc-equal]
 instead.  One benefit of this approach is that it helps to avoids \"false\"
@@ -533,6 +575,9 @@ becomes a ~c[gethash] operation on the hidden hash table.~/~/"
 ~c[(hons-acons key val alist)] is the main way to create or extend
 ~il[fast-alists].~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically, ~c[hons-acons] is like ~ilc[acons] except that its guard does not
 require ~ilc[alistp]; we leave it enabled and would think it odd to ever prove
 a theorem about it.
@@ -559,8 +604,9 @@ accommodate them, and this resizing has some runtime cost.
 
 When a natural number is used as a fast alist's name, we interpret it as a size
 hint.  For example, ~c[(hons-acons 'foo 'bar 1000)] instructs us to use 1000 as
-the initial size for this hash table and binds 'foo to 'bar.  We ignore size
-hints that request fewer than 60 elements.
+the initial size for this hash table and binds 'foo to 'bar.  The resulting
+table should not need to be resized until more than 1000 elements are added.
+We ignore size hints that request fewer than 60 elements.
 
 Because of hash collisions, hash tables typically need to have a larger size
 than the actual number of elements they contain.  The hash tables for fast
@@ -592,9 +638,12 @@ and size components, but for now we have not implemented this capability."
 (defn hons-acons! (key val alist)
   ":Doc-Section Hons-and-Memoization
 
-~c[(hons-acons! key val alist) is an alternative to ~ilc[hons-acons] that
+~c[(hons-acons! key val alist)] is an alternative to ~ilc[hons-acons] that
 produces ~il[normed], fast alists.~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically, ~c[hons-acons!] is like ~ilc[acons] except that its guard does not
 require ~ilc[alistp]; we leave it enabled and would think it odd to ever prove
 a theorem about it.
@@ -656,6 +705,9 @@ and this new object would not be associated with the fast alist's hash table."
 ~c[(hons-shrink-alist alist ans)] can be used to eliminate \"shadowed pairs\"
 from an alist or to copy ~il[fast-alists].~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically, ~c[hons-shrink-alist] is defined as follows:
 
 ~bv[]
@@ -703,6 +755,9 @@ similar to ~c[(append ans alist)], but in a different order.~/~/"
 ~c[(hons-shrink-alist! alist ans)] is an alternative to ~ilc[hons-shrink-alist]
 that produces a ~il[normed] result.~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically this function is just ~c[hons-shrink-alist]; we leave it enabled and
 would think it odd to ever prove a theorem about it.
 
@@ -720,6 +775,9 @@ understand the drawbacks discussed in ~ilc[hons-acons!].~/~/"
 
 ~c[(fast-alist-len alist)] counts the number of unique keys in a fast alist.~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically this function counts how many elements would remain in the alist were
 we to shrink it with ~ilc[hons-shrink-alist].
 
@@ -738,6 +796,9 @@ and doesn't require us to actually shrink the alist.~/~/"
 ~c[(fast-alist-free alist)] throws away the hash table associated with a fast
 alist.~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically, this function is the identity; we leave it enabled and would think
 it odd to ever prove a theorem about it.
 
@@ -763,6 +824,9 @@ that have already been freed and objects which are not alists at all."
 ~c[(fast-alist-summary)] prints some basic statistics about any current fast
 alists.~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Logically, ~c[fast-alist-summary] just returns nil; we leave it enabled and
 would think it odd to ever prove a theorem about it.
 
@@ -777,7 +841,11 @@ should have been freed with ~ilc[fast-alist-free]~/~/"
   ":Doc-Section Hons-and-Memoization
 
 ~c[(cons-subtrees x nil)] builds a fast alist that associates each subtree
-of X with T, without duplication.~/~/~/"
+of X with T, without duplication.~/
+
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+~/~/"
 
   (cond ((atom x)
          al)
@@ -795,6 +863,9 @@ of X with T, without duplication.~/~/~/"
 ~c[(number-subtrees x)] returns the number of distinct subtrees of X, in the
 sense of ~ilc[equal]~/
 
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 In the logic, ~c[number-subtrees] is defined as the length of
 ~ilc[cons-subtrees].
 
@@ -808,8 +879,13 @@ count the number of unique conses in X using an EQ hash table.~/~/"
 (defn clear-hash-tables ()
   ":Doc-Section Hons-and-Memoization
 
+deprecated feature~/
+
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
 Deprecated.  Calls ~ilc[clear-memoize-tables] and then ~ilc[hons-clear] or
-~ilc[hons-wash], whichever makes sense for the underlying Common Lisp.~/~/~/"
+~ilc[hons-wash], whichever makes sense for the underlying Common Lisp.~/~/"
 
   ;; Has an under-the-hood implementation
   nil)
@@ -817,7 +893,12 @@ Deprecated.  Calls ~ilc[clear-memoize-tables] and then ~ilc[hons-clear] or
 (defn flush-hons-get-hash-table-link (alist)
   ":Doc-Section Hons-and-Memoization
 
-Deprecated.  Alias for ~ilc[fast-alist-free]~/~/~/"
+deprecated feature~/
+
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
+Deprecated.  Alias for ~ilc[fast-alist-free]~/~/"
 
   (fast-alist-free alist))
 
