@@ -1,4 +1,4 @@
-; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -21,9 +21,9 @@
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
-; Department of Computer Sciences
+; Department of Computer Science
 ; University of Texas at Austin
-; Austin, TX 78712-1188 U.S.A.
+; Austin, TX 78701 U.S.A.
 
 ; memoize.lisp -- Logical definitions for memoization functions, only
 ; to be included in the experimental HONS version of ACL2.
@@ -376,18 +376,21 @@
   ~/")
 
 (defun hons-enabledp (state)
-  (declare (xargs :guard (and (state-p state)
-                              (boundp-global 'hons-enabled state))))
-  (f-get-global 'hons-enabled state))
+  (declare (xargs :guard (state-p state)))
+  (global-val 'hons-enabled (w state)))
 
 #+(or acl2-loop-only (not hons))
 (defn clear-memoize-table (fn)
 
   ":Doc-Section Hons-and-Memoization
 
-  Forget values remembered for the function FN~/
+  forget values remembered for the given function~/
 
-  A logical no-op.  The values memoized for fn are forgotten.~/~/"
+  This ~il[documentation] topic relates to the experimental extension of ACL2
+  supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
+  This function returns its argument, ~c[fn], unchanged.  The values memoized
+  for ~c[fn] are forgotten.~/~/"
 
   fn)
 
@@ -396,15 +399,59 @@
 
   ":Doc-Section Hons-and-Memoization
 
-  Forget values remembered for all the memoized functions.~/
+  forget values remembered for all the memoized functions~/
 
-  A logical no-op.  All memoized values are forgotten.
+  This ~il[documentation] topic relates to the experimental extension of ACL2
+  supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
 
-  Returns NIL.  Invokes CLEAR-MEMOIZE-TABLE for
-  each memoized function.~/~/"
+  ~c[Clear-memoize-tables] is a logical no-op.  All memoized values are
+  forgotten.  It returns ~c[nil], invoking ~ilc[clear-memoize-table] for each
+  memoized function.~/~/"
 
   nil)
 
+#+(or acl2-loop-only (not hons))
+(defn memoize-summary ()
+  ":Doc-Section Hons-and-Memoization
+
+  display all collected profiling and memoization table info~/
+
+  This ~il[documentation] topic relates to the experimental extension of ACL2
+  supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
+  Logically, this function just returns ~c[nil], but it displays profiling and
+  memoization table information.  The profiling statistics may be cleared with
+  ~c[(]~ilc[clear-memoize-statistics]~c[)].~/~/"
+
+  nil)
+
+#+(or acl2-loop-only (not hons))
+(defn clear-memoize-statistics ()
+  ":Doc-Section Hons-and-Memoization
+
+  clears all profiling info displayed by ~c[(]~ilc[memoize-summary]~c[)]~/
+
+  This ~il[documentation] topic relates to the experimental extension of ACL2
+  supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
+  Logically, this function just returns ~c[nil].  It clears all profiling info
+  displayed by ~c[(]~ilc[memoize-summary]~c[)]~/~/"
+
+  nil)
+
+(defmacro memsum ()
+  ":Doc-Section Hons-and-Memoization
+
+  display all collected profiling and memoization info~/
+
+  This ~il[documentation] topic relates to the experimental extension of ACL2
+  supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
+  This macro is abbreviation for ~ilc[memoize-summary].  Logically, it just
+  returns ~c[nil].~/~/"
+
+  '(memoize-summary))
+
 ; The macros MEMOIZE-LET, MEMOIZE-ON, and MEMOIZE-OFF typically cause
 ; "under the hood" effects that, though not changing the semantics of
 ; what ACL2 returns, may affect the speed and/or space utilization of
@@ -439,7 +486,9 @@
     flush-hons-get-hash-table-link
     ;; from memoize.lisp
     clear-memoize-table
-    clear-memoize-tables))
+    clear-memoize-tables
+    memoize-summary
+    clear-memoize-statistics))
 
 (defconst *hons-primitives*
   ;; hons-related macros and primitive fns
@@ -449,26 +498,31 @@
             memoize
             unmemoize
             memoize-on
-            memoize-off)
+            memoize-off
+            memsum)
           *hons-primitive-fns*))
 
 (defconst *mht-default-size* 60)
 
 (defun memoize-form (fn condition condition-p condition-fn hints otf-flg inline
-                        trace commutative forget memo-table-init-size)
+                        trace commutative forget memo-table-init-size aokp
+                        ideal-okp)
   (declare (xargs :guard t))
   (cond
    ((and condition-fn (null condition-p))
     `(progn (table memoize-table
                    (deref-macro-name ,fn (macro-aliases world))
-                   (list (cons :condition-fn ,condition-fn)
-                         (cons :inline ,inline)
-                         (cons :trace ,trace)
-                         (cons :commutative ,commutative)
-                         (cons :forget ,forget)
-                         (cons :memo-table-init-size
-                               ,(or memo-table-init-size
-                                    *mht-default-size*))))
+                   (list* (cons :condition-fn ,condition-fn)
+                          (cons :inline ,inline)
+                          (cons :trace ,trace)
+                          (cons :commutative ,commutative)
+                          (cons :forget ,forget)
+                          (cons :memo-table-init-size
+                                ,(or memo-table-init-size
+                                     *mht-default-size*))
+                          (cons :aokp ,aokp)
+                          (and (not (eq ,ideal-okp :default))
+                               (list (cons :ideal-okp ,ideal-okp)))))
             (value-triple (deref-macro-name
                            ,fn
                            (macro-aliases (w state))))))
@@ -496,7 +550,9 @@
              (trace ,trace)
              (commutative ,commutative)
              (forget ,forget)
-             (memo-table-init-size ,memo-table-init-size))
+             (memo-table-init-size ,memo-table-init-size)
+             (aokp ,aokp)
+             (ideal-okp ,ideal-okp))
         (cond ((not (and
                      (symbolp fn)
                      (not (eq t formals))
@@ -551,25 +607,31 @@
                                         `(:otf-flg ,otf-flg)))
                   (table memoize-table
                          ',fn
-                         (list (cons :condition-fn ',condition-fn)
-                               (cons :inline ',inline)
-                               (cons :trace ',trace)
-                               (cons :commutative ',commutative)
-                               (cons :forget ',forget)
-                               (cons :memo-table-init-size
-                                     (or ,memo-table-init-size
-                                         *mht-default-size*))))
+                         (list* (cons :condition-fn ',condition-fn)
+                                (cons :inline ',inline)
+                                (cons :trace ',trace)
+                                (cons :commutative ',commutative)
+                                (cons :forget ',forget)
+                                (cons :memo-table-init-size
+                                      (or ,memo-table-init-size
+                                          *mht-default-size*))
+                                (cons :aokp ',aokp)
+                                (and (not (eq ',ideal-okp :default))
+                                     (list (cons :ideal-okp ',ideal-okp)))))
                   (value-triple ',fn)))))))
    (t `(progn (table memoize-table
                      (deref-macro-name ,fn (macro-aliases world))
-                     (list (cons :condition-fn t)
-                           (cons :inline ,inline)
-                           (cons :trace ,trace)
-                           (cons :commutative ,commutative)
-                           (cons :forget ,forget)
-                           (cons :memo-table-init-size
-                              (or ,memo-table-init-size
-                                    *mht-default-size*))))
+                     (list* (cons :condition-fn t)
+                            (cons :inline ,inline)
+                            (cons :trace ,trace)
+                            (cons :commutative ,commutative)
+                            (cons :forget ,forget)
+                            (cons :memo-table-init-size
+                                  (or ,memo-table-init-size
+                                      *mht-default-size*))
+                            (cons :aokp ',aokp)
+                            (and (not (eq ',ideal-okp :default))
+                                 (list (cons :ideal-okp ',ideal-okp)))))
               (value-triple (deref-macro-name
                              ,fn
                              (macro-aliases (w state))))))))
@@ -583,6 +645,8 @@
                       commutative
                       forget
                       memo-table-init-size
+                      aokp
+                      (ideal-okp ':default)
                       (verbose 't))
 
 ; WARNING: If you add a new argument here, consider making corresponding
@@ -598,10 +662,10 @@
 
   turn on memoization for one function~/
 
-  This documentation topic relates to an experimental extension of
-  ACL2 under development by Bob Boyer and Warren Hunt.
-  ~l[hons-and-memoization] for a general discussion of memoization and
-  the related features of hash consing and applicative hash tables.
+  This ~il[documentation] topic relates to an experimental extension of ACL2
+  under development by Bob Boyer and Warren Hunt.  ~l[hons-and-memoization] for
+  a general discussion of memoization and the related features of hash consing
+  and applicative hash tables.
 
   ~bv[]
   Examples:
@@ -614,7 +678,10 @@
                                       ;   a call of the given function
   (memoize 'foo :inline nil)          ; do not inline the definition
                                       ;   of foo
-  (memoize 'foo :recursive nil)       ; as above, i.e. :inline nil~/
+  (memoize 'foo :recursive nil)       ; as above, i.e. :inline nil
+  (memoize 'foo :aokp t)              ; attachments OK for stored results
+  (memoize 'foo :ideal-okp t)         ; memoize even if foo is in :logic mode
+                                      ;   but has not been guard-verified~/
 
   General Form:
   (memoize fn                         ; memoizes fn and returns fn
@@ -629,6 +696,8 @@
            :commutative  t/lemma-name ; optional (default nil)
            :forget       t/nil        ; optional (default nil)
            :memo-table-init-size size ; optional (default *mht-default-size*)
+           :aokp         t/nil        ; optional (default nil)
+           :ideal-okp    t/:warn/nil  ; optional (default nil)
            :verbose      t/nil        ; optional (default t)
            )
   ~ev[]
@@ -749,6 +818,38 @@
   If ~c[:memo-table-init-size] is supplied, then it should be a positive
   integer specifying the initial size of an associated hash table.
 
+  Argument ~c[:aokp] is relevant only when attachments are used; ~pl[defattach]
+  for background on attachments.  When ~c[:aokp] is ~c[nil], the default,
+  computed values are not stored when an attachment was used, or even when an
+  attachment may have been used because a function was called that had been
+  memoized using ~c[:aokp t].  Otherwise, computed values are always stored,
+  but saved values are not used except when attachments are allowed.  To
+  summarize:
+  ~bf[]
+    aokp=nil (default): ``Pure'', i.e., values do not depend on attachments
+    - Fetch: always legal
+    - Store: only store resulting value when attachments were not used
+
+    aokp=t: ``Impure'', i.e., values may depend on attachments
+    - Fetch: only legal when attachments are allowed (e.g., not during proofs)
+    - Store: always legal
+  ~ef[]
+
+  If ~c[:ideal-okp] is supplied and not ~c[nil], then it is permitted to
+  memoize an ``ideal-mode'' function: one in ~c[:]~ilc[logic] mode whose
+  ~il[guard]s have not been verified.  In general, it is ill-advised to memoize
+  an ideal-mode function, because its calls are typically evaluated ``in the
+  logic'' without calling a memoized ``raw Lisp'' version of the function.
+  However, if the function is called by a ~c[:]~ilc[program] mode function,
+  evaluation can transfer to raw Lisp before reaching the call of the memoized
+  function, in which case memoization will take place.  For such situations you
+  can provide value ~c[:warn] or ~c[t] for keyword parameter ~c[:ideal-okp].
+  Both of these values allow memoization of ideal-mode functions, but if
+  ~c[:warn] is supplied then a warning will take place.  Note that you may set
+  the key ~c[:memoize-ideal-okp] of the ~ilc[acl2-defaults-table] to value
+  ~c[t] or ~c[:warn] to change the default, but if parameter ~c[:ideal-okp] is
+  supplied, the ~ilc[acl2-defaults-table] value is ignored.
+
   If ~c[:verbose] is supplied, it should either be ~c[nil], which will inhibit
   proof, event, and summary output (~pl[with-output]), or else ~c[t] (the
   default), which does not inhibit output.  If the output baffles you, try
@@ -766,7 +867,8 @@
   (declare (xargs :guard t)
            (ignorable condition-p condition condition-fn hints otf-flg inline
                       inline-supplied-p recursive recursive-supplied-p
-                      trace commutative forget memo-table-init-size verbose))
+                      trace commutative forget memo-table-init-size aokp
+                      ideal-okp verbose))
   #-acl2-loop-only
   `(progn (when (eql *ld-level* 0)
 
@@ -823,10 +925,12 @@
                                ',trace
                                (kwote commutative)
                                ',forget
-                               ',memo-table-init-size)))))
+                               ',memo-table-init-size
+                               ',aokp
+                               ',ideal-okp)))))
            (t (memoize-form fn condition condition-p condition-fn
-                            hints otf-flg inline trace
-                            commutative forget memo-table-init-size)))))
+                            hints otf-flg inline trace commutative forget
+                            memo-table-init-size aokp ideal-okp)))))
     (cond (verbose form)
           (t `(with-output
                :off (summary prove event)
@@ -877,6 +981,58 @@
            (value-triple
             (deref-macro-name ,fn (macro-aliases (w state)))))))
 
+(defmacro profile (fn &rest r)
+
+  ":Doc-Section Events
+
+  turn on profiling for one function~/
+
+  NOTE: An alternative to this utility, which has a lot less functionality but
+  doesn't depend on the experimental extension of ACL2 mentioned just below,
+  may be found in distributed book ~c[books/misc/profiling.lisp].
+
+  This documentation topic relates to an experimental extension of ACL2 under
+  development by Bob Boyer and Warren Hunt.  ~l[hons-and-memoization] for a
+  general discussion of memoization, on which this ~c[profile] utility is
+  built, and the related features of hash consing and applicative hash tables.
+
+  ~c[Profile] can be useful in Common Lisp debugging and performance analysis,
+  including examining the behavior of ACL2 functions.
+
+  ~bv[]
+  Example:
+  (profile 'fn)       ; keep count of the calls of fn
+  (profile 'fn        ; as above, with with some memoize options
+           :trace t
+           :forget t)
+  (memsum) ; report statistics on calls of memoized functions (e.g., fn)
+  (clear-memoize-statistics) ; clear memoization (and profiling) statistics
+  ~ev[]
+
+  Evaluation of ~c[(profile 'fn)] redefines ~c[fn] so that a count will be kept
+  of the calls of FN.  The information recorded may be displayed, for example,
+  by invoking ~c[(]~ilc[memoize-summary]~c[)] or (equivalently) ~c[(memsum)].
+  If you wish to gather fresh statistics for the evaluation of a top-level
+  form, ~pl[clear-memoize-statistics].
+
+  ~c[Profile] is just a macro that calls ~ilc[memoize] to do its work.
+  ~c[Profile] gives the two keyword parameters ~c[:CONDITION] and ~c[:INLINE]
+  of ~ilc[memoize] the value ~c[nil].  Other keyword parameters for
+  ~c[memoize], which must not include ~c[:CONDITION], ~c[:CONDITION-fn], or
+  ~c[:INLINE], are passed through.  To eliminate profiling, use
+  ~ilc[unmemoize]; for example, to eliminate profiling for function ~c[fn],
+  evaluate ~c[(unmemoize 'fn)].
+
+  You may find raw Lisp functions ~c[profile-all] and ~c[profile-acl2] to be
+  useful.  Please contact the ACL2 developers if you want versions of these
+  functions to be executable from inside the ACL2 read-eval-print loop.~/~/"
+
+  (declare (xargs :guard (and (keyword-value-listp r)
+                              (not (assoc-keyword :condition r))
+                              (not (assoc-keyword :condition-fn r))
+                              (not (assoc-keyword :inline r)))))
+  `(memoize ,fn :condition nil :inline nil ,@r))
+
 #-hons
 (defmacro memoize-on-raw (fn form)
   (declare (ignore fn))
@@ -922,15 +1078,19 @@
 
   `(return-last 'memoize-let-raw ,fn ,form))
 
+(defmacro memoizedp-world (fn wrld)
+  `(let ((fn ,fn) (wrld ,wrld))
+     (cond
+      ((not (global-val 'hons-enabled wrld))
+       (er hard 'memoizedp
+           "Memoizedp cannot be called in this ACL2 image, as it requires a ~
+            hons-aware ACL2.  See :DOC hons-and-memoization."))
+      (t
+       (cdr (assoc-eq fn (table-alist 'memoize-table wrld)))))))
+
 (defmacro memoizedp (fn)
   (declare (xargs :guard t))
-  `(cond
-    ((not (hons-enabledp state))
-     (er hard 'memoizedp
-         "Memoizedp cannot be called in this ACL2 image, as it requires a ~
-          hons-aware ACL2.  See :DOC hons-and-memoization."))
-    (t
-     (cdr (assoc-eq ,fn (table-alist 'memoize-table (w state)))))))
+  `(memoizedp-world ,fn (w state)))
 
 ;;; hons-shrink-alist
 
@@ -940,28 +1100,26 @@
 ; the first argument deleted.  Informal remark: the alist
 ; returned is a hons when the initial ANS is not an atom.
 
-#|| Comment about the last clause above.  Or really? 
-Counterexamples?
-
-mbu> stp
-? (honsp (hons-shrink-alist '((a . b) (a . b2)) (hons-acons 1 2 3)))
-NIL
-
-mbu> stp
-? (honsp (hons-shrink-alist '((a . b) (a . b2)) nil))
-NIL
-? 
-
-||#
+; Comment about the last clause above.  Or really? 
+; Counterexamples?
+; 
+; mbu> stp
+; ? (honsp (hons-shrink-alist '((a . b) (a . b2)) (hons-acons 1 2 3)))
+; NIL
+; 
+; mbu> stp
+; ? (honsp (hons-shrink-alist '((a . b) (a . b2)) nil))
+; NIL
+; ? 
 
 (defconst *untroublesome-characters*
 
   (coerce "~/:{}=+-_.,!?@#%^%<>|&$*`\\/\";()" 'list)
 
   ":Doc-Section Hons-and-Memoization
-  Some not so weird characters.~/
 
-  *UNTROUBLESOME-CHARACTERS* should not include single quote,
-  whitespace characters, control characters, or anything else weird
-  for a file name.  Such characters should only be used in file names
-  by experts at quoting.~/~/")
+  some not so weird characters~/
+
+  ~c[*UNTROUBLESOME-CHARACTERS*] should not include single quote, whitespace
+  characters, control characters, or anything else weird for a file name.  Such
+  characters should only be used in file names by experts at quoting.~/~/")
