@@ -1,4 +1,4 @@
-; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -20,9 +20,9 @@
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
-; Department of Computer Sciences
+; Department of Computer Science
 ; University of Texas at Austin
-; Austin, TX 78712-1188 U.S.A.
+; Austin, TX 78701 U.S.A.
 
 ; We thank David L. Rager for contributing an initial version of this file.
 
@@ -211,103 +211,6 @@
 
 ; End of Essay on Parallelism Strategy
 
-; Parallelism Constants
-
-(defun core-count-raw (&optional (ctx nil) default)
-
-; If ctx is supplied, then we cause an error using the given ctx.  Otherwise we
-; return a suitable default value (see below).
-
-  #+ccl (declare (ignore ctx default))
-  #+ccl (ccl:cpu-count)
-  #-ccl
-  (if ctx
-      (er hard ctx
-          "It is illegal to call cpu-core-count in this Common Lisp ~
-           implementation.")
-
-; If the host Lisp does not provide a means for obtaining the number of cores,
-; then we simply estimate on the high side.  A high estimate is desired in
-; order to make it unlikely that we have needlessly idle cores.  We thus
-; believe that 8 cores is a reasonable estimate for early 2007; but we may well
-; want to increase this number later.
-
-    (or default 8)))
-
-(progn
-
-(defconstant *core-count*
-  (core-count-raw)
-  "The total number of CPU cores in the system.")
-
-(defconstant *unassigned-and-active-work-count-limit*
-
-; The *unassigned-and-active-work-count-limit* limits work on the *work-queue*
-; to what we think the system will be able to process in a reasonable amount of
-; time.  Suppose we have 8 CPU cores.  This means that there can be 8 active
-; work consumers, and that generally not many more than 24 pieces of
-; paralellism work are stored in the *work-queue* to be processed.  This
-; provides us the guarantee that if all worker threads were immediately to
-; finish their piece of parallelism work, that each of them would immediately
-; be able to grab another piece from the work queue.
-
-; We could increase the following coefficient from 4 and further guarantee that
-; consumers have parallelism work to process, but this would come at the
-; expense of backlogging the *work-queue".  We prefer simply to avoid the
-; otherwise parallelized computations in favor of their serial equivalents.
-
-  (* 4 *core-count*))
-
-(defconstant *total-work-limit* ; unassigned, started, resumed AND pending
-
-; The number of pieces of work in the system, *parallelism-work-count*, must be
-; less than *total-work-limit* in order to enable creation of new pieces of
-; work.  (However, we could go from 49 to 69 pieces of work when encountering a
-; pand; just not from 50 to 52.)
-
-; Why limit the amount of work in the system?  :Doc parallelism-how-to
-; (subtopic "Another Granularity Issue Related to Thread Limitations") provides
-; an example showing how cdr recursion can rapidly create threads.  That
-; example shows that if there is no limit on the amount of work we may create,
-; then eventually, many successive cdrs starting at the top will correspond to
-; waiting threads.  If we do not limit the amount of work that can be created,
-; this can exhaust the supply of Lisp threads available to process the elements
-; of the list.
-
-  (let ((val
-
-; Warning: It is possible, in principle to create (+ val
-; *max-idle-thread-count*) threads.  Presumably you'll get a hard Lisp error
-; (or seg fault!) if your Lisp cannot create that many threads.
-
-         50)
-        (bound (* 2 *core-count*)))
-    (when (< val bound)
-      (error "The variable *total-work-limit* needs to be at least ~s, i.e., ~%~
-              at least double the *core-count*.  Please redefine ~%~
-              *total-work-limit* so that it is not ~s."
-             bound
-             val))
-    val))
-
-; We don't want to spawn more worker threads (which are initially idle) when we
-; already have sufficiently many idle worker threads.  We use
-; *max-idle-thread-count* to limit this spawning in function
-; spawn-worker-threads-if-needed.
-
-(defconstant *max-idle-thread-count* (* 2 *core-count*))
-
-; *intial-threads* stores a list of threads that are considered to be part of
-; the non-threaded part of ACL2.  When terminating parallelism threads, only
-; those not appearing in this list will be terminated.  Warning: If ACL2 uses
-; parallelism during the build process, this variable could incorrectly record
-; parallelism threads as initial threads.
-
-(defvar *initial-threads* (all-threads))
-
-) ; end of constant list
-
-
 ; Parallelism Structures
 
 ; If the shape of parallelism-piece changes, update the *work-queue*
@@ -434,18 +337,6 @@
   (defvar *check-work-and-core-availability*)
   (defvar *check-core-availability-for-resuming*)
 
-; When we terminate threads due to a break and abort, we need a way to
-; terminate all threads.  We implement this by having them throw the
-; :worker-thread-no-longer-needed tag.  Unfortunately, sometimes the threads
-; are outside the scope of the associated catch, when throwing the tag would
-; cause a warning.  We avoid this warning by maintaining the dynamically-bound
-; variable *throwable-worker-thread*.  When the throwable context is entered,
-; we let a new copy of the variable into existence and set it to T.  Now, when
-; we throw :worker-thread-no-longer-needed, we only throw it if
-; *throwable-worker-thread* is non-nil.
-
-  (defvar *throwable-worker-thread* nil)
-
 ; *total-parallelism-piece-historical-count* tracks the total number of pieces
 ; of parallelism work processed over the lifetime of the ACL2 session.  It is
 ; reset whenever the parallelism variables are reset.  It is only used for
@@ -460,61 +351,7 @@
 ; Following are definitions of functions that help us restore the
 ; parallelism system to a stable state after an interrupt occurs.
 
-(defun throw-all-threads-in-list (thread-list)
-
-; We interrupt each of the given threads with a throw to the catch at the top
-; of consume-work-on-work-queue-when-there, which is the function called
-; by run-thread in spawn-worker-threads-if-needed.
-
-; Compare with kill-all-threads-in-list, which kills all of the given threads
-; (typically all user-produced threads), not just those self-identified as
-; being within the associated catch block.
-
-  (if (endp thread-list)
-      nil
-    (progn
-      (interrupt-thread
-       (car thread-list)
-       #'(lambda () (when *throwable-worker-thread*
-                      (throw :worker-thread-no-longer-needed nil))))
-      (throw-all-threads-in-list (cdr thread-list)))))
-
-(defun kill-all-threads-in-list (thread-list)
-
-; Compare with throw-all-threads-in-list, which uses throw instead of killing
-; threads directly, but only affects threads self identified as being within an
-; associated catch block.
-
-  (if (endp thread-list)
-      nil
-    (progn
-      (kill-thread (car thread-list))
-      (kill-all-threads-in-list (cdr thread-list)))))
-
-(defun all-threads-except-initial-threads-are-dead ()
-  #+sbcl
-  (<= (length (all-threads)) 1)
-  #-sbcl
-  (null (set-difference (all-threads) *initial-threads*)))
-
-
-(defun send-die-to-all-except-initial-threads ()
-
-; This function is evaluated only for side effect.
-
-  (let ((target-threads (set-difference (all-threads)
-                                        *initial-threads*)))
-    (throw-all-threads-in-list target-threads))
-  (thread-wait 'all-threads-except-initial-threads-are-dead))
-
-(defun kill-all-except-initial-threads ()
-
-; This function is evaluated only for side effect.
-
-  (let ((target-threads (set-difference (all-threads)
-                                        *initial-threads*)))
-    (kill-all-threads-in-list target-threads))
-  (thread-wait 'all-threads-except-initial-threads-are-dead))
+(defparameter *reset-parallelism-variables* nil)
 
 (defun reset-parallelism-variables ()
 
@@ -526,9 +363,10 @@
 
 ; If a user kills threads directly from raw Lisp, for example using functions
 ; above, then they should call reset-parallelism-variables.  Note that
-; reset-parallelism-variables is called automatically on any call of LD,
-; including the case that an ACL2 user interrupts with control-c and then
-; aborts to get back to the ACL2 top-level loop.
+; reset-parallelism-variables is called automatically on any top-level call of
+; LD (i.e., a call with *ld-level* = 0), as well as any time we return to the
+; prompt after entering a raw Lisp break (using
+; *reset-parallelism-variables*).
 
 ; (a) Kill all worker threads.
 
@@ -557,13 +395,22 @@
   (setf *total-parallelism-piece-historical-count* 0)
 
   (setf *initial-threads* (all-threads))
+
+  (setf *core-count* (core-count-raw))
+
+  (setf *reset-parallelism-variables* nil)
+
+  t ; return t
 )
 
-; We reset parallelism variables as a standard part of compilation to make
-; sure the code behind declaring variables and resetting variables is
-; consistent and that we are in a stable state.
+; Parallelism wart: consider deleting the following comment and commented 
+; code.
 
-(reset-parallelism-variables)
+; Once upon a time, we reset parallelism variables as a standard part of 
+; compilation to ensure that the code behind declaring variables and resetting
+; variables is consistent and that we are in a stable state.
+
+; (reset-parallelism-variables)
 
 ;---------------------------------------------------------------------
 ; Section:  Work Consumer Code
@@ -1242,7 +1089,6 @@
             ,gran-form)
     '(parallelism-resources-available)))
 
-#+acl2-par
 (defmacro pargs (&rest forms)
 
 ; This is the raw lisp version for threaded Lisps.
@@ -1301,7 +1147,6 @@
           (make-list-until-non-declare (cdr x) nil)
           (mv (car x) declare-forms body)))
 
-#+acl2-par
 (defmacro plet (&rest forms)
 
 ; This is the raw Lisp version for threaded Lisps.
@@ -1333,7 +1178,6 @@
     (and (car x)
          (and-list (cdr x)))))
 
-#+acl2-par
 (defmacro pand (&rest forms)
 
 ; This is the raw Lisp version for threaded Lisps.
@@ -1363,7 +1207,6 @@
         t
         (or-list (cdr x)))))
 
-#+acl2-par
 (defmacro por (&rest forms)
 
 ; This is the raw Lisp version for threaded Lisps.
@@ -1385,3 +1228,118 @@
                          remainder-forms)
                    or-early-termination-function)
              (list 'if (cons 'or remainder-forms) t nil))))))
+
+(defun signal-semaphores (sems)
+  (if (atom sems)
+      nil
+    (progn (signal-semaphore (car sems))
+           (signal-semaphores (cdr sems)))))
+
+(defmacro spec-mv-let (bindings computation body)
+  (assert
+   (and (true-listp body) 
+        (equal (length body) 4) 
+
+; We could easily also allow "mv?-let," but we don't for now, because we don't
+; have a need.  With work, we could also allow "let." We don't do so for now,
+; because we don't have a need, and we also don't want to a non-trivial amount
+; of work.
+
+        (or (equal (car body) 'mv-let@par)
+            (equal (car body) 'mv-let))))
+   
+  (let* ((inner-let (car body))
+         (inner-bindings (cadr body))
+         (inner-body (caddr body))
+         (ite (cadddr body)))
+    (assert (and (true-listp ite) 
+                 (equal (length ite) 4) 
+                 (equal (car ite) 'if)))
+    (let* ((test (cadr ite))
+           (true-branch (caddr ite))
+           (false-branch (cadddr ite)))
+
+; Keep variable name "the-very-obscure-feature" in sync with
+; check-vars-not-free in logical definition.
+
+      `(let ((the-very-obscure-feature (future ,computation)))
+         (,inner-let 
+          ,inner-bindings
+          ,inner-body
+          (if ,test
+              (mv?-let ,bindings
+                       (future-read the-very-obscure-feature)
+                       ,true-branch)
+            (progn (future-abort the-very-obscure-feature)
+                   ,false-branch)))))))
+
+; Parallelism wart: should we make a function called "set-gc-threshold" and
+; document it in a performance section?
+
+; Parallelism wart: should there be a documentation section called performance,
+; with links to sections "Parallelism", "Hons", and "Set-gc-threshold"?
+
+#+ccl
+(ccl:egc nil)
+
+#+ccl
+(ccl:set-lisp-heap-gc-threshold (expt 2 30))
+
+#+sbcl
+(setf (sb-ext:bytes-consed-between-gcs) (1- (expt 2 30)))
+
+(defvar *resource-and-timing-based-parallelizations*
+  0
+  "Tracks the number of times that we parallelize execution when
+  waterfall-parallelism is set to :resource-and-timing-based")
+
+(defvar *resource-and-timing-based-serializations*
+  0
+  "Tracks the number of times that we do not parallize execution when
+  waterfall-parallelism is set to :resource-and-timing-based")
+
+(defvar *resource-based-parallelizations*
+  0
+  "Tracks the number of times that we parallelize execution when
+  waterfall-parallelism is set to :resource-based")
+
+(defvar *resource-based-serializations*
+  0
+  "Tracks the number of times that we do not parallize execution when
+  waterfall-parallelism is set to :resource-based")
+
+(defun print-interesting-parallelism-variables()
+  (format t "Printing vars related to executing in parallel.~%")
+  (format t "*idle-future-core-count* is ~s~%"
+          (atomically-modifiable-counter-read *idle-future-core-count*))
+  (format t "*idle-future-resumptive-core-count* is ~s~%"
+          (atomically-modifiable-counter-read
+           *idle-future-resumptive-core-count*))
+  (format t "*idle-future-thread-count* is ~s~%"
+          (atomically-modifiable-counter-read *idle-future-thread-count*))
+  (format t "*unassigned-and-active-future-count* is ~s~%"
+          (atomically-modifiable-counter-read
+           *unassigned-and-active-future-count*))
+  (format t "*total-future-count* is ~s~%"
+          (atomically-modifiable-counter-read *total-future-count*))  
+  (format t "*futures-resources-available-count* is ~s~%"
+          *futures-resources-available-count*)
+  (format t "*futures-resources-unavailable-count* is ~s~%"
+          *futures-resources-unavailable-count*)
+
+  (format t "~%Printing vars related to aborting futures.~%")
+  (format t "*aborted-futures-total* is ~s~%" *aborted-futures-total*)
+  (format t "*aborted-futures-via-throw* is ~s~%" *aborted-futures-via-throw*)
+  (format t "*aborted-futures-via-flag* is ~s~%" *aborted-futures-via-flag*)
+  (format t "*almost-aborted-future-count* is ~s~%~%"
+          *almost-aborted-future-count*)
+
+  (format t "*resource-based-parallelizations* is ~s~%"
+          *resource-based-parallelizations*)
+  (format t "*resource-based-serializations* is ~s~%"
+          *resource-based-serializations*)
+
+  (format t "*resource-and-timing-based-parallelizations* is ~s~%"
+          *resource-and-timing-based-parallelizations*)
+  (format t "*resource-and-timing-based-serializations* is ~s~%"
+          *resource-and-timing-based-serializations*))
