@@ -1,4 +1,4 @@
-; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -20,9 +20,9 @@
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
-; Department of Computer Sciences
+; Department of Computer Science
 ; University of Texas at Austin
-; Austin, TX 78712-1188 U.S.A.
+; Austin, TX 78701 U.S.A.
 
 (in-package "ACL2")
 
@@ -74,20 +74,18 @@
    (string-append (symbol-name var) "-FN")
    var))
 
-(defmacro define-global (var val)
+(defmacro define-global (var)
   (let ((var-fn (define-global-name var)))
     `(progn (defun ,var-fn (state)
-              (if (f-boundp-global ',var state)
-                  (f-get-global ',var state)
-                ,val))
+              (f-get-global ',var state))
             (defmacro ,var ()
               '(,var-fn state)))))
 
-(define-global pc-prompt "->: ")
-(define-global pc-prompt-depth-prefix "#")
-(define-global print-macroexpansion-flg nil)
+(define-global pc-prompt)
+(define-global pc-prompt-depth-prefix)
+(define-global pc-print-macroexpansion-flg)
 ; Turn the following on for debugging macro commands.
-(define-global print-prompt-and-instr-flg t)
+(define-global pc-print-prompt-and-instr-flg)
 
 ; We will maintain an invariant that there are no unproved goals hanging around
 ; in the pc-state.  Moreover, for simplicity, we leave it up to each command to
@@ -684,8 +682,9 @@
                        state)))))))))
 
 (defun accumulate-ttree-in-pc-state (pc-state state)
-  (er-let* ((ttree (accumulate-ttree-into-state
+  (er-let* ((ttree (accumulate-ttree-and-step-limit-into-state
                     (access pc-state pc-state :tag-tree)
+                    :skip
                     state)))
            (value (change-pc-state pc-state :tag-tree ttree))))
 
@@ -694,17 +693,17 @@
 ; Like process-assumptions, but returns (mv clauses known-assumptions ttree
 ; state).
 
-  (let ((n (quickly-count-assumptions ttree 0 101)))
+  (let ((n (count-assumptions ttree)))
     (pprogn
      (cond
       ((< n 101)
        state)
       (t
        (io? prove nil state
-            nil
-            (fms "~%Note: processing over 100 forced hypotheses which we now ~
+            (n)
+            (fms "~%Note: processing ~x0 forced hypotheses which we now ~
                   collect)~%"
-                 nil
+                 (list (cons #\0 n))
                  (proofs-co state) state nil))))
      (mv-let
       (n0 assns pairs ttree1)
@@ -782,7 +781,8 @@
           :oncep-override (match-free-override wrld)
           :force-info t
           :nonlinearp (non-linearp wrld)
-          :backchain-limit-rw (backchain-limit wrld :rewrite)))
+          :backchain-limit-rw (backchain-limit wrld :rewrite)
+          :rw-cache-state (rw-cache-state wrld)))
 
 (defun make-new-goals-fixed-hyps (termlist hyps goal-name start-index)
   ;; similar to make-new-goals
@@ -817,175 +817,180 @@
 
 ; Vals is (x replaced-state), where x is a pc-state or nil.
 
-          (assert$
-           (equal (car stobjs-out/vals) '(nil state))
-           (cond
-            (erp
-             (pprogn (print-no-change
+          (cond
+           (erp
+            (pprogn (print-no-change
 
 ; We used to say "Very odd" here, but it is perfectly natural to get such an
 ; error if there is an rdepth-error.
 
-                      "An error occurred in executing ~x0."
-                      (list (cons #\0 instr)))
-                     (mv 'pc-single-step-error-primitive nil state)))
-            ((car vals) ;so, there is a new state
-             (let ((pc-ens (make-pc-ens (pc-ens) state)))
-               (mv-let
-                (bad-ass ttree)
-                (resume-suspended-assumption-rewriting
-                 (access pc-state (car vals) :local-tag-tree)
-                 nil  ;ancestors
-                 nil  ;gstack
-                 nil  ;simplify-clause-pot-lst
-                 (initial-rcnst-from-ens pc-ens wrld)
-                 wrld
-                 state)
-                (cond
-                 (bad-ass
-                  (pprogn
-                   (let ((assumnote
+                     "An error occurred in executing ~X01."
+                     (list (cons #\0 instr)
+                           (cons #\1 (abbrev-evisc-tuple state))))
+                    (mv 'pc-single-step-error-primitive nil state)))
+           (t
+            (assert$
+             (equal (car stobjs-out/vals) '(nil state))
+             (cond
+              ((car vals) ;so, there is a new state
+               (let ((pc-ens (make-pc-ens (pc-ens) state)))
+                 (mv-let
+                  (step-limit bad-ass ttree)
+                  (resume-suspended-assumption-rewriting
+                   (access pc-state (car vals) :local-tag-tree)
+                   nil ;ancestors
+                   nil ;gstack
+                   nil ;simplify-clause-pot-lst
+                   (initial-rcnst-from-ens pc-ens wrld)
+                   wrld
+                   state
+                   (initial-step-limit wrld state))
+                  (declare (ignore step-limit))
+                  (cond
+                   (bad-ass
+                    (pprogn
+                     (let ((assumnote
 
 ; Is the assumnotes field always non-empty?
 
-                          (car (access assumption bad-ass :assumnotes))))
-                     (print-no-change
-                      "A false assumption was encountered from applying the rune ~
-                   ~x0 to the target ~x1."
-                      (list (cons #\0 (access assumnote assumnote :rune))
-                            (cons #\1 (access assumnote assumnote :target)))))
-                   (mv nil nil state)))
-                 (t
-                  (let* ((remaining-goals (unproved-goals (car vals)))
+                            (car (access assumption bad-ass :assumnotes))))
+                       (print-no-change
+                        "A false assumption was encountered from applying the ~
+                         rune ~x0 to the target ~x1."
+                        (list (cons #\0 (access assumnote assumnote :rune))
+                              (cons #\1 (access assumnote assumnote :target)))))
+                     (mv nil nil state)))
+                   (t
+                    (let* ((remaining-goals (unproved-goals (car vals)))
 
 ; Just what is the invariant on primitive instructions?  Can they leave a goal
 ; with conclusion *t* other than the top one being operated upon?
 
-                         (goal-name (goal-name))
-                         (goal-unproved-p
-                          (and remaining-goals
-                               (equal goal-name
-                                      (access goal (car remaining-goals) :goal-name))))
-                         (hyps (hyps))
-                         (depends-on (depends-on)))
-                    (mv-let
-                     (cl-set assns ttree state)
-                     (pc-process-assumptions pc-ens ttree wrld state)
-                     (mv-let
-                      (contradictionp hyps-type-alist ttree0)
-                      (if (and assns goal-unproved-p)
-                          (type-alist-clause (dumb-negate-lit-lst hyps)
-                                             nil nil nil pc-ens wrld
-                                             nil nil)
-                        ;; else don't bother here
-                        (mv nil nil nil))
-                      (cond
-                       (contradictionp
-                        (er-let*
-                         ((new-pc-state
-                           (let ((local-ttree (cons-tag-trees ttree ttree0)))
-                             (accumulate-ttree-in-pc-state
-                              (change-pc-state
-                               (car vals)
-                               :goals
-                               (cdr goals)
-                               :tag-tree
+                           (goal-name (goal-name))
+                           (goal-unproved-p
+                            (and remaining-goals
+                                 (equal goal-name
+                                        (access goal (car remaining-goals) :goal-name))))
+                           (hyps (hyps))
+                           (depends-on (depends-on)))
+                      (mv-let
+                       (cl-set assns ttree state)
+                       (pc-process-assumptions pc-ens ttree wrld state)
+                       (mv-let
+                        (contradictionp hyps-type-alist ttree0)
+                        (if (and assns goal-unproved-p)
+                            (type-alist-clause (dumb-negate-lit-lst hyps)
+                                               nil nil nil pc-ens wrld
+                                               nil nil)
+                          ;; else don't bother here
+                          (mv nil nil nil))
+                        (cond
+                         (contradictionp
+                          (er-let*
+                              ((new-pc-state
+                                (let ((local-ttree (cons-tag-trees ttree ttree0)))
+                                  (accumulate-ttree-in-pc-state
+                                   (change-pc-state
+                                    (car vals)
+                                    :goals
+                                    (cdr goals)
+                                    :tag-tree
 
 ; We are concerned about the comment in note-2-8-bug-fixes about the
 ; proof-checker running slowly because of tag-trees.  So we play it safe here
 ; by avoiding duplication.
 
-                               (scons-tag-trees local-ttree old-tag-tree)
-                               :local-tag-tree
-                               local-ttree)
-                              state))))
-                         (pprogn (io? proof-checker nil state
-                                      (instr goal-name)
-                                      (fms0 "~|AHA!  A contradiction has been ~
+                                    (cons-tag-trees local-ttree old-tag-tree)
+                                    :local-tag-tree
+                                    local-ttree)
+                                   state))))
+                            (pprogn (io? proof-checker nil state
+                                         (instr goal-name)
+                                         (fms0 "~|AHA!  A contradiction has been ~
                                              discovered in the hypotheses of ~
                                              goal ~x0 in the course of ~
                                              executing instruction ~x1, in ~
                                              the process of preparing to deal ~
                                              with forced assumptions.~|"
-                                            (list (cons #\0 goal-name)
-                                                  (cons #\0 instr))
-                                            0 nil))
-                                 (io? proof-checker nil state
-                                      (goals)
-                                      (maybe-print-proved-goal-message
-                                       (car goals) goals (cdr goals) state))
-                                 (pc-assign
-                                  state-stack
-                                  (cons
-                                   new-pc-state
-                                   (state-stack)))
-                                 (value new-pc-state))))
-                       (t
-                        (let* ((termlist
-                                (cl-set-to-implications cl-set))
-                               (forced-goals
-                                (make-new-goals-fixed-hyps
-                                 termlist hyps goal-name depends-on))
-                               (new-goals
-                                (add-assumptions-to-top-goal
-                                 goal-unproved-p
-                                 (known-assumptions hyps-type-alist assns)
-                                 forced-goals
-                                 remaining-goals))
-                               (pc-state-1
-                                (change-pc-state (car vals)
-                                                 :goals new-goals
-                                                 :tag-tree
-                                                 (scons-tag-trees
-                                                  ttree old-tag-tree)
-                                                 :local-tag-tree ttree)))
-                          (er-let* ((new-pc-state
-                                     (accumulate-ttree-in-pc-state
-                                      pc-state-1
-                                      state)))
-                                   (pprogn
-                                    (cond
-                                     (forced-goals
-                                      (io? proof-checker nil state
-                                           (forced-goals)
-                                           (fms0
-                                            "~|NOTE (forcing):  Creating ~n0 ~
-                                             new ~#1~[~/goal~/goals~] due to ~
-                                             forcing assumptions.~%"
-                                            (list
-                                             (cons #\0 (length forced-goals))
-                                             (cons #\1
-                                                   (zero-one-or-more
-                                                    (length forced-goals)))))))
-                                     (t state))
+                                               (list (cons #\0 goal-name)
+                                                     (cons #\0 instr))
+                                               0 nil))
                                     (io? proof-checker nil state
-                                         (new-goals goals)
+                                         (goals)
                                          (maybe-print-proved-goal-message
-                                          (car goals) goals new-goals state))
+                                          (car goals) goals (cdr goals) state))
                                     (pc-assign
                                      state-stack
                                      (cons
                                       new-pc-state
                                       (state-stack)))
-                                    (value new-pc-state))))))))))))))
-            (t
-             (mv nil nil state))))))))))
+                                    (value new-pc-state))))
+                         (t
+                          (let* ((termlist
+                                  (cl-set-to-implications cl-set))
+                                 (forced-goals
+                                  (make-new-goals-fixed-hyps
+                                   termlist hyps goal-name depends-on))
+                                 (new-goals
+                                  (add-assumptions-to-top-goal
+                                   goal-unproved-p
+                                   (known-assumptions hyps-type-alist assns)
+                                   forced-goals
+                                   remaining-goals))
+                                 (pc-state-1
+                                  (change-pc-state (car vals)
+                                                   :goals new-goals
+                                                   :tag-tree
+                                                   (cons-tag-trees
+                                                    ttree old-tag-tree)
+                                                   :local-tag-tree ttree)))
+                            (er-let* ((new-pc-state
+                                       (accumulate-ttree-in-pc-state
+                                        pc-state-1
+                                        state)))
+                              (pprogn
+                               (cond
+                                (forced-goals
+                                 (io? proof-checker nil state
+                                      (forced-goals)
+                                      (fms0
+                                       "~|NOTE (forcing):  Creating ~n0 ~
+                                             new ~#1~[~/goal~/goals~] due to ~
+                                             forcing assumptions.~%"
+                                       (list
+                                        (cons #\0 (length forced-goals))
+                                        (cons #\1
+                                              (zero-one-or-more
+                                               (length forced-goals)))))))
+                                (t state))
+                               (io? proof-checker nil state
+                                    (new-goals goals)
+                                    (maybe-print-proved-goal-message
+                                     (car goals) goals new-goals state))
+                               (pc-assign
+                                state-stack
+                                (cons
+                                 new-pc-state
+                                 (state-stack)))
+                               (value new-pc-state))))))))))))))
+              (t
+               (mv nil nil state))))))))))))
 
 (defun maybe-print-macroexpansion (instr raw-instr state)
-  (let ((print-macroexpansion-flg (print-macroexpansion-flg)))
-    (if (and print-macroexpansion-flg
+  (let ((pc-print-macroexpansion-flg (pc-print-macroexpansion-flg)))
+    (if (and pc-print-macroexpansion-flg
              (not (eq (car instr) (make-official-pc-command 'lisp t)))
              (not (equal instr (make-official-pc-instr raw-instr state))))
         (io? proof-checker nil state
-             (print-macroexpansion-flg instr)
+             (pc-print-macroexpansion-flg instr)
              (fms0 ">> ~x0~|" (list (cons #\0 instr)) 0
-                   (if (and (consp print-macroexpansion-flg)
-                            (integerp (car print-macroexpansion-flg))
-                            (integerp (cdr print-macroexpansion-flg))
-                            (> (car print-macroexpansion-flg) 0)
-                            (> (cdr print-macroexpansion-flg) 0))
-                       (evisc-tuple (car print-macroexpansion-flg)
-                                    (cdr print-macroexpansion-flg)
+                   (if (and (consp pc-print-macroexpansion-flg)
+                            (integerp (car pc-print-macroexpansion-flg))
+                            (integerp (cdr pc-print-macroexpansion-flg))
+                            (> (car pc-print-macroexpansion-flg) 0)
+                            (> (cdr pc-print-macroexpansion-flg) 0))
+                       (evisc-tuple (car pc-print-macroexpansion-flg)
+                                    (cdr pc-print-macroexpansion-flg)
                                     nil nil)
                      nil)))
       state)))
@@ -1071,7 +1076,7 @@
       acc
     (union-lastn-pc-tag-trees (1- n)
                               (cdr ss)
-                              (scons-tag-trees
+                              (cons-tag-trees
                                (access pc-state (car ss) :local-tag-tree)
                                acc))))
 
@@ -1109,7 +1114,7 @@
 (defconst *pc-complete-signal* 'acl2-pc-complete)
 
 (defun pc-main-loop (instr-list quit-conditions last-value
-                                print-prompt-and-instr-flg state)
+                                pc-print-prompt-and-instr-flg state)
   ;; Returns an error triple whose state has
   ;; the new state-stack "installed".  Here instr-list is a (true) list of
   ;; instructions or else is a non-NIL atom, probably *standard-oi*,
@@ -1129,12 +1134,12 @@
   (if (null instr-list)
       (mv nil last-value state)
     (mv-let (col state)
-            (if print-prompt-and-instr-flg
+            (if pc-print-prompt-and-instr-flg
                 (print-pc-prompt state)
               (mv 0 state))
             (mv-let (erp instr state)
                     (if (consp instr-list)
-                        (pprogn (if print-prompt-and-instr-flg
+                        (pprogn (if pc-print-prompt-and-instr-flg
                                     (io? proof-checker nil state
                                          (col instr-list)
                                          (fms0 "~y0~|"
@@ -1166,7 +1171,7 @@
                                  ;; If that decision is changed, then change documentation in CATCH
                                  ;; meta command.
                                  (and last-value (null signal) val)
-                                 print-prompt-and-instr-flg
+                                 pc-print-prompt-and-instr-flg
                                  state))))))))
 
 (defun make-initial-goal (term)
@@ -1200,11 +1205,18 @@
                           nil))
     (pc-assign old-ss nil)))
 
+(defun pc-main1 (instr-list quit-conditions pc-print-prompt-and-instr-flg
+                            state)
+  (with-prover-step-limit!
+   :start
+   (pc-main-loop instr-list quit-conditions t pc-print-prompt-and-instr-flg
+                 state)))
+
 (defun pc-main (term raw-term event-name rule-classes instr-list
-                     quit-conditions print-prompt-and-instr-flg state)
+                     quit-conditions pc-print-prompt-and-instr-flg state)
   (pprogn (install-initial-state-stack term raw-term event-name rule-classes)
-          (pc-main-loop instr-list quit-conditions t print-prompt-and-instr-flg
-                        state)))
+          (pc-main1 instr-list quit-conditions pc-print-prompt-and-instr-flg
+                    state)))
 
 (defun pc-top (raw-term event-name rule-classes instr-list quit-conditions state)
   ;; Here instr-list can have a non-nil last cdr, meaning "proceed
@@ -1247,51 +1259,57 @@
                (illegal-fnp-list (cdr x) w)))))
 )
 
-(defun verify-fn (raw-term raw-term-supplied-p event-name rule-classes instructions state)
-  (if (f-get-global 'in-verify-flg state)
-      (er soft 'verify
-          "You are apparently already inside the VERIFY interactive loop.  It is ~
-           illegal to enter such a loop recursively.")
+(defun verify-fn (raw-term raw-term-supplied-p event-name rule-classes
+                           instructions state)
+  (cond
+   ((f-get-global 'in-verify-flg state)
+    (er soft 'verify
+        "You are apparently already inside the VERIFY interactive loop.  It ~
+         is illegal to enter such a loop recursively."))
+   (t
     (mv-let
-      (erp val state)
-      (if raw-term-supplied-p
-          (state-global-let*
-           ((in-verify-flg t)
-            (print-base 10)
-            (inhibit-output-lst
-             (remove1-eq 'proof-checker
-                         (f-get-global 'inhibit-output-lst state))))
-           (pc-top raw-term event-name rule-classes
-                   (append instructions *standard-oi*)
-                   (list 'exit)
-                   state))
-        (if (null (state-stack))
-            (er soft 'verify "There is no interactive verification to re-enter!")
-          (let ((bad-fn
-                 (illegal-fnp
-                  (access goal
-                          (car (access pc-state (car (last (state-stack)))
-                                       :goals))
-                          :conc)
-                  (w state))))
-            (if bad-fn
-                (er soft 'verify
-                    "The current proof-checker session was begun in an ACL2 ~
-                     world with function symbol ~x0, but that function symbol ~
-                     no longer exists."
-                    bad-fn)
-              (state-global-let*
-               ((in-verify-flg t)
-                (print-base 10)
-                (inhibit-output-lst
-                 (remove1-eq 'proof-checker
-                             (f-get-global 'inhibit-output-lst state))))
-               (pc-main-loop (append instructions *standard-oi*)
-                             (list 'exit)
-                             t t state))))))
-      (if (equal erp *pc-complete-signal*)
-          (value val)
-        (mv erp val state)))))
+     (erp val state)
+     (cond
+      (raw-term-supplied-p
+       (state-global-let*
+        ((in-verify-flg t)
+         (print-base 10)
+         (inhibit-output-lst
+          (remove1-eq 'proof-checker
+                      (f-get-global 'inhibit-output-lst state))))
+        (pc-top raw-term event-name rule-classes
+                (append instructions *standard-oi*)
+                (list 'exit)
+                state)))
+      ((null (state-stack))
+       (er soft 'verify "There is no interactive verification to re-enter!"))
+      (t
+       (let ((bad-fn
+              (illegal-fnp
+               (access goal
+                       (car (access pc-state (car (last (state-stack)))
+                                    :goals))
+                       :conc)
+               (w state))))
+         (cond
+          (bad-fn
+           (er soft 'verify
+               "The current proof-checker session was begun in an ACL2 world ~
+                with function symbol ~x0, but that function symbol no longer ~
+                exists."
+               bad-fn))
+          (t
+           (state-global-let*
+            ((in-verify-flg t)
+             (print-base 10)
+             (inhibit-output-lst
+              (remove1-eq 'proof-checker
+                          (f-get-global 'inhibit-output-lst state))))
+            (pc-main1 (append instructions *standard-oi*)
+                      (list 'exit) t state)))))))
+     (cond ((equal erp *pc-complete-signal*)
+            (value val))
+           (t (mv erp val state)))))))
 
 (defun print-unproved-goals-message (goals state)
   (io? proof-checker nil state
